#!/usr/bin/env python
#
# Writes DXF files: 2D and 3D
#

from VectorAlgebra import *

# For spheres
DisplaySubdiv = 16

def ArcPoints(v1,v2):
	tanum = VectorNorm(CrossProduct(v1,v2))
	taden = InnerProduct(v1,v2)
	a = math.atan2(tanum,taden)
	na = int(math.ceil(DisplaySubdiv*a/(math.pi/2)))
	amlt = 1/math.sin(a)
	vts = []
	for k in xrange(na+1):
		frac = float(k)/na
		mlt1 = amlt*math.sin(a*(1-frac))
		mlt2 = amlt*math.sin(a*frac)
		mv1 = VectorScalMult(mlt1,v1)
		mv2 = VectorScalMult(mlt2,v2)
		mv = VectorAdd(mv1,mv2)
		vts.append(mv)
	return vts

def writestr(f,x): f.write("%s\n" % str(x))
def writeint(f,x): writestr(f, "%d" % int(x))
def writefloat(f,x): writestr(f, "%f" % float(x))

def writebegin(f):
	writeint(f,0)
	writestr(f,"SECTION")
	writeint(f,2)
	writestr(f,"HEADER")
	writeint(f,9)
	writestr(f,"$ACADVER")
	writeint(f,1)
	writestr(f,"AC1009")
	writeint(f,0)
	writestr(f,"ENDSEC")
	writeint(f,999)
	writestr(f,"Delaunay triangulation, convex hull, Voronoi diagram 3D model")
	writeint(f,999)
	writestr(f,"Written by SphericalDelaunayVoronoi.py")
	writeint(f,0)
	writestr(f,"SECTION")
	writeint(f,2)
	writestr(f,"ENTITIES")

def writeend(f):
	writeint(f,0)
	writestr(f,"ENDSEC")
	writeint(f,0)
	writestr(f,"EOF")

DefaultLayer = "DEFAULT"
DefaultColor = 0

def writepoint(f,v,lyr=DefaultLayer,clr=DefaultColor):
	writeint(f,0)
	writestr(f,"POINT")
	writeint(f,8)
	writestr(f,lyr)
	writeint(f,62)
	writeint(f,clr)
	for k in xrange(len(v)):
		writeint(f,10*(k+1))
		writefloat(f,v[k])

def writepoints(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	[writepoint(f,v,lyr,clr) for v in vts]

def writeline(f,v1,v2,lyr=DefaultLayer,clr=DefaultColor):
	writeint(f,0)
	writestr(f,"LINE")
	writeint(f,8)
	writestr(f,lyr)
	writeint(f,62)
	writeint(f,clr)
	for k in xrange(len(v1)):
		writeint(f,10*(k+1))
		writefloat(f,v1[k])
	for k in xrange(len(v2)):
		writeint(f,10*(k+1)+1)
		writefloat(f,v2[k])

def writelinevlst(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	writeline(f,vts[0],vts[1],lyr,clr)

def writecrosspoint(f,v,cl,lyr=DefaultLayer,clr=DefaultColor):
	for k in xrange(len(v)):
		v1 = list(v)
		v2 = list(v)
		v1[k] -= cl
		v2[k] += cl
		writeline(f,v1,v2,lyr,clr)
	
def writecrosspoints(f,vts,cl,lyr=DefaultLayer,clr=DefaultColor):
	[writecrosspoint(f,v,cl,lyr,clr) for v in vts]

def writepolyline(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	vtlen = len(vts)
	for k in xrange(vtlen-1):
		writeline(f,vts[k],vts[k+1],lyr,clr)

def writepolygon(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	writepolyline(f,list(vts)+[vts[0]],lyr,clr)

def writetriface(f,v1,v2,v3,lyr=DefaultLayer,clr=DefaultColor):
	writeint(f,0)
	writestr(f,"3DFACE")
	writeint(f,8)
	writestr(f,lyr)
	writeint(f,62)
	writeint(f,clr)
	for k in xrange(len(v1)):
		writeint(f,10*(k+1))
		writefloat(f,v1[k])
	for k in xrange(len(v2)):
		writeint(f,10*(k+1)+1)
		writefloat(f,v2[k])
	for k in xrange(len(v3)):
		writeint(f,10*(k+1)+2)
		writefloat(f,v3[k])
	for k in xrange(len(v3)):
		writeint(f,10*(k+1)+3)
		writefloat(f,v3[k])

def writetrifacevlst(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	writetriface(f,vts[0],vts[1],vts[2],lyr,clr)

# For a sphere's surface

def writearc(f,v1,v2,lyr=DefaultLayer,clr=DefaultColor):
	vts = ArcPoints(v1,v2)
	n = len(vts)
	for k in xrange(n-1):
		kn = k+1
		v1 = vts[k]
		v2 = vts[kn]
		writeline(f,v1,v2,lyr,clr)

def writearcvlst(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	writearc(f,vts[0],vts[1],lyr,clr)

def writepolyarc(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	vtlen = len(vts)
	for k in xrange(vtlen-1):
		writearcvlst(f,vts[k:k+2],lyr,clr)

def writepolygonarc(f,vts,lyr=DefaultLayer,clr=DefaultColor):
	if len(vts) > 0: writepolyarc(f,list(vts)+[vts[0]],lyr,clr)

# Cardinal points for a sphere
cardpoints = ((-1,0,0),(1,0,0),(0,-1,0),(0,1,0),(0,0,-1),(0,0,1))
