#!/usr/bin/env python
#
# Determinant Timing
#
# Args:
# Random-number seed (arbitrary string)
# Number of tests of each size
# Max size

import sys, math, random, time

from VectorAlgebra import *


ranseed = sys.argv[1]
numpts = int(sys.argv[2])
maxsize = int(sys.argv[3])


def TestPermutations(n):
	for perm, sig in Permutations(n):
		print perm, sig


DetAlgorithms = ( \
	("Explicit", DetExplicit), \
	("Leibniz", DetLeibniz), \
	("Laplace", DetLaplace), \
	("Gauss", DetGauss), \
	("LU Decomposition", DetLUD) \
	)

FastDetAlgorithms = ( \
	("Explicit", DetExplicit), \
	("Gauss", DetGauss), \
	("LU Decomposition", DetLUD) \
	)

def showdets(mat):
	print mat
	for name, alg in DetAlgorithms:
		print name, alg(mat)

smallmats = ( \
	((7,),), \
	((-3,-7),(-3,-5)), \
	((-3,-7,-3),(-5,-6,-3),(4,-1,5)), \
	((-3,-7,-3,-5),(-6,-3,4,-1),(5,-6,-1,-4),(7,-1,-4,7)), \
	((-3,-7,-3,-5,-6),(-3,4,-1,5,-6),(-1,-4,7,-1,-4),(7,6,3,6,5),(4,-4,-3,5,3)) \
	)

# Results: 7, -6, -79, -2826, -23214
print "Small matrices"
for mat in smallmats: showdets(mat); print

print "Cross product"
sm3 = smallmats[2][:2]
print CrossProduct(sm3[0],sm3[1])
print SuperCrossProduct(sm3)
print

print "Timing tests"
random.seed(ranseed)
for dim in xrange(1,maxsize+1):
	mats = []
	for n in xrange(numpts):
		mat = [[random.gauss(0,1) for d1 in xrange(dim)] for d2 in xrange(dim)]
		mats.append(mat)
	print dim
	dta = FastDetAlgorithms if dim > 4 else DetAlgorithms
	for name, alg in dta:
		t0 = time.clock()
		for mat in mats: alg(mat)
		t1 = time.clock()
		print name, t1-t0
	