#!/usr/bin/env python
#
# Lots of vector algebra

import math

def VectorAdd(v1,v2):
	return [x1+x2 for x1,x2 in zip(v1,v2)]

def VectorAddTo(v1,v2):
	for k in xrange(len(v1)):
		v1[k] += v2[k]
	return v1

def VectorMultiAdd(vlst):
	vsum = list(vlst[0])
	for v in vlst[1:]:
		VectorAddTo(vsum,v)
	return vsum

def VectorMean(vlst):
	vmean = VectorMultiAdd(vlst)
	VectorScalMultBy(1.0/len(vlst),vmean)
	return vmean

def VectorConst(n,c):
	return [c for k in xrange(n)]

def VectorScalAdd(s,v):
	return [x+s for x in v]

def VectorScalAddTo(s,v):
	for k in xrange(len(v)):
		v[k] += s
	return v

def VectorSub(v1,v2):
	return [x1-x2 for x1,x2 in zip(v1,v2)]

def VectorSubFrom(v1,v2):
	for k in xrange(len(v1)):
		v1[k] -= v2[k]
	return v1

def VectorScalMult(s,v):
	return [s*x for x in v]

def VectorScalMultBy(s,v):
	for k in xrange(len(v)):
		v[k] *= s

def InnerProduct(v1,v2):
	ipsum = 0
	for x1,x2 in zip(v1,v2):
		ipsum += x1*x2
	return ipsum

def ScalarCross(v1,v2):
	return v1[0]*v2[1]-v1[1]*v2[0]

def ScalarCrossList(vts):
	return ScalarCross(vts[0],vts[1])

def CrossProduct(v1,v2):
	return [v1[1]*v2[2]-v1[2]*v2[1], v1[2]*v2[0]-v1[0]*v2[2], v1[0]*v2[1]-v1[1]*v2[0]]

def TripleProduct(v1,v2,v3):
	return (v1[1]*v2[2]-v1[2]*v2[1])*v3[0] + \
		(v1[2]*v2[0]-v1[0]*v2[2])*v3[1] + \
		(v1[0]*v2[1]-v1[1]*v2[0])*v3[2]

def TripleProductList(vts):
	return TripleProduct(vts[0],vts[1],vts[2])

def VectorAbsSqr(v): return InnerProduct(v,v)

def VectorNorm(v): return math.sqrt(VectorAbsSqr(v))

def NormalizeVector(v):
	norm = VectorNorm(v)
	if norm > 0:
		return VectorScalMult(1.0/norm,v)
	else:
		return v

#

def IndexMin(xlst):
	ixmn = 0
	xmin = xlst[ixmn]
	for ix,x in enumerate(xlst):
		if x < xmin:
			xmin = x
			ixmn = ix
	return ixmn
	
def IndexMax(xlst):
	ixmx = 0
	xmax = xlst[ixmx]
	for ix,x in enumerate(xlst):
		if x > xmax:
			xmax = x
			ixmx = ix
	return ixmx

def IndexMinBySortFunc(xlst, sortfunc):
	ixmn = 0
	xmin = xlst[ixmn]
	for ix,x in enumerate(xlst):
		if sortfunc(xmin,x):
			xmin = x
			ixmn = ix
	return ixmn
	
def IndexMaxBySortFunc(xlst, sortfunc):
	ixmx = 0
	xmax = xlst[ixmx]
	for ix,x in enumerate(xlst):
		if sortfunc(x,xmax):
			xmax = x
			ixmx = ix
	return ixmx	

def IndexMinBySortByFunc(xlst, sortbyfunc):
	ixmn = 0
	xmin = sortbyfunc(xlst[ixmn])
	for ix,x in enumerate(xlst):
		xs = sortbyfunc(x)
		if xs < xmin:
			xmin = xs
			ixmn = ix
	return ixmn

def IndexMaxBySortByFunc(xlst, sortbyfunc):
	ixmx = 0
	xmax = sortbyfunc(xlst[ixmx])
	for ix,x in enumerate(xlst):
		xs = sortbyfunc(x)
		if xs > xmax:
			xmax = xs
			ixmx = ix
	return ixmx

def MinBySortFunc(xlst, sortfunc):
	return xlst[IndexMinBySortFunc(xlst, sortfunc)]

def MaxBySortFunc(xlst, sortfunc):
	return xlst[IndexMaxBySortFunc(xlst, sortfunc)]

def MinBySortByFunc(xlst, sortbyfunc):
	return xlst[IndexMinBySortByFunc(xlst, sortbyfunc)]

def MaxBySortByFunc(xlst, sortbyfunc):
	return xlst[IndexMaxBySortByFunc(xlst, sortbyfunc)]

def SelWithList(sllst, xlst):
	return [x for sl,x in zip(sllst,xlst) if sl]

# Makes (index,element)
def AddIndices(xlst): return list(enumerate(xlst))

def RotateToMin(xlst):
	if len(xlst) > 0:
		ixmin = IndexMin(xlst)
		return list(xlst[ixmin:]) + list(xlst[:ixmin])
	else:
		return []

def RotateLeft(xlst,n=1):
	xs = list(xlst)
	xlen = len(xlst)
	if xlen == 0: return xs
	nx = n % xlen
	if nx == 0: return xs
	else: return xs[nx:] + xs[:nx]

def RotateRight(xlst,n=1):
	return RotateLeft(xlst,-n)

#

# Edges and triangle-edge references
# TriEdgeRefs assumes that the input triangle is a tuple

def TriEdges(tri):
	return ((tri[0],tri[1]), (tri[1],tri[2]), (tri[2],tri[0]))

def TriEdgeRefs(tri, trieds, edtri):
	eds = TriEdges(tri)
	trieds[tri] = eds
	for ed in eds: edtri[ed] = tri

# 2D triangle area
# Factor of 1/2 omitted

def TriArea2D(vts):
	vd1 = VectorSub(vts[1],vts[0])
	vd2 = VectorSub(vts[2],vts[0])
	return ScalarCross(vd1,vd2)

# 2D triangle orientation -- true if canonical

def TriOrient2D(vts) : return TriArea2D(vts) > 0

# Speed up testing of 2D triangle area and orientation

def TriAreaData2D(vts):
	dt1 = VectorMean(vts[:2])
	dt2 = VectorSub(vts[1],vts[0])
	dt2 = [-dt2[1],dt2[0]]
	return (tuple(dt1), tuple(dt2))

def TriAreaEval2D(data, vt):
	return InnerProduct(data[1], VectorSub(vt, data[0]))

def TriOrientEval2D(data, vt): return TriAreaEval2D(data, vt) > 0

# Related to spherical triangle area
# The minus sign is for consistency with the flat-space definitions

def TriArea2Sph(vts):
	return TripleProduct(vts[0],vts[1],vts[2])

# Spherical triangle orientation -- true if canonical

def TriOrient2Sph(vts): return TriArea2Sph(vts) > 0

# Speed up testing of spherical triangle area and orientation

def TriAreaData2Sph(vts):
	return CrossProduct(vts[0],vts[1])

def TriAreaEval2Sph(data, vt): return InnerProduct(data, vt)

def TriOrientEval2Sph(data, vts): return TriAreaEval2Sph(data, vts) > 0


# 3D tetrahedron volume 
# Factor of 1/6 omitted

def TetVolume3D(vts):
	vd1 = VectorSub(vts[1],vts[0])
	vd2 = VectorSub(vts[2],vts[0])
	vd3 = VectorSub(vts[3],vts[0])
	return TripleProduct(vd1,vd2,vd3)

# 3D tetrahedron orientation -- true if canonical
def TetOrient3D(vts): return TetVolume3D(vts) > 0

# Speed up testing of 3D tetrahedron volume and orientation
def TetVolumeData3D(vts):
	dt1 = VectorMean(vts[:3])
	dt21 = VectorSub(vts[1],vts[0])
	dt22 = VectorSub(vts[2],vts[0])
	dt2 = CrossProduct(dt21,dt22)
	return (tuple(dt1), tuple(dt2))

def TetVolumeEval3D(data, vt):
	return InnerProduct(data[1], VectorSub(vt, data[0]))

def TetOrientEval3D(data, vt): return TetVolumeEval3D(data, vt) > 0

# Triangle area in 3D: area and orientation

def TriArea3D(vts):
	vd1 = VectorSub(vts[1],vts[0])
	vd2 = VectorSub(vts[2],vts[0])
	arvec = CrossProduct(vd1,vd2)
	return arvec

def TriArea3DSqr(vts):
	return VectorAbsSqr(TriArea3D(vts))


# Determinants: for the ND case

# In-place permutation generator (Knuth, non-recursive)
# Implemented as a Python generator
def Permutations(n):
	perm = range(n)
	sig = 1
	
	while True:
		yield (tuple(perm), sig)
		
		k1 = None
		for i in xrange(n - 1):
			if perm[i] < perm[i+1]:
				k1 = i
		if k1 == None: break
		
		k2 = k1 + 1
		for i in xrange(k1+1, n):
			if perm[k1] < perm[i]:
				k2 = i
		
		perm[k1], perm[k2] = perm[k2], perm[k1]
		sig *= -1
		perm[k1+1:] = reversed(perm[k1+1:])
		sig *= (-1 if (((n - (k1+1)) // 2) % 2) == 1 else 1)


# Explicit
def DetExplicit(mat):
	n = len(mat)
	if n == 0:
		return 1
	elif n == 1:
		return mat[0][0]
	elif n == 2:
		mat0 = mat[0]; mat1 = mat[1]
		return mat0[0]*mat1[1] - mat0[1]*mat1[0]
	elif n == 3:
		mat0 = mat[0]; mat1 = mat[1]; mat2 = mat[2]
		return \
		mat0[0]*(mat1[1]*mat2[2] - mat1[2]*mat2[1]) \
		- mat0[1]*(mat1[0]*mat2[2] - mat1[2]*mat2[0]) \
		+ mat0[2]*(mat1[0]*mat2[1] - mat1[1]*mat2[0])
	elif n == 4:
		mat0 = mat[0]; mat1 = mat[1]; mat2 = mat[2]; mat3 = mat[3]
		mat01 = mat2[0]*mat3[1] - mat2[1]*mat3[0]
		mat02 = mat2[0]*mat3[2] - mat2[2]*mat3[0]
		mat03 = mat2[0]*mat3[3] - mat2[3]*mat3[0]
		mat12 = mat2[1]*mat3[2] - mat2[2]*mat3[1]
		mat13 = mat2[1]*mat3[3] - mat2[3]*mat3[1]
		mat23 = mat2[2]*mat3[3] - mat2[3]*mat3[2]
		return \
		mat0[0]*(mat1[1]*mat23 - mat1[2]*mat13 + mat1[3]*mat12) \
		- mat0[1]*(mat1[0]*mat23 - mat1[2]*mat03 + mat1[3]*mat02) \
		+ mat0[2]*(mat1[0]*mat13 - mat1[1]*mat03 + mat1[3]*mat01) \
		- mat0[3]*(mat1[0]*mat12 - mat1[1]*mat02 + mat1[2]*mat01)
	else: return None


# Leibniz's algorithm
def DetLeibniz(mat):
	n = len(mat)
	if n == 0:
		return 1
	
	det = 0
	for perm, sig in Permutations(n):
		term = sig
		for k in xrange(n):
			term *= mat[k][perm[k]]
		det += term
	return det


# Laplace's algorithm (recursive)
def DetLaplace(mat):
	n = len(mat)
	if n == 0:
		return 1
	elif n == 1:
		return mat[0][0]

	det = 0
	sig = 1
	for k in xrange(n):
		submat = [row[1:] for i,row in enumerate(mat) if i != k]
		det += sig*mat[k][0]*DetLaplaceNext(submat)
		sig *= -1
	return det

# Function for continuing
DetLaplaceNext = DetLaplace


# Gaussian elimination (recursive)
def DetGauss(mat):
	n = len(mat)
	if n == 0:
		return 1
	elif n == 1:
		return mat[0][0]
	
	# Split into first column and remaining columns
	FirstCol = [row[0] for row in mat]
	RestCols = [list(row[1:]) for row in mat]
	
	# Find the pivot element, row, and signature
	PivRow = 0
	PivEl = FirstCol[PivRow]
	PivElAbs = abs(PivEl)
	PivSig = 1
	Sig = 1
	for k in xrange(n):
		El = FirstCol[k]
		ElAbs = abs(El)
		if ElAbs > PivElAbs:
			PivRow = k
			PivEl = El
			PivElAbs = ElAbs
			PivSig = Sig
		Sig *= -1
	
	# Do the Gaussian elimination
	ElimRow = VectorScalMult(1.0/PivEl, RestCols[PivRow])
	ElimMat = [VectorSub(RestCols[k], VectorScalMult(FirstCol[k],ElimRow)) \
		for k in xrange(n) if k != PivRow]
	
	return PivSig*PivEl*DetGaussNext(ElimMat)

# Function for continuing
DetGaussNext = DetGauss
	

# LU decomposition
# http://en.wikipedia.org/wiki/LU_decomposition
# http://rosettacode.org/wiki/LU_decomposition
# http://mathworld.wolfram.com/LUDecomposition.html

# Find matrix pivots:
#   signature
#   pivot vector: original indices in resulting order
#   pivoted matrix
def MatrixPivots(mat):
	n = len(mat)
	Sig = 1
	PivIxs = range(n)
	PivMat = list(mat)
	for i in xrange(n):
		PivIdx = i
		PivEl = abs(PivMat[i][i])
		for j in xrange(i+1,n):
			ElAbs = abs(PivMat[j][i])
			if ElAbs > PivEl:
				PivEl = ElAbs
				PivIdx = j
		if PivIdx > i:
			Sig *= -1
			PivIxs[i], PivIxs[PivIdx] = PivIxs[PivIdx], PivIxs[i]
			PivMat[i], PivMat[PivIdx] = PivMat[PivIdx], PivMat[i]
	
	return (Sig, PivIxs, PivMat)

# U and L are in place in the result matrix
# the diagonal of the result matrix is the diagonal of U
# The diagonal of L is assumed to be 1's
# Output:
# Signature of pivot permutation (needed for determinants)
# Pivot indices
# Decomposed matrix (all floating-point)
# Calculated with floating point to avoid (integer)/(integer)
def LUDecomposition(mat):
	Sig, PivIxs, PivMat = MatrixPivots(mat)
	LUDMat = [[float(val) for val in list(row)] for row in PivMat]
	n = len(LUDMat)
	
	for i in xrange(1,n):
		# L's
		for j in xrange(i):
			val = LUDMat[i][j]
			for k in xrange(j):
				val -= LUDMat[k][j]*LUDMat[i][k]
			LUDMat[i][j] = val/LUDMat[j][j]
		
		# U's
		for j in xrange(i,n):
			val = LUDMat[i][j]
			for k in xrange(i):
				val -= LUDMat[k][j]*LUDMat[i][k]
			LUDMat[i][j] = val
	
	return (LUDMat, PivIxs, Sig)

def DetLUD(mat):
	n = len(mat)
	if n == 0:
		return 1
	
	LUDMat, PivIxs, Sig = LUDecomposition(mat)
	
	det = Sig
	for k in xrange(n):
		det *= LUDMat[k][k]
	
	return det

# Final choice:
def Det(mat):
	det = DetExplicit(mat)
	if det == None:
		det = DetLUD(mat)
	return det


# Generalization of cross product to different dimensions -- n-D requires (n-1) vectors

def SuperCrossProduct(vecs):
	n = len(vecs)
	prod = []
	sig = (-1)**n # So Det(mat) = SuperCrossProduct(mat[:-1]) . mat[-1]
	for k in xrange(n+1):
		vred = [list(v[:k]) + list(v[k+1:]) for v in vecs]
		prod.append(sig*Det(vred))
		sig *= -1
	
	return prod


# Solving a system of linear equations using their LU decomposition
# Args: LU decomposition, vector
# Returns: vector with solution
def LUDSolve(LUD, vec):
	LUDMat, PivIxs, Sig = LUD
	n = len(PivIxs)
	
	# Apply pivoting
	outvec = [float(vec[PivIxs[k]]) for k in xrange(n)]
	
	# Apply inverse of L
	for i in xrange(1,n):
		row = LUDMat[i]
		val = outvec[i]
		for j in xrange(i):
			val -= row[j]*outvec[j]
		outvec[i] = val
	
	# Apply inverse of U
	for i in xrange(n-1,-1,-1):
		row = LUDMat[i]
		val = outvec[i]
		for j in xrange(i+1,n):
			val -= row[j]*outvec[j]
		outvec[i] = val/row[i]
	
	return outvec

# Args: matrix, vector
# Returns: vector with solution
def LinearSolve(mat, vec): return LUDSolve(LUDecomposition(mat), vec)

def LinearSolveList(mat, veclist):
	LUD = LUDecomposition(mat)
	return [LUDSolve(LUD,v) for v in veclist]


# Matrix algebra

def MatrixConst(n1,n2,c):
	return [[c for k2 in xrange(n2)] for k1 in xrange(n1)]

def IdentityMatrix(n):
	mat = [n*[0] for k in xrange(n)]
	for k in xrange(n): mat[k][k] = 1
	return mat

def MatrixAdd(m1,m2):
	n1 = len(m1)
	n2 = len(m1[0])
	m = MatrixConst(n1,n2,0)
	for k1 in xrange(n1):
		mr1 = m1[k1]
		mr2 = m2[k1]
		mr = m[k1]
		for k2 in xrange(n2):
			mr[k2] = mr1[k2] + mr2[k2]
	return m

def MatrixAddTo(m1,m2):
	n1 = len(m1)
	n2 = len(m1[0])
	for k1 in xrange(n1):
		mr1 = m1[k1]
		mr2 = m2[k1]
		for k2 in xrange(n2):
			mr1[k2] += mr2[k2]
	return m1

def MatrixSub(m1,m2):
	n1 = len(m1)
	n2 = len(m1[0])
	m = MatrixConst(n1,n2,0)
	for k1 in xrange(n1):
		mr1 = m1[k1]
		mr2 = m2[k1]
		mr = m[k1]
		for k2 in xrange(n2):
			mr[k2] = mr1[k2] - mr2[k2]
	return m

def MatrixSubFrom(m1,m2):
	n1 = len(m1)
	n2 = len(m1[0])
	for k1 in xrange(n1):
		mr1 = m1[k1]
		mr2 = m2[k1]
		for k2 in xrange(n2):
			mr1[k2] -= mr2[k2]
	return m1

def MatrixMult(m1,m2):
	n1 = len(m1)
	nx = len(m1[0])
	n2 = len(m2[0])
	m = MatrixConst(n1,n2,0)
	for k1 in xrange(n1):
		mr1 = m1[k1]
		mr = m[k1]
		for k2 in xrange(n2):
			msum = 0
			for kx in xrange(nx):
				msum += mr1[kx]*m2[kx][k2]
			mr[k2] = msum
	return m

def MatVecMult(mat,v):
	return [InnerProduct(row,v) for row in mat]

def Transpose(mat):
	n1 = len(mat)
	n2 = len(mat[0])
	return [[mat[k1][k2] for k1 in xrange(n1)] for k2 in xrange(n2)]

def Inverse(mat):
	return Transpose(LinearSolveList(mat, IdentityMatrix(len(mat))))


# Simplex functions

# Flips a simplex to get the opposite orientation. 
# A single point ought to have a polarity,
# either begin or end of a line.

def SimplexFlipInPlace(sx):
	if len(sx) >= 2:
		sx[-1], sx[-2] = sx[-2], sx[-1]

def SimplexFlip(sx):
	sxlst = list(sx)
	SimplexFlipInPlace(sxlst)
	return tuple(sxlst)

# Gets a simplex into canonical order:
# sorted or sorted with last two interchanged.

def SimplexCanon(sx):
	sxlst = list(sx)
	
	# Dumb sort
	# Done this way to get the sign of the sort permutation
	n = len(sx)
	sgn = 1
	for i in xrange(n-1):
		for j in xrange(i+1,n):
			if sxlst[i] > sxlst[j]:
				sgn *= -1
				sxlst[i], sxlst[j] = sxlst[j], sxlst[i]
	
	if sgn < 0: SimplexFlipInPlace(sxlst)
	return tuple(sxlst)

# Subsets of the simplex: one step down. Also references
# SimplexSubRefs assumes that the input simplex is a tuple

def SimplexSubs(sx):
	sxlst = list(sx)
	
	n = len(sx)
	sgn = 1
	subs = []
	for k in xrange(n):
		sbsx = sxlst[:]
		del sbsx[-(k+1)]
		if sgn < 0: SimplexFlipInPlace(sbsx)
		subs.append(tuple(sbsx))
		sgn *= -1
	
	return tuple(subs)

def SimplexSubRefs(sx, sxtosbsxs, sbsxtosx):
	sbsxs = SimplexSubs(sx)
	sxtosbsxs[sx] = sbsxs
	for sbsx in sbsxs:
		sbsxtosx[sbsx] = sx

# Volume factor: true volume is 1/(# dims)! times it

def SubFirstFromVecSet(vts):
	vt0 = vts[0]
	return [VectorSub(vt,vt0) for vt in vts[1:]]

def SimplexVolume(vts): return Det(SubFirstFromVecSet(vts))

def SimplexOrient(vts): return SimplexVolume(vts) > 0

# For calculating several volumes

def SimplexVolData(vts):
	return (VectorMean(vts), SuperCrossProduct(SubFirstFromVecSet(vts)))

def SimplexVolEval(data, vt):
	return InnerProduct(data[1], VectorSub(vt,data[0]))

def SimplexOrientEval(data, vt): return SimplexVolEval(data, vt) > 0

# Circumscribed circle / sphere / hypersphere

def CircumData(vts):
	vtsred = SubFirstFromVecSet(vts)
	vtrdsq = [0.5*VectorAbsSqr(v) for v in vtsred]
	ctrx = LinearSolve(vtsred,vtrdsq)
	return (VectorAdd(vts[0],ctrx), VectorAbsSqr(ctrx))

def CircumEval(data, vt):
	return VectorAbsSqr(VectorSub(vt,data[0])) - data[1]

def CircumInside(data, vt): return CircumEval(data, vt) <= 0

# Simplex on a hypersphere

def SphSimplexVolume(vts): return - Det(vts)

def SphSimplexOrient(vts): return SphSimplexVolume(vts) > 0

# For calculating several volumes

def SphSimplexVolData(vts):
	data = SuperCrossProduct(vts)
	VectorScalMultBy(-1,data)
	return data

def SphSimplexVolEval(data, vt): return InnerProduct(data, vt)

def SphSimplexOrientEval(data, vt): return SphSimplexVolEval(data, vt) > 0

# Circumscribed circle / sphere / hypersphere

def SphCircumData(vts): return LinearSolve(vts, len(vts)*[1])

def SphCircumEval(data, vt): return InnerProduct(data, vt)

def SphCircumInside(data, vt): return SphCircumEval(data, vt) >= 1
