(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    479680,      11485]
NotebookOptionsPosition[    462209,      10994]
NotebookOutlinePosition[    462568,      11010]
CellTagsIndexPosition[    462525,      11007]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Introduction", "Section",
 CellChangeTimes->{{3.575709005861823*^9, 3.575709008338258*^9}}],

Cell[TextData[{
 StyleBox["Computational Geometry",
  FontSize->18,
  FontWeight->"Bold"],
 "\n",
 StyleBox["Convex hull\nDelaunay triangulation / tessellation\nVoronoi \
diagram",
  FontSize->14],
 "\n\n",
 StyleBox["Convex Hull",
  FontSize->18,
  FontWeight->"Bold"],
 "\n\n",
 StyleBox["2D flat",
  FontSize->14,
  FontWeight->"Bold"],
 "\nPlanar\nArg: list of 2D vertices\nReturns: indices of convex hull\nStarts \
from lowest index\nAreas of all point triplets in order around the hull: \
positive by TriArea2D\n\nCH2GrahamScan\nCH2GiftWrapping\nCH2DivideConquer\n\
CH2ChanShattering\nCH2QuickHull\nCH2Incremental\nCH2MonotoneChain\n\n\
CH2ChanShattering has has an optional additional argument: which algorithm to \
use for the parts (default: CH2GrahamScan)\n\nNot implemented: Kirkpatrick\
\[Dash]Seidel algorithm (marriage-before-conquest)\n\n",
 StyleBox["3-Spherical",
  FontSize->14,
  FontWeight->"Bold"],
 "\nLike 2D, but with directions in a 3D sphere\n\nCH2SphIncremental\n(only \
one)\n\n\n",
 StyleBox["3D flat",
  FontSize->14,
  FontWeight->"Bold"],
 "\nFlat-Space\nArg: list of 3D vertices\nReturns: {sorted list of vertices, \
sorted list of triangles}\nThe triangles' vertices start from their lowest \
indices\nCH3GiftWrapping\nCH3DivideConquer\nCH3ChanShattering\nCH3QuickHull\n\
CH3Incremental\n\nCH3ChanShattering has has an optional additional argument: \
which algorithm to use for the parts (default: CH3GiftWrapping)\n\nA 3D \
convex hull is a closed triangle-faced polyhedron.\nFor n vertices, it has \
3n-6 edges and 2n-4 faces.\n\n",
 StyleBox["4-Hyperspherical",
  FontSize->14,
  FontWeight->"Bold"],
 "\nLike 3D, but with directions in a 4D sphere\n\nCH3SphIncremental\n(only \
one)\n\n\n",
 StyleBox["ND flat",
  FontSize->14,
  FontWeight->"Bold"],
 "\nArg: list if ND vertices\nDimension of vertices >= 3\nReturns: {sorted \
list of vertices, sorted list of simplexes}\nThe simplexes are sorted with \
the last two indices reversed if needed\nso that the sorting will be an even \
permutation\n\nDivide and conquer was not implemented because of a lack of a \
general algorithm for the belt between each pair of subhulls\n\n\
CHNGiftWrapping\nCHNChanShattering\nCHNQuickHull\nCHNIncremental\n\n\
CHNChanShattering has has an optional additional argument: which algorithm to \
use for the parts (default: CHNGiftWrapping)\n\n",
 StyleBox["(N+1)-Hyperspherical",
  FontSize->14,
  FontWeight->"Bold"],
 "\nLike ND, but with directions in a (N+1)D hypersphere\nDimension of \
vertices >= 4\n\nCHNSphIncremental\n(only one)\n\n\n",
 StyleBox["Hull Sizes",
  FontSize->14,
  FontWeight->"Bold"],
 "\n2D:\nVertices: n\nEdges: n\n3D:\nVertices: n\nEdges: 3n-6\nFaces: 2n-4\n\
4:\nVertices: n\nEdges: n + m\nSurfaces: 2m\nFaces: m -- from (n-3) to \
(n-2)(n-3)/2\nND:\nFaces: ~ n^(floor(d/2)) for d dimensions\n\n\n",
 StyleBox["Delaunay Triangulation / Tessellation",
  FontSize->18,
  FontWeight->"Bold"],
 "\nArg: list of vertices\nReturns: {convex hull, Delaunay tri/tess, \
upper-face tri/tess or empty, number of flips in each pass or none}\n\nAll of \
them are 2D except for the convex-hull one\n\nDT2DivideConquer\nDT2IncrFlip\n\
DT2IncrFlipLin\nDT2IncrFlipRad\nDTConvexHull\n\nFor a spherical surface. All \
of them are for a 3-sphere (2D surface) except for the convex-hull one\n\n\
DT2SphIncrFlip\nDT2SphIncrFlipLin\nDT2SphIncrFlipRad\nDTSphConvexHull\n\nThe \
incremental-flipping (IncrFlip) algorithms add vertices, then flip edges to \
improve the triangulation\nThe \"lin\" ones add the vertices from left to \
right\nThe \"rad\" ones add the vertices from their centroid outward\n\nThe \
convex-hull one uses convex hulls,\nand it can work in any dimension of \
points\n\nConvex-hull functions and their defaults:\n\nDTBoundaryCH2 = \
CH2QuickHull;\nDTBoundaryCH3 = CH3QuickHull;\nDTBoundaryCHN = CHNQuickHull;\n\
DTSphBoundaryCH2 = CH2SphIncremental;\nDTSphBoundaryCHN = CHNSphIncremental;\n\
DTTessCH3 = CH3QuickHull;\nDTTessCHN = CHNQuickHull;\n\n",
 StyleBox["Triangulation Sizes",
  FontSize->14,
  FontWeight->"Bold"],
 "\n2D:\nVertices: n\nHull Vertices: m\nEdges: 3n-3-m\nFaces: 2n-2-m\n\n2D \
Spherical (hull present): same as flat 2D\n2D Spherical (hull absent):\n\
Vertices: n\nEdges: 3n-6\nFaces: 2n-4\n\n",
 StyleBox["References",
  FontSize->18,
  FontWeight->"Bold"],
 "\nWikipedia is usually a good place to start.\n\n2D convex hull\n\
http://www.tcs.fudan.edu.cn/rudolf/Courses/Algorithms/Alg_cs_07w/Webprojects/\
Zhaobo_hull\n\
http://drona.csa.iisc.ernet.in/~gsat/Course/DAA/lecture_notes/jeff_convexhull.\
pdf\nhttp://www.ahristov.com/tutorial/geometry-games/convex-hull.html\n\n3D \
convex hull\nhttp://www.cse.unsw.edu.au/~lambert/java/3d/hull.html\n\
http://www.industrial-geometry.at/events/strobl2006/2006-06steiner.pdf\n\
https://cw.felk.cvut.cz/lib/exe/fetch.php/misc/projects/oppa_oi_english/\
courses/ae4m39vg/lectures/05-convexhull-3d.pdf\n\
http://www.hfink.eu/quickhull3d\n\nND convex hull\n\
http://www.denison.edu/academics/departments/mathcs/eilberg.pdf\n\nDelaunay \
triangulation\n\
http://www.cs.umd.edu/class/spring2012/cmsc754/Lects/lect14.pdf\n\
http://www.karlchenofhell.org/cppswp/lischinski.pdf\nhttp://www.s-hull.org/\n\
http://www.geom.uiuc.edu/~samuelp/del_project.html\n\
http://www.cis.upenn.edu/~jean/gbooks/new281-282.pdf"
}], "Text",
 CellChangeTimes->{{3.5757086473393087`*^9, 3.575708682743266*^9}, {
   3.575708813138597*^9, 3.57570882583245*^9}, {3.575709596858274*^9, 
   3.575709607808126*^9}, {3.575720329625772*^9, 3.5757203416763763`*^9}, {
   3.575721170802681*^9, 3.575721235443544*^9}, 3.575729111680682*^9, 
   3.575786627035974*^9, {3.575983433728527*^9, 3.575983437272941*^9}, {
   3.5759843911425533`*^9, 3.57598439152601*^9}, {3.5759902353113337`*^9, 
   3.575990235631196*^9}, {3.57608170697045*^9, 3.576081726308917*^9}, {
   3.5760821194983397`*^9, 3.5760821211129923`*^9}, {3.57608227759905*^9, 
   3.576082288498782*^9}, {3.576087032727404*^9, 3.576087033155764*^9}, {
   3.576156977601728*^9, 3.57615699392829*^9}, {3.576158989244607*^9, 
   3.576159022450403*^9}, {3.5761590760825253`*^9, 3.57615910088316*^9}, {
   3.576168664487938*^9, 3.5761686783980713`*^9}, {3.5761697310341063`*^9, 
   3.576169731548504*^9}, {3.576169768966826*^9, 3.576169793690432*^9}, {
   3.5761698791643753`*^9, 3.576169914482916*^9}, {3.5761703706058826`*^9, 
   3.576170382136086*^9}, {3.5761704478260403`*^9, 3.576170452600649*^9}, {
   3.5761721432984037`*^9, 3.576172143746828*^9}, {3.576172443372746*^9, 
   3.5761724437616043`*^9}, {3.576874504726158*^9, 3.576874590030792*^9}, {
   3.5768748327829103`*^9, 3.576874880018065*^9}, {3.577820869846613*^9, 
   3.577820870176915*^9}, {3.577932579560114*^9, 3.5779325848721523`*^9}, {
   3.57888573755882*^9, 3.578885778414256*^9}, {3.578886080687964*^9, 
   3.5788861457377367`*^9}, {3.578886224988495*^9, 3.578886255298909*^9}, {
   3.5788934900592537`*^9, 3.578893497780953*^9}, {3.5790228842058773`*^9, 
   3.5790229096355753`*^9}, {3.57904257200988*^9, 3.5790427663743258`*^9}, {
   3.5790428195896683`*^9, 3.579042925135214*^9}, {3.579068762289257*^9, 
   3.579068763766045*^9}, {3.579068988121089*^9, 3.5790689952032137`*^9}, {
   3.579069109359342*^9, 3.579069163817078*^9}, {3.579069284098137*^9, 
   3.579069285433959*^9}, {3.579069863166267*^9, 3.5790700803862257`*^9}, {
   3.579070156694129*^9, 3.5790701738471937`*^9}, {3.5791427992345457`*^9, 
   3.579142800021406*^9}, {3.5793479906883698`*^9, 3.579348021656098*^9}, {
   3.579547618597674*^9, 3.579547619340282*^9}, {3.579956316558197*^9, 
   3.57995633192556*^9}, {3.579958075118164*^9, 3.579958104866881*^9}, {
   3.57995818017162*^9, 3.579958241908229*^9}, {3.5799582823666763`*^9, 
   3.579958343781229*^9}, {3.5799583895456057`*^9, 3.5799584196306353`*^9}, {
   3.5799584562270823`*^9, 3.579958576032432*^9}, 3.579958609832409*^9, {
   3.579959237234036*^9, 3.579959250123816*^9}, {3.579959505101997*^9, 
   3.579959528674169*^9}, {3.581236674669025*^9, 3.581236682458003*^9}, {
   3.581401067452249*^9, 3.581401103317503*^9}, {3.5814011930795193`*^9, 
   3.5814012189996367`*^9}, {3.581401412408719*^9, 3.581401565213357*^9}, {
   3.581580100912373*^9, 3.5815801098542747`*^9}, {3.581582306034643*^9, 
   3.581582314354409*^9}, {3.5819173981868668`*^9, 3.581917398616232*^9}, {
   3.581996378961898*^9, 3.581996422786406*^9}, {3.581996867292213*^9, 
   3.581996892189315*^9}, {3.582009919074086*^9, 3.582009933304792*^9}, {
   3.582515730329763*^9, 3.582515781361785*^9}, {3.5825159933032846`*^9, 
   3.582515994619647*^9}, {3.5825163481889887`*^9, 3.582516352403843*^9}, 
   3.582516414316691*^9, {3.582516469746818*^9, 3.582516494378866*^9}, {
   3.582516549893321*^9, 3.5825165502734137`*^9}, {3.582534659088058*^9, 
   3.582534659494598*^9}, {3.5825534828513803`*^9, 3.582553483168413*^9}, {
   3.582554957798801*^9, 3.582554963353796*^9}, {3.58255560991336*^9, 
   3.582555654927126*^9}, {3.5825563209516077`*^9, 3.582556382320307*^9}, {
   3.582556412680973*^9, 3.582556510369801*^9}, {3.582556562090815*^9, 
   3.582556645515142*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shared", "Section",
 CellChangeTimes->{{3.575715443411428*^9, 3.575715457704907*^9}}],

Cell[CellGroupData[{

Cell["General", "Subsection",
 CellChangeTimes->{{3.576157016863895*^9, 3.576157018477602*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Indices", " ", "of", " ", "the", " ", "minimum", " ", "and", " ", 
   "maximum", " ", "values"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5757191334747458`*^9, 3.5757191411703444`*^9}, {
  3.575719313122664*^9, 3.5757193248645678`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMin", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", 
   RowBox[{"Ordering", "[", 
    RowBox[{"xlst", ",", "1"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575719158952238*^9, 3.575719169815287*^9}, {
  3.575719246185299*^9, 3.575719273352806*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMax", "[", "xlst_", "]"}], " ", ":=", "  ", 
  RowBox[{"First", "[", 
   RowBox[{"Ordering", "[", 
    RowBox[{"xlst", ",", 
     RowBox[{"-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575719158952238*^9, 3.575719169815287*^9}, {
  3.575719246185299*^9, 3.575719307074003*^9}, {3.575990373275371*^9, 
  3.5759903752186537`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMinBySortFunc", "[", 
   RowBox[{"xlst_", ",", "sortfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", 
   RowBox[{"Ordering", "[", 
    RowBox[{"xlst", ",", "1", ",", "sortfunc"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575722005043613*^9, 3.575722023980178*^9}, {
  3.5761466160084677`*^9, 3.576146619374783*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMaxBySortFunc", "[", 
   RowBox[{"xlst_", ",", "sortfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", 
   RowBox[{"Ordering", "[", 
    RowBox[{"xlst", ",", 
     RowBox[{"-", "1"}], ",", "sortfunc"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575722025667889*^9, 3.575722045297617*^9}, {
  3.576087676152766*^9, 3.576087683094648*^9}, {3.576146623241604*^9, 
  3.5761466398897953`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMinBySortByFunc", "[", 
   RowBox[{"xlst_", ",", "sortbyfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"IndexMinBySortFunc", "[", 
   RowBox[{"xlst", ",", 
    RowBox[{
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"sortbyfunc", "[", "#1", "]"}], ",", 
        RowBox[{"sortbyfunc", "[", "#2", "]"}]}], "}"}], "]"}], "&"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576146649304223*^9, 3.576146686431912*^9}, {
  3.576147103645054*^9, 3.57614710439293*^9}, {3.5761471385910263`*^9, 
  3.576147149829365*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"IndexMaxBySortByFunc", "[", 
   RowBox[{"xlst_", ",", "sortbyfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"IndexMaxBySortFunc", "[", 
   RowBox[{"xlst", ",", 
    RowBox[{
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"sortbyfunc", "[", "#1", "]"}], ",", 
        RowBox[{"sortbyfunc", "[", "#2", "]"}]}], "}"}], "]"}], "&"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576146649304223*^9, 3.576146683599831*^9}, {
  3.576147106940734*^9, 3.5761471077961683`*^9}, {3.576147141506432*^9, 
  3.576147146685993*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MinBySortFunc", "[", 
   RowBox[{"xlst_", ",", "sortfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"xlst", "[", 
   RowBox[{"[", 
    RowBox[{"IndexMinBySortFunc", "[", 
     RowBox[{"xlst", ",", "sortfunc"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5757228773730516`*^9, 3.5757228966914454`*^9}, {
  3.576087708134068*^9, 3.576087722547716*^9}, {3.576146316903205*^9, 
  3.5761463176358624`*^9}, {3.576146718344101*^9, 3.576146727316235*^9}, {
  3.57614702620408*^9, 3.576147026847568*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MaxBySortFunc", "[", 
   RowBox[{"xlst_", ",", "sortfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"xlst", "[", 
   RowBox[{"[", 
    RowBox[{"IndexMaxBySortFunc", "[", 
     RowBox[{"xlst", ",", "sortfunc"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575722899487109*^9, 3.5757229111729717`*^9}, {
  3.576087728866679*^9, 3.5760877334969673`*^9}, {3.5761463207403173`*^9, 
  3.5761463213000727`*^9}, {3.5761467217537603`*^9, 3.576146734427072*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MinBySortByFunc", "[", 
   RowBox[{"xlst_", ",", "sortbyfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"xlst", "[", 
   RowBox[{"[", 
    RowBox[{"IndexMinBySortByFunc", "[", 
     RowBox[{"xlst", ",", "sortbyfunc"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.576146322691777*^9, 3.576146399557166*^9}, {
  3.576146743806253*^9, 3.576146763960333*^9}, {3.576147029700067*^9, 
  3.576147030266934*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MaxBySortByFunc", "[", 
   RowBox[{"xlst_", ",", "sortbyfunc_"}], "]"}], " ", ":=", " ", 
  RowBox[{"xlst", "[", 
   RowBox[{"[", 
    RowBox[{"IndexMaxBySortByFunc", "[", 
     RowBox[{"xlst", ",", "sortbyfunc"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.576146322691777*^9, 3.576146402214673*^9}, {
  3.57614674380829*^9, 3.576146767104652*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Add", " ", 
   RowBox[{"indices", " ", "--"}], " ", 
   RowBox[{"(", 
    RowBox[{"index", ",", "value"}], ")"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575719384569254*^9, 3.5757194203732567`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AddIndices", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"Transpose", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"Length", "[", "xlst", "]"}], "]"}], ",", "xlst"}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575719422725151*^9, 3.575719440851308*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Select", " ", "using", " ", "list", " ", "of", " ", "conditions"}],
   " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5757195068626328`*^9, 3.575719534482728*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SelWithList", "[", 
   RowBox[{"sllst_", ",", "xlst_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"#", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", 
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"sllst", ",", "xlst"}], "}"}], "]"}], ",", 
     RowBox[{
      RowBox[{"#", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "&"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5757195391642523`*^9, 3.575719581108732*^9}, {
  3.575719660917794*^9, 3.575719670981834*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Rotate", " ", "a", " ", "list", " ", "so", " ", "that", " ", "it", " ", 
   "starts", " ", "with", " ", "its", " ", "minimum", " ", "value"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.5759904637574*^9, 3.575990474665557*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"RotateToMin", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "ixmin", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Length", "[", "xlst", "]"}], ">", "0"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"ixmin", " ", "=", " ", 
        RowBox[{"IndexMin", "[", "xlst", "]"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"Take", "[", 
          RowBox[{"xlst", ",", 
           RowBox[{"{", 
            RowBox[{"ixmin", ",", 
             RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
         RowBox[{"Take", "[", 
          RowBox[{"xlst", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"ixmin", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", "xlst"}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.575990257793954*^9, 3.575990289503443*^9}, {
  3.575990394720148*^9, 3.57599044141038*^9}, {3.575990611254284*^9, 
  3.5759906126522017`*^9}, {3.575990766654619*^9, 3.575990783886755*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Connect", " ", "the", " ", "ends"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575717383191905*^9, 3.575717392029852*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeLoop", "[", "lst_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "lst", "]"}], ">", "0"}], ",", 
    RowBox[{"Append", "[", 
     RowBox[{"lst", ",", 
      RowBox[{"First", "[", "lst", "]"}]}], "]"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575717398748879*^9, 3.575717407697768*^9}, {
  3.57616122467772*^9, 3.576161238728237*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Extract", " ", "from", " ", "Reap", " ", "output"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579017496570155*^9, 3.579017504021861*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ExtractReaped", "[", 
   RowBox[{"rpres_", ",", "k_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "rpi", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"rpi", " ", "=", " ", 
      RowBox[{"rpres", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "k"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "rpi", "]"}], " ", ">", " ", "0"}], ",", 
       RowBox[{"rpi", "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{"{", "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.579017506328601*^9, 3.5790175694188833`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2D", "Subsection",
 CellChangeTimes->{{3.576157040984404*^9, 3.5761570413747053`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Equivalent", " ", "to", " ", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{"v1", ",", "v2"}], "}"}], "]"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575716727095564*^9, 3.5757167335598097`*^9}, {
  3.5796087220577173`*^9, 3.5796087224013157`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cross2D", "[", 
   RowBox[{"v1_", ",", "v2_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"v1", "[", 
     RowBox[{"[", "1", "]"}], "]"}], "*", 
    RowBox[{"v2", "[", 
     RowBox[{"[", "2", "]"}], "]"}]}], " ", "-", " ", 
   RowBox[{
    RowBox[{"v1", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "*", 
    RowBox[{"v2", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.575716724133827*^9, 3.5757167521685343`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Edges", " ", "and", " ", "triangle"}], "-", 
   RowBox[{"edge", " ", "references"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579598449399969*^9, 3.57959845376414*^9}, {
  3.5795985265197353`*^9, 3.579598531092183*^9}, {3.57959863946367*^9, 
  3.579598644877589*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriEdges", "[", "tri_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"tri", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"tri", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"tri", "[", 
       RowBox[{"[", "2", "]"}], "]"}], ",", 
      RowBox[{"tri", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"tri", "[", 
       RowBox[{"[", "3", "]"}], "]"}], ",", 
      RowBox[{"tri", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.579598455705456*^9, 3.5795984799017982`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriEdgeRefs", "[", 
   RowBox[{"tri_", ",", "trieds_", ",", "edtri_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"eds", ",", "ed"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"eds", " ", "=", " ", 
      RowBox[{"TriEdges", "[", "tri", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"trieds", "[", "tri", "]"}], " ", "=", " ", "eds"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"edtri", "[", "ed", "]"}], " ", "=", " ", "tri"}], ",", 
       RowBox[{"{", 
        RowBox[{"ed", ",", "eds"}], "}"}]}], "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.57768187804889*^9, 3.5776819508002243`*^9}, {
  3.5795987163698473`*^9, 3.579598717076333*^9}, {3.579598769281355*^9, 
  3.579598802674*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"2", "D", " ", "triangle", " ", 
   RowBox[{"area", ".", " ", "Factor"}], " ", "of", " ", 
   RowBox[{"1", "/", "2"}], " ", "ignored"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.57571691118429*^9, 3.575716921866921*^9}, {
  3.575786865855875*^9, 3.575786866001341*^9}, {3.5796087253932734`*^9, 
  3.579608728977374*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriArea2D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Cross2D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vts", "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{"vts", "[", 
      RowBox[{"[", "3", "]"}], "]"}], "-", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575716908486466*^9, 3.57571696087994*^9}, {
  3.579608678298369*^9, 3.579608678714731*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"2", "D", " ", "triangle", " ", 
   RowBox[{"orientation", " ", "--"}], " ", "true", " ", "if", " ", 
   "canonical"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5757868547040167`*^9, 3.5757868965681343`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriOrient2D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TriArea2D", "[", "vts", "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.5757868741069937`*^9, 3.575786886657173*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Speed", " ", "up", " ", "testing", " ", "of", " ", "2", "D", " ", 
   "triangle", " ", "area", " ", "and", " ", "orientation"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.577873661246558*^9, 3.5778736745832253`*^9}, {
   3.577876140128874*^9, 3.577876143168006*^9}, 3.577876473848072*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriAreaData2D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Mean", "[", "vts", "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "}"}], "&"}], " ", "@", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"vts", "[", 
        RowBox[{"[", "2", "]"}], "]"}], " ", "-", " ", 
       RowBox[{"vts", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5778737073811502`*^9, 3.5778737679790363`*^9}, {
   3.577874924575107*^9, 3.577874928389188*^9}, {3.577875001754714*^9, 
   3.577875006403315*^9}, {3.577876477413889*^9, 3.5778764794529047`*^9}, 
   3.579608683370965*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriAreaEval2D", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"data", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"vt", " ", "-", " ", 
     RowBox[{"data", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.577873777383772*^9, 3.577873804917939*^9}, {
  3.5778742215116577`*^9, 3.577874223429715*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriOrientEval2D", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TriAreaEval2D", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.577874080185028*^9, 3.5778740939380283`*^9}, {
  3.5778742060638638`*^9, 3.5778742144453382`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2D Spherical", "Subsection",
 CellChangeTimes->{{3.5761570487920732`*^9, 3.5761570511739893`*^9}, {
  3.581582407041135*^9, 3.581582407683325*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Related", " ", "to", " ", "spherical", " ", "triangle", " ", 
    RowBox[{"area", ".", " ", "The"}], " ", "minus", " ", "sign", " ", "is", 
    " ", "for", " ", "consistency", " ", "with", " ", "the", " ", "flat"}], 
   "-", 
   RowBox[{"space", " ", "definitions"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576158124498707*^9, 3.576158138675808*^9}, {
  3.579950702043469*^9, 3.579950704902857*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriArea2Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Det", "[", 
   RowBox[{"Take", "[", 
    RowBox[{"vts", ",", "3"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.576158141359612*^9, 3.576158159808469*^9}, {
  3.5790695216462*^9, 3.579069522958518*^9}, {3.579070498867197*^9, 
  3.5790705023208637`*^9}, {3.579950708608038*^9, 3.579950708863665*^9}, {
  3.581581442768238*^9, 3.581581442930389*^9}, {3.58158242304638*^9, 
  3.581582424189713*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Spherical", " ", "triangle", " ", 
   RowBox[{"orientation", " ", "--"}], " ", "true", " ", "if", " ", 
   "canonical"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576158170685521*^9, 3.576158174243894*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriOrient2Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TriArea2Sph", "[", "vts", "]"}], " ", ">", " ", "0"}]}]], "Input",\

 CellChangeTimes->{{3.5761581784445744`*^9, 3.576158192613125*^9}, {
  3.5815824267664824`*^9, 3.581582430591964*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Speed", " ", "up", " ", "testing", " ", "of", " ", "spherical", " ", 
   "triangle", " ", "area", " ", "and", " ", "orientation"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.577873661246558*^9, 3.5778736745832253`*^9}, {
  3.577874108909979*^9, 3.577874111236553*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriAreaData2Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{"vts", "[", 
     RowBox[{"[", "1", "]"}], "]"}], ",", 
    RowBox[{"vts", "[", 
     RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577876065919635*^9, 3.5778760939714737`*^9}, {
   3.579950714792351*^9, 3.5799507153682528`*^9}, {3.581581448292611*^9, 
   3.581581448500106*^9}, 3.5815824327277184`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriAreaEval2Sph", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"data", ".", "vt"}]}]], "Input",
 CellChangeTimes->{{3.577876098036178*^9, 3.5778761068444853`*^9}, {
   3.577876169569675*^9, 3.5778761711284323`*^9}, 3.581582434085641*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriOrientEval2Sph", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TriAreaEval2Sph", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.5778761092097816`*^9, 3.577876124584244*^9}, {
  3.5778761743797827`*^9, 3.5778761764489*^9}, {3.581582435663911*^9, 
  3.581582436869897*^9}, {3.58158248185532*^9, 3.581582482751403*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3D", "Subsection",
 CellChangeTimes->{{3.577056729632843*^9, 3.577056730014923*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"3", "D", " ", "tetrahedron", " ", 
   RowBox[{"volume", ".", " ", "Factor"}], " ", "of", " ", 
   RowBox[{"1", "/", "6"}], " ", "ignored"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.57705685402111*^9, 3.5770568611701317`*^9}, {
  3.579608704921092*^9, 3.579608708673395*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolume3D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Det", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"vts", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "-", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"vts", "[", 
       RowBox[{"[", "3", "]"}], "]"}], "-", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ",", " ", 
     RowBox[{
      RowBox[{"vts", "[", 
       RowBox[{"[", "4", "]"}], "]"}], "-", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5770568758333683`*^9, 3.5770569187745533`*^9}, 
   3.579608700979981*^9, {3.579954274289496*^9, 3.5799543154709253`*^9}, {
   3.579954980151134*^9, 3.5799550368147793`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"3", "D", " ", "tetrahedron", " ", 
   RowBox[{"orientation", " ", "--"}], " ", "true", " ", "if", " ", 
   "canonical"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.577056937677507*^9, 3.577056942447257*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetOrient3D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TetVolume3D", "[", "vts", "]"}], " ", ">", " ", "0"}]}]], "Input",\

 CellChangeTimes->{{3.577057015672566*^9, 3.577057022428958*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Speed", " ", "up", " ", "testing", " ", "of", " ", "3", "D", " ", 
   "tetrahedron", " ", "volume", " ", "and", " ", "orientation"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.577873661246558*^9, 3.5778736745832253`*^9}, {
  3.577876140128874*^9, 3.577876143168006*^9}, {3.577876359573406*^9, 
  3.577876375594421*^9}, {3.5778764659230137`*^9, 3.577876466723444*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolumeData3D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Mean", "[", "vts", "]"}], ",", " ", 
    RowBox[{"Cross", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vts", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "-", 
       RowBox[{"vts", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"vts", "[", 
        RowBox[{"[", "3", "]"}], "]"}], "-", 
       RowBox[{"vts", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5778737073811502`*^9, 3.5778737679790363`*^9}, {
   3.577874924575107*^9, 3.577874928389188*^9}, {3.577875001754714*^9, 
   3.577875006403315*^9}, {3.577876380295166*^9, 3.577876382668236*^9}, {
   3.577876428152095*^9, 3.5778764589160624`*^9}, {3.577887198977751*^9, 
   3.577887199248979*^9}, 3.579608697403314*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolumeEval3D", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"data", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"vt", " ", "-", " ", 
     RowBox[{"data", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.577873777383772*^9, 3.577873804917939*^9}, {
   3.5778742215116577`*^9, 3.577874223429715*^9}, 3.5778763883822813`*^9, {
   3.577887202736855*^9, 3.5778872029526663`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetOrientEval3D", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TetVolumeEval3D", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.577874080185028*^9, 3.5778740939380283`*^9}, {
  3.5778742060638638`*^9, 3.5778742144453382`*^9}, {3.5778763912052917`*^9, 
  3.577876395981761*^9}, {3.577887207849173*^9, 3.57788721236836*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Triangle", " ", "area", " ", "in", " ", "3", 
   RowBox[{"D", ":", " ", 
    RowBox[{"area", " ", "and", " ", "orientation"}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.577274575262575*^9, 3.577274584062673*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriArea3D", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Cross", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vts", "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{"vts", "[", 
      RowBox[{"[", "3", "]"}], "]"}], "-", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577274586224032*^9, 3.577274595202552*^9}, {
  3.5772746364151993`*^9, 3.5772746454729156`*^9}, {3.579608691410984*^9, 
  3.5796086920899878`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"VecSqr", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"x", ".", "x"}]}]], "Input",
 CellChangeTimes->{{3.579615114276672*^9, 3.579615118166005*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TriArea3DSqr", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"VecSqr", "[", 
   RowBox[{"TriArea3D", "[", "vts", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577274714919837*^9, 3.577274741816778*^9}, {
  3.5820877730004673`*^9, 3.582087776053844*^9}, {3.582187569556369*^9, 
  3.5821875710028343`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"3", "D", " ", "convex"}], "-", 
   RowBox[{"hull", " ", "output", " ", 
    RowBox[{"format", ":", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
          "list", " ", "of", " ", "vertices", " ", "in", " ", "increasing", 
           " ", "order"}], ")"}], ",", " ", 
         RowBox[{"(", 
          RowBox[{"list", " ", "of", " ", "triangles"}], ")"}]}], ")"}], ".", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "This"}], " ", 
      "function", " ", "composes", " ", "the", " ", "output", " ", "from", 
      " ", "a", " ", "list", " ", "of", " ", 
      RowBox[{"triangles", "."}]}]}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.577166314770895*^9, 3.577166364512662*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3DOutputFromTriangles", "[", "hulltris_", "]"}], " ", ":=", " ", 
  
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hullvts", ",", "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hullvts", " ", "=", " ", 
      RowBox[{"Union", "[", 
       RowBox[{"Flatten", "[", "hulltris", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"hullvts", ",", 
        RowBox[{"Sort", "[", 
         RowBox[{"RotateToMin", " ", "/@", " ", "hulltris"}], "]"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", "hull"}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input",
 CellChangeTimes->{{3.577166370890436*^9, 3.5771664077296467`*^9}, {
  3.5788859733688507`*^9, 3.57888598451553*^9}, {3.578886160005533*^9, 
  3.57888616257943*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3D Spherical", "Subsection",
 CellChangeTimes->{{3.579069749464383*^9, 3.5790697512947493`*^9}, {
  3.5815824096274347`*^9, 3.581582411475564*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Related", " ", "to", " ", "hyperspherical", " ", "tetrahedron", " ", 
    RowBox[{"volume", ".", " ", "The"}], " ", "minus", " ", "sign", " ", "is",
     " ", "for", " ", "consistency", " ", "with", " ", "the", " ", "flat"}], 
   "-", 
   RowBox[{"space", " ", "definitions"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576158124498707*^9, 3.576158138675808*^9}, {
  3.579069795475903*^9, 3.5790698013453627`*^9}, {3.579950675614585*^9, 
  3.579950686695346*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolume3Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"-", " ", 
   RowBox[{"Det", "[", 
    RowBox[{"Take", "[", 
     RowBox[{"vts", ",", "4"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.576158141359612*^9, 3.576158159808469*^9}, {
   3.5790695216462*^9, 3.579069522958518*^9}, 3.579069790788884*^9, {
   3.579070518040554*^9, 3.579070521393173*^9}, {3.579950656999172*^9, 
   3.5799506574064293`*^9}, {3.58157992953209*^9, 3.581579929940625*^9}, {
   3.581582548697569*^9, 3.581582549790663*^9}, {3.581637861107698*^9, 
   3.5816378613447943`*^9}, {3.581918210422843*^9, 3.581918210731071*^9}, {
   3.581918290303645*^9, 3.581918290527206*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Hyperspherical", " ", "tetrahedron", " ", 
   RowBox[{"orientation", " ", "--"}], " ", "true", " ", "if", " ", 
   "canonical"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576158170685521*^9, 3.576158174243894*^9}, {
  3.5790698062105503`*^9, 3.579069810497485*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetOrient3Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TetVolume3Sph", "[", "vts", "]"}], " ", ">", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.5761581784445744`*^9, 3.576158192613125*^9}, 
   3.5790698166688023`*^9, {3.5815825535604563`*^9, 3.581582559470694*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Speed", " ", "up", " ", "testing", " ", "of", " ", "hyperspherical", " ", 
   "triangle", " ", "volume", " ", "and", " ", "orientation"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.577873661246558*^9, 3.5778736745832253`*^9}, {
  3.577874108909979*^9, 3.577874111236553*^9}, {3.579069823156052*^9, 
  3.5790698279697723`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolumeData3Sph", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"-", " ", 
   RowBox[{"Cross", "[", 
    RowBox[{
     RowBox[{"vts", "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", 
     RowBox[{"vts", "[", 
      RowBox[{"[", "3", "]"}], "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5790695738762283`*^9, 3.5790695861426353`*^9}, {
  3.579950663143281*^9, 3.5799506634391623`*^9}, {3.581579933444241*^9, 
  3.581579933668062*^9}, {3.581582567059061*^9, 3.581582572769559*^9}, {
  3.581637866498323*^9, 3.581637866761888*^9}, {3.58191821462141*^9, 
  3.581918214828308*^9}, {3.581918294298311*^9, 3.581918295812649*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetVolumeEval3Sph", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"data", ".", "vt"}]}]], "Input",
 CellChangeTimes->{{3.579069601728591*^9, 3.579069621231536*^9}, 
   3.581582570091796*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TetOrientEval3Sph", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"TetVolumeEval3Sph", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.579069600382653*^9, 3.579069641968616*^9}, {
  3.581582579008102*^9, 3.581582580962297*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["ND", "Subsection",
 CellChangeTimes->{{3.57906967348816*^9, 3.5790696736140547`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Functions", " ", "for", " ", 
   RowBox[{"simplexes", ":", " ", 
    RowBox[{"n", "-", 
     RowBox[{
     "D", " ", "generalizations", " ", "of", " ", "triangles", " ", "and", 
      " ", "tetrahedra"}]}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5790866205372667`*^9, 3.5790866569244432`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Flips", " ", "a", " ", "simplex", " ", "to", " ", "get", " ", "the", " ", 
    "opposite", " ", 
    RowBox[{"orientation", ".", " ", "A"}], " ", "single", " ", "point", " ", 
    "ought", " ", "to", " ", "have", " ", "a", " ", "polarity"}], ",", " ", 
   RowBox[{
   "either", " ", "begin", " ", "or", " ", "end", " ", "of", " ", "a", " ", 
    RowBox[{"line", "."}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5790868789081593`*^9, 3.579086887059494*^9}, {
  3.579335681434708*^9, 3.5793357119611683`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexFlip", "[", "sx_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "sx", "]"}], " ", "\[GreaterEqual]", " ", "2"}], 
    ",", " ", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"Drop", "[", 
       RowBox[{"sx", ",", 
        RowBox[{"-", "2"}]}], "]"}], ",", 
      RowBox[{"sx", "[", 
       RowBox[{"[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "2"}]}], "}"}], "]"}], "]"}]}], "]"}], ",", "sx"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.579086734735416*^9, 3.579086780075836*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Gets", " ", "a", " ", "simplex", " ", "into", " ", "canonical", " ", 
   RowBox[{"order", ":", " ", 
    RowBox[{
    "sorted", " ", "or", " ", "sorted", " ", "with", " ", "last", " ", "two", 
     " ", "interchanged"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.57908689444995*^9, 3.5790869136203413`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexCanon", "[", "sx_", "]"}], " ", ":=", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Signature", "[", "sx", "]"}], "*", 
        RowBox[{"Signature", "[", "#", "]"}]}], " ", "<", " ", "0"}], ",", 
      " ", 
      RowBox[{"SimplexFlip", "[", "#", "]"}], ",", "#"}], "]"}], "&"}], " ", 
   "@", " ", 
   RowBox[{"Sort", "[", "sx", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5790867958735657`*^9, 3.579086866117363*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Subsets", " ", "of", " ", "the", " ", 
   RowBox[{"simplex", ":", " ", 
    RowBox[{"one", " ", "step", " ", 
     RowBox[{"down", ".", " ", "Also"}], " ", "references"}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579088271645232*^9, 3.5790882834356422`*^9}, {
  3.5790957052239513`*^9, 3.579095706799354*^9}, {3.57959924867059*^9, 
  3.5795992518124037`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexSubs", "[", "sx_", "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"EvenQ", "[", "k", "]"}], ",", 
        RowBox[{"SimplexFlip", "[", "#", "]"}], ",", "#"}], "]"}], "&"}], " ",
      "@", " ", 
     RowBox[{"Delete", "[", 
      RowBox[{"sx", ",", 
       RowBox[{"-", "k"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"Length", "[", "sx", "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5790882853274393`*^9, 3.579088335196212*^9}, {
  3.579095699995903*^9, 3.579095701234111*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexSubRefs", "[", 
   RowBox[{"sx_", ",", " ", "sxtosbsxs_", ",", " ", "sbsxtosx_"}], "]"}], " ",
   ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sbsxs", ",", "sbsx"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"sbsxs", " ", "=", " ", 
      RowBox[{"SimplexSubs", "[", "sx", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"sxtosbsxs", "[", "sx", "]"}], " ", "=", " ", "sbsxs"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"sbsxtosx", "[", "sbsx", "]"}], " ", "=", " ", "sx"}], ",", 
       RowBox[{"{", 
        RowBox[{"sbsx", ",", "sbsxs"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5790954978772287`*^9, 3.579095674424193*^9}, {
  3.5790958466698313`*^9, 3.579095919124938*^9}, {3.579145592034299*^9, 
  3.579145593296064*^9}, {3.57914564250698*^9, 3.5791457023474092`*^9}, {
  3.5791458562546864`*^9, 3.579145857021172*^9}, {3.5796001904837923`*^9, 
  3.5796002625640717`*^9}, {3.579600945010766*^9, 3.57960094860165*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Volume", " ", 
   RowBox[{"factor", ":", " ", 
    RowBox[{"true", " ", "volume", " ", "is", " ", 
     RowBox[{"1", "/", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"#", " ", "dims"}], ")"}], "!"}]}], " ", "times", " ", 
     "it"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5790931400611362`*^9, 3.579093140946187*^9}, {
  3.579093200997542*^9, 3.579093219285946*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SubFirstFromVecSet", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"vt", "-", "#"}], ",", 
      RowBox[{"{", 
       RowBox[{"vt", ",", 
        RowBox[{"Rest", "[", "vts", "]"}]}], "}"}]}], "]"}], "&"}], " ", "@", 
   " ", 
   RowBox[{"First", "[", "vts", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579094962385428*^9, 3.579094998756989*^9}, {
  3.579095032571306*^9, 3.5790950390501347`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolume", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Det", "[", 
   RowBox[{"SubFirstFromVecSet", "[", "vts", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5790932211028833`*^9, 3.5790932434641943`*^9}, {
  3.579093388754973*^9, 3.579093411168788*^9}, {3.579095045148293*^9, 
  3.579095046703265*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexOrient", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SimplexVolume", "[", "vts", "]"}], " ", ">", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.579093529302081*^9, 3.5790935430013943`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"For", " ", "calculating", " ", "several", " ", "volumes"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579093547819935*^9, 3.579093557640921*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolData", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Mean", "[", "vts", "]"}], ",", 
    RowBox[{"(", 
     RowBox[{"Cross", " ", "@@", " ", 
      RowBox[{"SubFirstFromVecSet", "[", "vts", "]"}]}], ")"}]}], 
   "}"}]}]], "Input",
 CellChangeTimes->{{3.579094934839735*^9, 3.5790949531126127`*^9}, {
  3.5790950553808117`*^9, 3.579095064793631*^9}, {3.579146785874645*^9, 
  3.5791467883296413`*^9}, {3.579345050355288*^9, 3.579345050658032*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"data", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"vt", "-", 
     RowBox[{"data", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.579095070593383*^9, 3.579095094397604*^9}, {
  3.579146741428403*^9, 3.579146741752597*^9}, {3.579146824197542*^9, 
  3.5791468244023*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexOrientEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SimplexVolEval", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.579095098532827*^9, 3.5790951225537653`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Verification", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.581916493467774*^9, 3.581916496071574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolDataTest", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SimplexVolEval", "[", 
    RowBox[{
     RowBox[{"SimplexVolData", "[", 
      RowBox[{"Most", "[", "vts", "]"}], "]"}], ",", 
     RowBox[{"Last", "[", "vts", "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"SimplexVolume", "[", "vts", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579345057803644*^9, 3.579345108573387*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolDataTestArray", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SimplexVolDataTest", "[", 
    RowBox[{"Array", "[", 
     RowBox[{"a", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"n", "+", "1"}], ",", "n"}], "}"}]}], "]"}], "]"}], " ", "//",
    " ", "Factor"}]}]], "Input",
 CellChangeTimes->{{3.5793451111798687`*^9, 3.579345172683342*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SimplexVolDataTestArrSet", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Array", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SimplexVolDataTestArray", "[", 
      RowBox[{"a", ",", "#"}], "]"}], "&"}], ",", 
    RowBox[{"n", "-", "1"}], ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579345149435354*^9, 3.579345192163514*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Circumscribed", " ", 
   RowBox[{
    RowBox[{"circle", "/", "sphere"}], "/", "hypersphere"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579614876582326*^9, 3.5796148899633636`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CircumData", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vtsred", ",", "ctrx"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vtsred", " ", "=", " ", 
      RowBox[{"SubFirstFromVecSet", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ctrx", " ", "=", " ", 
      RowBox[{
       RowBox[{"LinearSolve", "[", 
        RowBox[{"vtsred", ",", 
         RowBox[{"VecSqr", " ", "/@", " ", "vtsred"}]}], "]"}], "/", "2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"First", "[", "vts", "]"}], "+", "ctrx"}], ",", 
       RowBox[{"VecSqr", "[", "ctrx", "]"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579614898380863*^9, 3.579614908987775*^9}, {
  3.579615178649371*^9, 3.579615190039318*^9}, {3.579615779809709*^9, 
  3.5796158529758863`*^9}, {3.579615950796104*^9, 3.579616006617489*^9}, {
  3.579616129541875*^9, 3.5796161746756496`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CircumEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"VecSqr", "[", 
    RowBox[{"vt", " ", "-", " ", 
     RowBox[{"data", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"data", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579614910927075*^9, 3.5796149622940607`*^9}, {
  3.579615002412698*^9, 3.579615012919197*^9}, {3.579615050769951*^9, 
  3.579615053927165*^9}, {3.579615127155712*^9, 3.579615159862734*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CircumInside", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"CircumEval", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", "\[LessEqual]", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.579615015756401*^9, 3.579615047102148*^9}, {
  3.579616258828965*^9, 3.579616261290764*^9}, {3.579616516390966*^9, 
  3.5796165207497272`*^9}, {3.579699622602871*^9, 3.579699642014464*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CircumTest", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "data", "}"}], ",", 
    RowBox[{
     RowBox[{"data", " ", "=", " ", 
      RowBox[{"CircumData", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"CircumEval", "[", 
        RowBox[{"data", ",", "#"}], "]"}], "&"}], " ", "/@", " ", "vts"}]}]}],
    "]"}]}]], "Input",
 CellChangeTimes->{{3.57961631314327*^9, 3.579616343707999*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CircumTestArray", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CircumTest", "[", 
    RowBox[{"Array", "[", 
     RowBox[{"a", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"n", "+", "1"}], ",", "n"}], "}"}]}], "]"}], "]"}], " ", "//",
    " ", "Factor"}]}]], "Input",
 CellChangeTimes->{{3.579615746955818*^9, 3.5796157729018497`*^9}, {
  3.579615860615044*^9, 3.579615907777042*^9}, {3.579616215220326*^9, 
  3.5796162180919228`*^9}, {3.579616347831851*^9, 3.579616373507783*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Assumes", " ", "that", " ", "the", " ", "faces", " ", "are", " ", 
   "already", " ", "in", " ", "canonical", " ", "form"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.581395290593731*^9, 3.581395298160865*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNDOutputFromSimplexes", "[", "hullfcs_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hullvts", ",", "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hullvts", " ", "=", " ", 
      RowBox[{"Union", "[", 
       RowBox[{"Flatten", "[", "hullfcs", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"hullvts", ",", 
        RowBox[{"Sort", "[", "hullfcs", "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", "hull"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.577166370890436*^9, 3.5771664077296467`*^9}, {
  3.5788859733688507`*^9, 3.57888598451553*^9}, {3.578886160005533*^9, 
  3.57888616257943*^9}, {3.581395215993285*^9, 3.581395268074741*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["ND Spherical", "Subsection",
 CellChangeTimes->{{3.579095154054826*^9, 3.579095155943638*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolume", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"-", " ", 
   RowBox[{"Det", "[", "vts", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579095164768448*^9, 3.579095182170076*^9}, {
  3.5816377244209843`*^9, 3.581637724654861*^9}, {3.581918218155867*^9, 
  3.58191821836458*^9}, {3.581918280510251*^9, 3.5819182810436897`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexOrient", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SphSimplexVolume", "[", "vts", "]"}], " ", ">", " ", 
   "0"}]}]], "Input",
 CellChangeTimes->{{3.5790952109649467`*^9, 3.579095216187646*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"For", " ", "calculating", " ", "several", " ", "volumes"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579095184841638*^9, 3.579095197474828*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolData", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"-", " ", 
   RowBox[{"(", 
    RowBox[{"Cross", " ", "@@", " ", "vts"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.579095226743579*^9, 3.579095234618321*^9}, {
  3.579146759035008*^9, 3.579146759984922*^9}, {3.581637738832226*^9, 
  3.581637740927638*^9}, {3.581918222403776*^9, 3.581918222619463*^9}, {
  3.581918285980878*^9, 3.5819182862277527`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"data", ".", "vt"}]}]], "Input",
 CellChangeTimes->{{3.5790952555983057`*^9, 3.579095270299588*^9}, {
  3.5791467473461533`*^9, 3.579146755377219*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexOrientEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SphSimplexVolEval", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", ">", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.579095282662785*^9, 3.579095298915812*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Verification", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.581916493467774*^9, 3.581916496071574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolDataTest", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SphSimplexVolEval", "[", 
    RowBox[{
     RowBox[{"SphSimplexVolData", "[", 
      RowBox[{"Most", "[", "vts", "]"}], "]"}], ",", 
     RowBox[{"Last", "[", "vts", "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"SphSimplexVolume", "[", "vts", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.579345057803644*^9, 3.579345108573387*^9}, {
  3.5793477983529787`*^9, 3.579347826511345*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolDataTestArray", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SphSimplexVolDataTest", "[", 
    RowBox[{"Array", "[", 
     RowBox[{"a", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"n", "+", "1"}], ",", 
        RowBox[{"n", "+", "1"}]}], "}"}]}], "]"}], "]"}], " ", "//", " ", 
   "Factor"}]}]], "Input",
 CellChangeTimes->{{3.5793451111798687`*^9, 3.579345172683342*^9}, {
  3.579347804761333*^9, 3.579347817127388*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphSimplexVolDataTestArrSet", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Array", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SphSimplexVolDataTestArray", "[", 
      RowBox[{"a", ",", "#"}], "]"}], "&"}], ",", "n"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579345149435354*^9, 3.579345192163514*^9}, {
  3.579347807432255*^9, 3.579347815679371*^9}, {3.579347880483745*^9, 
  3.5793478837756033`*^9}, {3.581916409595655*^9, 3.581916443043983*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Circumscribed", " ", 
   RowBox[{
    RowBox[{"circle", "/", "sphere"}], "/", "hypersphere"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphCircumData", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"LinearSolve", "[", 
   RowBox[{"vts", ",", 
    RowBox[{"ConstantArray", "[", 
     RowBox[{"1", ",", 
      RowBox[{"Length", "[", "vts", "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579614898380863*^9, 3.579614908987775*^9}, {
  3.579615178649371*^9, 3.579615190039318*^9}, {3.579615779809709*^9, 
  3.5796158529758863`*^9}, {3.579615950796104*^9, 3.579616006617489*^9}, {
  3.579616129541875*^9, 3.5796161746756496`*^9}, {3.57961645772209*^9, 
  3.579616488637558*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphCircumEval", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"vt", ".", "data"}]}]], "Input",
 CellChangeTimes->{{3.579614910927075*^9, 3.5796149622940607`*^9}, {
   3.579615002412698*^9, 3.579615012919197*^9}, {3.579615050769951*^9, 
   3.579615053927165*^9}, {3.579615127155712*^9, 3.579615159862734*^9}, 
   3.579616462319816*^9, {3.5796164976703176`*^9, 3.5796164989086103`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphCircumInside", "[", 
   RowBox[{"data_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"SphCircumEval", "[", 
    RowBox[{"data", ",", "vt"}], "]"}], " ", "\[GreaterEqual]", " ", 
   "1"}]}]], "Input",
 CellChangeTimes->{{3.579615015756401*^9, 3.579615047102148*^9}, {
   3.579616258828965*^9, 3.579616261290764*^9}, 3.5796164632792807`*^9, {
   3.57961650553553*^9, 3.579616530829479*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphCircumTest", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "data", "}"}], ",", 
     RowBox[{
      RowBox[{"data", " ", "=", " ", 
       RowBox[{"SphCircumData", "[", "vts", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"SphCircumEval", "[", 
         RowBox[{"data", ",", "#"}], "]"}], "&"}], " ", "/@", " ", 
       "vts"}]}]}], "]"}], " ", "-", " ", "1"}]}]], "Input",
 CellChangeTimes->{{3.57961631314327*^9, 3.579616343707999*^9}, 
   3.579616464812295*^9, {3.579616538681087*^9, 3.57961654169044*^9}, {
   3.579619044988151*^9, 3.579619045656701*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphCircumTestArray", "[", 
   RowBox[{"a_", ",", "n_"}], "]"}], " ", ":=", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"SphCircumTest", "[", 
    RowBox[{"Array", "[", 
     RowBox[{"a", ",", 
      RowBox[{"{", 
       RowBox[{"n", ",", "n"}], "}"}]}], "]"}], "]"}], " ", "//", " ", 
   "Factor"}]}]], "Input",
 CellChangeTimes->{{3.579615746955818*^9, 3.5796157729018497`*^9}, {
   3.579615860615044*^9, 3.579615907777042*^9}, {3.579616215220326*^9, 
   3.5796162180919228`*^9}, {3.579616347831851*^9, 3.579616373507783*^9}, 
   3.579616466071912*^9, {3.5796165442644157`*^9, 3.579616549285861*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Verification and Comparison", "Subsection",
 CellChangeTimes->{{3.581545504108482*^9, 3.5815455080691347`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MultiResults", "[", 
   RowBox[{"algs_", ",", "vts_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "alg", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Prepend", "[", 
       RowBox[{
        RowBox[{"Timing", "[", 
         RowBox[{"alg", "[", "vts", "]"}], "]"}], ",", 
        RowBox[{"ToString", "[", "alg", "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"alg", ",", "algs"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.581569947494267*^9, 3.581569971246338*^9}, {
  3.581570016757779*^9, 3.581570057920858*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MRTimes", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"MatrixForm", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{"x", ",", "2"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xlst"}], "}"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58157676469284*^9, 3.581576864558556*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MRCount", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"Length", "[", 
   RowBox[{"Union", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"x", "[", 
       RowBox[{"[", "3", "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "xlst"}], "}"}]}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5815769096492147`*^9, 3.581576919617096*^9}, {
  3.5815769588355*^9, 3.581576963357353*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ResTimingCount", "[", 
   RowBox[{"algs_", ",", "vts_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MRTimes", "[", "#", "]"}], ",", 
      RowBox[{"MRCount", "[", "#", "]"}]}], "}"}], "&"}], " ", "@", " ", 
   RowBox[{"MultiResults", "[", 
    RowBox[{"algs", ",", "vts"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581577709763653*^9, 3.5815777880217533`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SmallResCounts", "[", 
   RowBox[{"algs_", ",", "nmax_", ",", "vts_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"MRCount", "[", 
     RowBox[{"MultiResults", "[", 
      RowBox[{"algs", ",", 
       RowBox[{"Take", "[", 
        RowBox[{"vts", ",", "k"}], "]"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"Min", "[", 
       RowBox[{"nmax", ",", 
        RowBox[{"Length", "[", "vts", "]"}]}], "]"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5815769737462997`*^9, 3.581577051564807*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Adjustments", " ", "for", " ", "Delaunay", " ", 
    RowBox[{"triangulation", " ", "--"}], " ", "use", " ", "only", " ", "the",
     " ", "first", " ", "two", " ", "outputs", " ", "for", " ", "array", " ", 
    "comparison"}], ",", " ", 
   RowBox[{
   "add", " ", "the", " ", "number", " ", "of", " ", "flips", " ", "to", " ", 
    "timing"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5820884959252787`*^9, 3.5820885060933123`*^9}, {
  3.582088733452071*^9, 3.58208876387262*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTGetFlips", "[", "dtres_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "dtres", "]"}], " ", "\[GreaterEqual]", " ", 
     "4"}], ",", " ", 
    RowBox[{"dtres", "[", 
     RowBox[{"[", "4", "]"}], "]"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582088595454286*^9, 3.582088623087409*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTMRTimes", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"MatrixForm", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Append", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"x", ",", "2"}], "]"}], ",", 
       RowBox[{"DTGetFlips", "[", 
        RowBox[{"x", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xlst"}], "}"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58157676469284*^9, 3.581576864558556*^9}, {
  3.582088637377832*^9, 3.582088680488645*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTMRCount", "[", "xlst_", "]"}], " ", ":=", " ", 
  RowBox[{"Length", "[", 
   RowBox[{"Union", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Take", "[", 
       RowBox[{
        RowBox[{"x", "[", 
         RowBox[{"[", "3", "]"}], "]"}], ",", "2"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "xlst"}], "}"}]}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5815769096492147`*^9, 3.581576919617096*^9}, {
  3.5815769588355*^9, 3.581576963357353*^9}, {3.5820887119420033`*^9, 
  3.582088721889681*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTResTimingCount", "[", 
   RowBox[{"algs_", ",", "vts_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"DTMRTimes", "[", "#", "]"}], ",", 
      RowBox[{"DTMRCount", "[", "#", "]"}]}], "}"}], "&"}], " ", "@", " ", 
   RowBox[{"MultiResults", "[", 
    RowBox[{"algs", ",", "vts"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581577709763653*^9, 3.5815777880217533`*^9}, {
  3.582088512320743*^9, 3.5820885126466093`*^9}, {3.582088690611046*^9, 
  3.582088691117244*^9}, {3.582088726587582*^9, 3.582088726914504*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSmallResCounts", "[", 
   RowBox[{"algs_", ",", "nmax_", ",", "vts_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"DTMRCount", "[", 
     RowBox[{"MultiResults", "[", 
      RowBox[{"algs", ",", 
       RowBox[{"Take", "[", 
        RowBox[{"vts", ",", "k"}], "]"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"Min", "[", 
       RowBox[{"nmax", ",", 
        RowBox[{"Length", "[", "vts", "]"}]}], "]"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5815769737462997`*^9, 3.581577051564807*^9}, {
  3.582088777893663*^9, 3.582088781794096*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Do", " ", "the", " ", "spherical", " ", "ones", " ", "have", " ", "the", 
    " ", "flat", " ", "ones", " ", "as", " ", "their", " ", "flat", " ", 
    RowBox[{
    "limits", "?", "\[IndentingNewLine]", "\[IndentingNewLine]", "To"}], " ", 
    "get", " ", "to", " ", "that", " ", "limit"}], ",", " ", 
   RowBox[{"make", " ", "the", " ", "vertices", " ", 
    RowBox[{"small", ".", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Args", ":", " ", 
      RowBox[{"the", " ", "vertices"}]}]}]}], ",", " ", 
   RowBox[{"which", " ", "direction", " ", "to", " ", "extend"}], ",", " ", 
   RowBox[{"the", " ", "spherical", " ", "CH"}], ",", " ", 
   RowBox[{"and", " ", "the", " ", "flat", " ", "CH"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.581581748740342*^9, 3.581581769626786*^9}, {
  3.581581899600889*^9, 3.5815819108166647`*^9}, {3.581582246262095*^9, 
  3.581582260131562*^9}, {3.581686890052603*^9, 3.581686901316806*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphericalFlatLimit", "[", 
   RowBox[{"vts_", ",", "ixoff_", ",", "CHSph_", ",", "CHFlat_"}], "]"}], " ",
   ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nvts", ",", "ndims", ",", "i", ",", "vtssph", ",", "vtsflat"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nvts", ",", "ndims"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vtssph", " ", "=", " ", 
      RowBox[{"Normalize", " ", "/@", " ", 
       RowBox[{"DoOffset", "[", 
        RowBox[{"vts", ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"i", "\[Equal]", "ixoff"}], ",", "1", ",", "0"}], "]"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "ndims"}], "}"}]}], "]"}]}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vtsflat", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"Delete", "[", 
         RowBox[{"#", ",", "ixoff"}], "]"}], "&"}], " ", "/@", " ", 
       "vtssph"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"CHSph", "[", "vtssph", "]"}], ",", 
       RowBox[{"CHFlat", "[", "vtsflat", "]"}]}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5815818809949636`*^9, 3.581582038078302*^9}, {
  3.5816375242127867`*^9, 3.581637530690359*^9}, {3.581663592657672*^9, 
  3.581663605324914*^9}, {3.5816867719810753`*^9, 3.5816868188327827`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Section",
 CellChangeTimes->{{3.575709869445517*^9, 3.5757098738427153`*^9}, 
   3.576158040464209*^9}],

Cell[CellGroupData[{

Cell["2D", "Subsection",
 CellChangeTimes->{{3.576158033426174*^9, 3.5761580338115587`*^9}, {
   3.576158208297756*^9, 3.576158209905963*^9}, {3.576158384598813*^9, 
   3.576158385620206*^9}, {3.57904391597789*^9, 3.579043917801429*^9}, {
   3.579102077944667*^9, 3.579102083688648*^9}, {3.579102188010131*^9, 
   3.57910220471154*^9}, 3.579141128011633*^9}],

Cell[CellGroupData[{

Cell["Graham Scan", "Subsubsection",
 CellChangeTimes->{{3.575720381517242*^9, 3.5757203834341793`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2GrahamScan", "[", "vts_", "]"}], " ", ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "ix0", ",", " ", "vt0", ",", " ", "vt", ",", " ", "ixvts", ",", " ", 
      "hull", ",", " ", "ptkeep", ",", " ", "ncnc", ",", " ", "k"}], "}"}], 
    ",", "\n", "  ", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\n", "  ", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vts", "]"}], " ", "<=", " ", "0"}], ",", "\n",
        "   ", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "}"}], "]"}]}], "\n", "   ", "]"}], ";", "\n", "  ", 
     RowBox[{"(*", " ", 
      RowBox[{"First", " ", "point"}], " ", "*)"}], "\n", "  ", 
     RowBox[{"ix0", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", "\n", "  ", 
     RowBox[{"(*", " ", 
      RowBox[{"All", " ", "the", " ", "others"}], " ", "*)"}], "\n", "  ", 
     RowBox[{"vt0", " ", "=", " ", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "ix0", "]"}], "]"}]}], ";", "\n", "  ", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"AddIndices", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"vt", " ", "-", " ", "vt0"}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"vt", ",", " ", "vts"}], "}"}]}], "]"}], "]"}]}], ";", "\n",
      "  ", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"ixvts", ",", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], " ", "!=", " ", "ix0"}], " ", 
         "&"}]}], "]"}]}], ";", "\n", "  ", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"ixvts", ",", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"Cross2D", "[", 
           RowBox[{
            RowBox[{"#1", "[", 
             RowBox[{"[", "2", "]"}], "]"}], ",", " ", 
            RowBox[{"#2", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}], " ", ">=", " ", "0"}], 
         " ", "&"}]}], "]"}]}], ";", "\n", "  ", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"Prepend", "[", 
       RowBox[{
        RowBox[{"First", " ", "/@", " ", "ixvts"}], ",", " ", "ix0"}], 
       "]"}]}], ";", "\n", "  ", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "Remove", " ", "concavities", " ", "with", " ", "the", " ", "three"}], 
       "-", 
       RowBox[{"penny", " ", "algorithm"}]}], " ", "*)"}], "\n", "  ", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\n", "   ", 
       RowBox[{
        RowBox[{"ptkeep", " ", "=", " ", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"True", ",", " ", 
           RowBox[{"Length", "[", "hull", "]"}]}], "]"}]}], ";", "\n", "   ", 
        
        RowBox[{"ncnc", " ", "=", " ", "0"}], ";", "\n", "   ", 
        RowBox[{"Do", "[", "\n", "    ", 
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", " ", 
              RowBox[{"ptkeep", "[", 
               RowBox[{"[", 
                RowBox[{"k", " ", "-", " ", "1"}], "]"}], "]"}]}], ",", " ", 
             RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\n", "    ", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", " ", 
              RowBox[{"TriOrient2D", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", 
                 RowBox[{"Take", "[", 
                  RowBox[{"hull", ",", " ", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"k", " ", "-", " ", "1"}], ",", " ", 
                    RowBox[{"k", " ", "+", " ", "1"}]}], "}"}]}], "]"}], 
                 "]"}], "]"}], "]"}]}], ",", "\n", "     ", 
             RowBox[{
              RowBox[{"ncnc", " ", "+=", " ", "1"}], ";", "\n", "     ", 
              RowBox[{
               RowBox[{"ptkeep", "[", 
                RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "False"}]}]}], 
            "]"}]}], ",", "\n", "    ", 
          RowBox[{"{", 
           RowBox[{"k", ",", " ", "2", ",", " ", 
            RowBox[{
             RowBox[{"Length", "[", "hull", "]"}], " ", "-", " ", "1"}]}], 
           "}"}]}], "]"}], ";", "\n", "   ", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"ncnc", " ", "==", " ", "0"}], ",", " ", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\n", "   ", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"SelWithList", "[", 
          RowBox[{"ptkeep", ",", " ", "hull"}], "]"}]}], ";"}]}], "\n", "   ",
       "]"}], ";", "\n", "  ", 
     RowBox[{"RotateToMin", "[", "hull", "]"}]}]}], "\n", "  ", 
   "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJwdx08oQwEAx/E3aVFKTbJY2kZzkZTMYZS3puaCsWYXMmYcViitmMZKyoHm
sKxYWm01iiazaCNkGLWWxhRhWQ5v2fJvS7t4v3f49ukrGBzrHs4jCIJHBwMS
Bcl3UuQTpZPCNMtgFdIWkXIbXCsmfHAy5WeUURk/PIyfBqC41huD7tj3A8zc
1i9X0ZYvZa3QoH+zw/CsywsTgdYY3DE7HqHeSL3AoPyXcVWlScCrRfYHtA2c
satpkxXuQvhqt3BgkzgngO6yEhHc4GvFMGzOdcHGfr8SjrtCGnjQV6iFHtn9
CFTnayYgSzo6DZ3PRkZdw4wJVkYjc/Dnj7UPs+oCH+QRnCA0fUovYE04HWVe
t30HVUebEhFt5N3TDB2W3hPIvSkNQiJl32ujXY/Pe6Fil9ve46JIvtXI2DJ0
3AETyalOqFV/KeH1JaGCIYtkpW6LIheE54z/93PmMw==
  "]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gift Wrapping", "Subsubsection",
 CellChangeTimes->{{3.575720390539815*^9, 3.575720398193655*^9}, 
   3.576169985403813*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Returns", " ", "whether", " ", "it", " ", "was", " ", "completed"}], ",", 
   " ", 
   RowBox[{"convex", " ", "hull", " ", "that", " ", "was", " ", "found"}]}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576158620950886*^9, 3.576158641880077*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2PartialGiftWrapping", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"indef_:", "True"}], ",", 
    RowBox[{"nmax_:", "0"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ix0", ",", "ixprev", ",", "ixcurr", ",", "ixvts", ",", 
      "ixvts1", ",", "vt", ",", "prevdir", ",", "ixnext", ",", "res", ",", 
      "iscomplete", ",", "n", ",", "hull"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"True", ",", 
          RowBox[{"Range", "[", "nvts", "]"}]}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"First", " ", "point"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ix0", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixprev", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixcurr", " ", "=", " ", "ix0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"All", " ", "the", " ", "others"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"AddIndices", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"iscomplete", " ", "=", " ", "False"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"n", "=", "0"}], ",", " ", 
         RowBox[{"indef", " ", "||", " ", 
          RowBox[{"(", 
           RowBox[{"n", "<", "nmax"}], ")"}]}], ",", 
         RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vt", " ", "=", " ", 
           RowBox[{"vts", "[", 
            RowBox[{"[", "ixcurr", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"prevdir", " ", "=", " ", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"ixprev", " ", "===", " ", "Null"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", 
               RowBox[{"-", "1"}]}], "}"}], ",", 
             RowBox[{"vt", " ", "-", " ", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "ixprev", "]"}], "]"}]}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{"Select", "[", 
            RowBox[{"ixvts", ",", 
             RowBox[{
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "ixcurr"}], 
              "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"prevdir", ".", 
                RowBox[{"Normalize", "[", 
                 RowBox[{
                  RowBox[{"#", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], "-", "vt"}], "]"}]}]}], 
              "}"}], "&"}], " ", "/@", " ", "ixvts1"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"ixnext", " ", "=", " ", 
           RowBox[{"First", "[", 
            RowBox[{"MaxBySortByFunc", "[", 
             RowBox[{"ixvts1", ",", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"ixnext", " ", "\[Equal]", " ", "ix0"}], ",", 
            RowBox[{
             RowBox[{"iscomplete", " ", "=", " ", "True"}], ";", 
             RowBox[{"Break", "[", "]"}]}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Sow", "[", "ixnext", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"ixprev", " ", "=", " ", "ixcurr"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"ixcurr", " ", "=", " ", "ixnext"}]}]}], 
        "\[IndentingNewLine]", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"Prepend", "[", 
       RowBox[{
        RowBox[{"res", "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "ix0"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"iscomplete", ",", "hull"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575720498126462*^9, 3.5757205570937023`*^9}, {
   3.575720605641973*^9, 3.575720645496376*^9}, {3.575720914152211*^9, 
   3.575720917932914*^9}, {3.5757210130927773`*^9, 3.575721108303186*^9}, {
   3.575721440421507*^9, 3.575721441178195*^9}, {3.5757215787969017`*^9, 
   3.575721744625984*^9}, {3.57572178808374*^9, 3.575721830252256*^9}, {
   3.575721879328432*^9, 3.575721934384519*^9}, {3.575721990730645*^9, 
   3.575721992519876*^9}, {3.575722065872141*^9, 3.575722169507578*^9}, {
   3.5757222288477583`*^9, 3.575722264022147*^9}, {3.575722308391553*^9, 
   3.575722423017088*^9}, {3.575722491385768*^9, 3.575722519864481*^9}, {
   3.5757225510738697`*^9, 3.575722578269298*^9}, {3.575722627596998*^9, 
   3.575722636747714*^9}, {3.575722685948593*^9, 3.575722697045492*^9}, {
   3.575722742525362*^9, 3.575722746680512*^9}, {3.575722800705459*^9, 
   3.575722810078446*^9}, {3.5757229299790993`*^9, 3.5757229356623487`*^9}, {
   3.575722966335867*^9, 3.575722992478039*^9}, {3.575723149849135*^9, 
   3.575723223648882*^9}, {3.575723263447886*^9, 3.575723350825279*^9}, {
   3.575723408697975*^9, 3.5757234113616533`*^9}, {3.575723579756805*^9, 
   3.575723601315377*^9}, {3.575723658200938*^9, 3.575723658628837*^9}, 
   3.575787240295793*^9, {3.575984141882782*^9, 3.575984233246285*^9}, 
   3.57614643238673*^9, {3.5761468354144897`*^9, 3.5761468428249063`*^9}, {
   3.576158408592629*^9, 3.576158450359576*^9}, {3.5761585025138273`*^9, 
   3.576158520163858*^9}, {3.576158611563833*^9, 3.576158616264432*^9}, {
   3.576431527757347*^9, 3.5764315386348763`*^9}, {3.577032816903575*^9, 
   3.577032898316066*^9}, {3.577032936441354*^9, 3.57703299102623*^9}, {
   3.577033021799588*^9, 3.577033022181904*^9}, {3.5770330623222847`*^9, 
   3.577033194835767*^9}, {3.577033289969062*^9, 3.577033300567684*^9}, {
   3.577033421551043*^9, 3.5770334263188763`*^9}, {3.577033457917357*^9, 
   3.57703350098311*^9}, {3.578885284539089*^9, 3.578885292343277*^9}, {
   3.581372817096334*^9, 3.581372874070697*^9}, {3.58137299876198*^9, 
   3.581373004351927*^9}, {3.581373064330934*^9, 3.581373067928298*^9}, {
   3.581373128273856*^9, 3.581373143451509*^9}, {3.58137322067128*^9, 
   3.581373264140091*^9}, {3.581373451454479*^9, 3.581373453783557*^9}, {
   3.5813734898483887`*^9, 3.581373568170331*^9}, {3.581373629609536*^9, 
   3.581373637918605*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2GiftWrapping", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"iscomplete", ",", " ", "hull"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"iscomplete", ",", " ", "hull"}], "}"}], " ", "=", " ", 
      RowBox[{"CH2PartialGiftWrapping", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"iscomplete", ",", 
       RowBox[{"RotateToMin", "[", "hull", "]"}], ",", 
       RowBox[{"{", "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575720498126462*^9, 3.5757205570937023`*^9}, {
   3.575720605641973*^9, 3.575720645496376*^9}, {3.575720914152211*^9, 
   3.575720917932914*^9}, {3.5757210130927773`*^9, 3.575721108303186*^9}, {
   3.575721440421507*^9, 3.575721441178195*^9}, {3.5757215787969017`*^9, 
   3.575721744625984*^9}, {3.57572178808374*^9, 3.575721830252256*^9}, {
   3.575721879328432*^9, 3.575721934384519*^9}, {3.575721990730645*^9, 
   3.575721992519876*^9}, {3.575722065872141*^9, 3.575722169507578*^9}, {
   3.5757222288477583`*^9, 3.575722264022147*^9}, {3.575722308391553*^9, 
   3.575722423017088*^9}, {3.575722491385768*^9, 3.575722519864481*^9}, {
   3.5757225510738697`*^9, 3.575722578269298*^9}, {3.575722627596998*^9, 
   3.575722636747714*^9}, {3.575722685948593*^9, 3.575722697045492*^9}, {
   3.575722742525362*^9, 3.575722746680512*^9}, {3.575722800705459*^9, 
   3.575722810078446*^9}, {3.5757229299790993`*^9, 3.5757229356623487`*^9}, {
   3.575722966335867*^9, 3.575722992478039*^9}, {3.575723149849135*^9, 
   3.575723223648882*^9}, {3.575723263447886*^9, 3.575723350825279*^9}, {
   3.575723408697975*^9, 3.5757234113616533`*^9}, {3.575723579756805*^9, 
   3.575723601315377*^9}, {3.575723658200938*^9, 3.575723658628837*^9}, 
   3.575787240295793*^9, {3.57614641885188*^9, 3.576146420293828*^9}, {
   3.5761468646853333`*^9, 3.576146866229311*^9}, {3.576158539321827*^9, 
   3.5761585482176867`*^9}, {3.578885313870903*^9, 3.578885386685997*^9}, {
   3.581372888063819*^9, 3.581372889137542*^9}, 3.5813731760508337`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Divide and Conquer", "Subsubsection",
 CellChangeTimes->{{3.5757481164782677`*^9, 3.575748120636183*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Goes", " ", "in", " ", "increasing", " ", "order", " ", "on", " ", "the", 
   " ", "first", " ", "hull", " ", "and", " ", "decreasing", " ", "order", 
   " ", "on", " ", "the", " ", "second", " ", "hull"}], " ", "*)"}]], "Input",\

 CellChangeTimes->{{3.576311071594057*^9, 3.576311087430151*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2TwoHullTangent", "[", 
   RowBox[{"hlvts1_", ",", "hlvts2_", ",", "ix10_", ",", "ix20_"}], "]"}], 
  " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "ix1", ",", "ix2", ",", "hlen1", ",", "hlen2", ",", "numadvs", ",", 
      "vt0", ",", "vt1", ",", "vt2", ",", "ixnx"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ix1", " ", "=", " ", "ix10"}], ";", " ", 
     RowBox[{"ix2", " ", "=", " ", "ix20"}], ";", "\[IndentingNewLine]", 
     RowBox[{"hlen1", " ", "=", " ", 
      RowBox[{"Length", "[", "hlvts1", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hlen2", " ", "=", " ", 
      RowBox[{"Length", "[", "hlvts2", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"numadvs", " ", "=", " ", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"vt0", " ", "=", " ", 
         RowBox[{"hlvts2", "[", 
          RowBox[{"[", "ix2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"vt1", " ", "=", " ", 
         RowBox[{"hlvts1", "[", 
          RowBox[{"[", "ix1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ixnx", " ", "=", " ", 
         RowBox[{"ix1", " ", "+", " ", "1"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"ixnx", " ", ">", " ", "hlen1"}], ",", 
          RowBox[{"ixnx", " ", "=", " ", "1"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vt2", " ", "=", " ", 
         RowBox[{"hlvts1", "[", 
          RowBox[{"[", "ixnx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"TriOrient2D", "[", 
           RowBox[{"{", 
            RowBox[{"vt0", ",", "vt2", ",", "vt1"}], "}"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ix1", " ", "=", " ", "ixnx"}], ";", "\[IndentingNewLine]", 
           RowBox[{"numadvs", " ", "+=", " ", "1"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"vt0", " ", "=", " ", 
         RowBox[{"hlvts1", "[", 
          RowBox[{"[", "ix1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"vt1", " ", "=", " ", 
         RowBox[{"hlvts2", "[", 
          RowBox[{"[", "ix2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ixnx", " ", "=", " ", 
         RowBox[{"ix2", " ", "-", " ", "1"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"ixnx", " ", "<", "1"}], ",", " ", 
          RowBox[{"ixnx", " ", "=", " ", "hlen2"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vt2", " ", "=", " ", 
         RowBox[{"hlvts2", "[", 
          RowBox[{"[", "ixnx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"TriOrient2D", "[", 
           RowBox[{"{", 
            RowBox[{"vt0", ",", "vt1", ",", "vt2"}], "}"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ix2", " ", "=", " ", "ixnx"}], ";", "\[IndentingNewLine]", 
           RowBox[{"numadvs", " ", "+=", " ", "1"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"numadvs", " ", "\[Equal]", " ", "0"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"ix1", ",", "ix2"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576310560906768*^9, 3.576310578570344*^9}, {
  3.5763106144582863`*^9, 3.5763106339229937`*^9}, {3.576310688575568*^9, 
  3.5763106955277653`*^9}, {3.576310873929102*^9, 3.576310914594962*^9}, {
  3.576310963054616*^9, 3.576311046541642*^9}, {3.576337956736011*^9, 
  3.576337994182428*^9}, {3.576338054841659*^9, 3.576338113549617*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Recursive", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575786636696672*^9, 3.575786639748083*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2DivideConquer", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "hull", ",", "ixvts", ",", "nspl", ",", "ixvts1", ",", 
      "ixvts2", ",", "ixs1", ",", "ixs2", ",", "vts1", ",", "vts2", ",", 
      "hull1", ",", "hull2", ",", "k", ",", "ixnr1", ",", "ixnr2", ",", 
      "ix11", ",", "ix12", ",", "ix21", ",", "ix22"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{"For", " ", "at", " ", "most", " ", "3", " ", "points"}], ",", 
      " ", 
      RowBox[{"find", " ", "the", " ", "hull", " ", "by", " ", "hand"}]}], 
     " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "3"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"nvts", " ", "\[Equal]", " ", "3"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"TriOrient2D", "[", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "hull", "]"}], "]"}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"hull", " ", "=", " ", 
             RowBox[{"{", 
              RowBox[{"1", ",", "3", ",", "2"}], "}"}]}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "hulls", " ", "of", " ", "the", " ", "two", 
       " ", "parts"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "Rotate", " ", "the", " ", "splitting", " ", "axis", " ", "for", " ", 
        "calculating", " ", "the", " ", "child", " ", "hulls"}], ",", " ", 
       RowBox[{"while", " ", "doing", " ", "a", " ", "reflection", " ", 
        RowBox[{"afterwards", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"AddIndices", "[", "vts", "]"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nspl", " ", "=", " ", 
      RowBox[{"Floor", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nvts", "+", "1"}], ")"}], "/", "2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts2", " ", "=", " ", 
      RowBox[{"Drop", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixs1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixs2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"CH2DivideConquer", "[", 
       RowBox[{"RotateLeft", " ", "/@", " ", "vts1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"CH2DivideConquer", "[", 
       RowBox[{"RotateLeft", " ", "/@", " ", "vts2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"RotateToMin", "[", 
       RowBox[{"Reverse", "[", "hull1", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"RotateToMin", "[", 
       RowBox[{"Reverse", "[", "hull2", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "closest", " ", "points"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"hull1", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", 
          RowBox[{"Length", "[", "ixs1", "]"}]}], ",", "\[IndentingNewLine]", 
         
         RowBox[{
          RowBox[{"ixnr1", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "hull1", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"hull2", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ixnr2", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "hull2", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Get", " ", "the", " ", "hulls", " ", "into", " ", "the", " ", "input", 
       " ", 
       RowBox[{"points", "'"}], " ", "indexing"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"ixs1", "[", 
       RowBox[{"[", "hull1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"ixs2", "[", 
       RowBox[{"[", "hull2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "tangent", " ", "lines", " ", "between", " ", "the", " ", 
       "hulls"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ix11", ",", "ix12"}], "}"}], " ", "=", " ", 
      RowBox[{"CH2TwoHullTangent", "[", 
       RowBox[{
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull1", "]"}], "]"}], ",", 
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull2", "]"}], "]"}], ",", "ixnr1", ",", "ixnr2"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ix22", ",", "ix21"}], "}"}], " ", "=", " ", 
      RowBox[{"CH2TwoHullTangent", "[", 
       RowBox[{
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull2", "]"}], "]"}], ",", 
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull1", "]"}], "]"}], ",", "ixnr2", ",", "ixnr1"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Stitch", " ", "the", " ", "hulls", " ", "together"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "If", " ", "the", " ", "top", " ", "and", " ", "bottom", " ", 
        "tangents", " ", "meet", " ", "at", " ", "the", " ", "same", " ", 
        "point"}], ",", "\[IndentingNewLine]", 
       RowBox[{
       "then", " ", "the", " ", "hull", " ", "segment", " ", "has", " ", 
        "zero", " ", "size"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix21", " ", "\[GreaterEqual]", " ", "ix11"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull1", ",", 
          RowBox[{"{", 
           RowBox[{"ix11", ",", "ix21"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"ix11", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix21"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix12", " ", "\[GreaterEqual]", " ", "ix22"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull2", ",", 
          RowBox[{"{", 
           RowBox[{"ix22", ",", "ix12"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"ix22", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix12"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"RotateToMin", "[", 
      RowBox[{"Join", "[", 
       RowBox[{"hull1", ",", "hull2"}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.575748121984099*^9, 3.575748133075938*^9}, {
   3.57578655472513*^9, 3.575786582390957*^9}, {3.5757873538331614`*^9, 
   3.575787401202379*^9}, {3.5757874326641903`*^9, 3.575787572516796*^9}, {
   3.575787679197785*^9, 3.575787751425363*^9}, {3.575787781803688*^9, 
   3.57578780940387*^9}, {3.575789031535413*^9, 3.5757890806103077`*^9}, {
   3.575789116911085*^9, 3.575789169414853*^9}, {3.575789291096561*^9, 
   3.575789413102585*^9}, {3.575789451627865*^9, 3.5757894572107363`*^9}, 
   3.575789492296549*^9, {3.575789688365378*^9, 3.5757897889042053`*^9}, {
   3.5757898650243673`*^9, 3.575789935440854*^9}, {3.575790070571764*^9, 
   3.575790093043057*^9}, {3.575790232950604*^9, 3.575790262168594*^9}, {
   3.575790303137864*^9, 3.575790332287765*^9}, {3.5757903735315533`*^9, 
   3.5757905746520348`*^9}, {3.5757906138218317`*^9, 3.575790620810421*^9}, {
   3.5757906855835752`*^9, 3.575790760742723*^9}, {3.575791533901527*^9, 
   3.575791535251452*^9}, {3.575791864239854*^9, 3.575791922414309*^9}, {
   3.575792073008532*^9, 3.5757920862298737`*^9}, {3.575792776685885*^9, 
   3.575792821001733*^9}, {3.575796120286574*^9, 3.575796160283103*^9}, {
   3.575796219716041*^9, 3.575796241511891*^9}, {3.575796304978487*^9, 
   3.5757963300170317`*^9}, {3.575796950870241*^9, 3.575797045448537*^9}, {
   3.575837879280663*^9, 3.575837923251533*^9}, {3.575981221608276*^9, 
   3.575981221773098*^9}, {3.575981561927153*^9, 3.575981562089891*^9}, {
   3.5759816611929407`*^9, 3.575981676221401*^9}, {3.575981809651079*^9, 
   3.575981980924674*^9}, {3.575982032807292*^9, 3.57598206231157*^9}, {
   3.575982247266596*^9, 3.5759822613789463`*^9}, {3.575982338357884*^9, 
   3.5759823624439173`*^9}, {3.5759824552495527`*^9, 3.575982537612626*^9}, {
   3.575982576129842*^9, 3.575982601422359*^9}, {3.575982641406695*^9, 
   3.575982641973921*^9}, {3.576309460401182*^9, 3.576309471037458*^9}, {
   3.576309501766316*^9, 3.576309637330697*^9}, {3.576311098693799*^9, 
   3.576311159244645*^9}, {3.5763111996144447`*^9, 3.576311216615343*^9}, {
   3.576311415654913*^9, 3.576311421370924*^9}, {3.5763114561413403`*^9, 
   3.576311575078356*^9}, {3.5763116191219893`*^9, 3.576311669662118*^9}, {
   3.576338123004759*^9, 3.576338143561308*^9}, 3.5765061307554417`*^9, {
   3.577059116148549*^9, 3.577059154308537*^9}, {3.577270918407454*^9, 
   3.577270935943115*^9}, {3.577270990784835*^9, 3.577270996988584*^9}, {
   3.577272031886281*^9, 3.577272048025474*^9}, {3.578885406969839*^9, 
   3.5788854279589233`*^9}, {3.578885613162485*^9, 3.578885613814839*^9}, {
   3.582078792221448*^9, 3.5820788319588127`*^9}, {3.582078946396646*^9, 
   3.582078963276052*^9}, {3.5820790578151913`*^9, 3.5820791267001743`*^9}, {
   3.5820792408755817`*^9, 3.582079283660409*^9}, {3.582079319993956*^9, 
   3.582079348431094*^9}, {3.582079384236154*^9, 3.582079394744967*^9}, {
   3.5821615541249733`*^9, 3.582161577446279*^9}, {3.5821619434460907`*^9, 
   3.582161973497273*^9}, {3.5825412817369947`*^9, 3.582541282208829*^9}, {
   3.582541373132753*^9, 3.582541377045301*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Chan Shattering", "Subsubsection",
 CellChangeTimes->{{3.5759837336083813`*^9, 3.575983737997348*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Need", " ", "partial", " ", "gift", " ", "wrapping", " ", "for", " ", 
   "it"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575984089201826*^9, 3.5759840953612623`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GenChanShattering", "[", 
   RowBox[{
   "vts_", ",", "HullVertices_", ",", "CHParts_", ",", "PartGiftWrap_", ",", 
    "MapVertices_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "rng", ",", "nhtrial", ",", "nregs", ",", "seg", ",", 
      "segs", ",", "shathulls", ",", "res", ",", "hull"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"CHParts", "[", "vts", "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Loop", " ", "through", " ", "trial", " ", "lengths"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"nhtrial", " ", "=", " ", "3"}], ",", "True", ",", 
       RowBox[{"nhtrial", " ", "*=", " ", "3"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"nregs", " ", "=", " ", 
         RowBox[{"Round", "[", 
          RowBox[{"nvts", "/", "nhtrial"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"segs", " ", "=", " ", 
         RowBox[{"Round", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"nvts", "/", "nregs"}], ")"}], "*", 
           RowBox[{"Range", "[", 
            RowBox[{"0", ",", "nregs"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"segs", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Range", " ", "@@", " ", 
             RowBox[{"(", 
              RowBox[{"#", "+", 
               RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}]}], ")"}]}], ")"}], "&"}], " ", 
          "/@", " ", 
          RowBox[{"Partition", "[", 
           RowBox[{"segs", ",", "2", ",", "1"}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"shathulls", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"hull", " ", "=", " ", 
             RowBox[{"CHParts", "[", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "seg", "]"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"seg", "[", 
             RowBox[{"[", 
              RowBox[{"HullVertices", "[", "hull", "]"}], "]"}], "]"}]}], ",",
            "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"seg", ",", "segs"}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "shathulls", "]"}], "\[Equal]", "1"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"shathulls", " ", "=", " ", 
         RowBox[{"Flatten", "[", "shathulls", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"res", " ", "=", " ", 
         RowBox[{"PartGiftWrap", "[", 
          RowBox[{
           RowBox[{"vts", "[", 
            RowBox[{"[", "shathulls", "]"}], "]"}], ",", "False", ",", 
           "nhtrial"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"res", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"hull", " ", "=", " ", 
            RowBox[{"MapVertices", "[", 
             RowBox[{"shathulls", ",", 
              RowBox[{"res", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "hull"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5759842392465878`*^9, 3.5759842477813396`*^9}, {
   3.575984279228026*^9, 3.575984333962162*^9}, 3.575984431899867*^9, {
   3.575984655970749*^9, 3.575984665182169*^9}, {3.575984995790472*^9, 
   3.5759850034720078`*^9}, {3.5759852012882967`*^9, 3.575985241503394*^9}, {
   3.575985354491205*^9, 3.5759853775697107`*^9}, {3.575988106153295*^9, 
   3.5759881403338213`*^9}, {3.575988421041587*^9, 3.57598843165917*^9}, {
   3.57598847417803*^9, 3.575988479097706*^9}, {3.575988512450953*^9, 
   3.575988516595394*^9}, {3.575988557901052*^9, 3.5759886223884068`*^9}, {
   3.575988693325954*^9, 3.575988745751175*^9}, {3.575988826804655*^9, 
   3.575988869280622*^9}, {3.5759889027476377`*^9, 3.575988951244072*^9}, {
   3.5759890007213707`*^9, 3.575989021656188*^9}, {3.5759891551852283`*^9, 
   3.575989166512979*^9}, {3.575989248763929*^9, 3.5759892543519897`*^9}, {
   3.575989507533321*^9, 3.5759895096431103`*^9}, {3.575989545765259*^9, 
   3.575989606813052*^9}, {3.57698509115254*^9, 3.576985105267116*^9}, {
   3.577032187758361*^9, 3.577032197768476*^9}, {3.577155867033648*^9, 
   3.577155912440123*^9}, {3.577155956180297*^9, 3.577156010818597*^9}, {
   3.577156269391323*^9, 3.577156279909171*^9}, {3.57715636084692*^9, 
   3.5771564485914097`*^9}, {3.5771566100042477`*^9, 3.577156618745656*^9}, {
   3.5771566514604473`*^9, 3.577156653296486*^9}, 3.577156766762269*^9, {
   3.577207993906454*^9, 3.5772080550840387`*^9}, {3.578887733275631*^9, 
   3.578887821882965*^9}, {3.578887992717383*^9, 3.578887996805252*^9}, 
   3.578888093979741*^9, {3.581372920243691*^9, 3.58137292096598*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2CSMapVertices", "[", 
   RowBox[{"shathulls_", ",", "hull_"}], "]"}], " ", ":=", " ", 
  RowBox[{"RotateToMin", "[", 
   RowBox[{"shathulls", "[", 
    RowBox[{"[", "hull", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.578887863777307*^9, 3.5788879108216457`*^9}, {
  3.57888924788697*^9, 3.578889259549775*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2ChanShattering", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"CHParts_:", "CH2GrahamScan"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"GenChanShattering", "[", 
   RowBox[{
   "vts", ",", "Identity", ",", "CHParts", ",", "CH2PartialGiftWrapping", ",",
     "CH2CSMapVertices"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577155899060878*^9, 3.5771559243873777`*^9}, {
   3.5771560441185093`*^9, 3.5771560496497726`*^9}, {3.578885513130163*^9, 
   3.578885517538021*^9}, {3.5788878462560587`*^9, 3.5788878502939796`*^9}, 
   3.5788879314592123`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["QuickHull", "Subsubsection",
 CellChangeTimes->{{3.576086576049953*^9, 3.5760865862963543`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", " ", "indices", " ", "of", " ", "positive"}], "-", 
   RowBox[{"side", " ", "vertices", " ", "and", " ", "the", " ", "maximum"}], 
   "-", 
   RowBox[{"distance", " ", "one"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576797278797093*^9, 3.5767972875907707`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2QHExternalVertices", "[", 
   RowBox[{"vts_", ",", "vtixs_", ",", "seg_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"tdval", ",", "ixvts", ",", "ixpos", ",", "ixmax"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{"ixvts", " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"#", ",", 
           RowBox[{"TriArea2D", "[", 
            RowBox[{"vts", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"seg", "[", 
                 RowBox[{"[", "2", "]"}], "]"}], ",", 
                RowBox[{"seg", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "]"}], 
             "]"}], "]"}]}], "}"}], "&"}], " ", "/@", " ", "vtixs"}]}], ";"}],
      " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"tdval", " ", "=", "  ", 
      RowBox[{"TriAreaData2D", "[", 
       RowBox[{"vts", "[", 
        RowBox[{"[", 
         RowBox[{"seg", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"2", ",", "1"}], "}"}], "]"}], "]"}], "]"}], "]"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"TriAreaEval2D", "[", 
           RowBox[{"tdval", ",", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "#", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
        " ", "vtixs"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixpos", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"Select", "[", 
        RowBox[{"ixvts", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ">", "0"}], "&"}]}], "]"}]}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixmax", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "ixvts", "]"}], " ", ">", " ", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "@", " ", 
         RowBox[{"MaxBySortByFunc", "[", 
          RowBox[{"ixvts", ",", 
           RowBox[{
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", "Null"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"ixpos", ",", "ixmax"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQAWIQrXrz7uovS145Oss17QDRCwwv7AHR09a57AXRIXye
B0F0wpmCUyD6TcmLcyA68Z/EeRDNFnMp7BuQvuUiHQGija5M5DNcBtQ3/y2Y
ttrpPtUISP8tewymJcrl5oFoto/f14LoN+nO+0G0nvbKQyD6guSRUyD69rI/
YPrJpsBLIPqEwx0wzbFz826HFa8cu1peg+kbnzv+gOh7i4+B6e1SR7kdgfSn
JB9BEH3sfp0oiN737Z0YmM66WQ2i/+2cWQ+iTR8+bQPRCXvSu0H0rKDDE0B0
Tm77JBB9+DPHLBDtv2c1mL7Rv2gNiA712wCmAULdto0=
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2QuickHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ix1", ",", "ix2", ",", "ixvts", ",", "tdval", ",", 
      "ixvtria", ",", "ivtng", ",", "ivtps", ",", "ixps", ",", "ixng", ",", 
      "taps", ",", "tang", ",", "ix3", ",", "hull", ",", "k", ",", "kn", ",", 
      "hlen", ",", "seg", ",", "extvts", ",", "kx", ",", "kxn", ",", "evsimx",
       ",", "evs", ",", "ixh", ",", "ixhn", ",", "ixmax", ",", "seg1", ",", 
      "seg2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"Range", "[", "nvts", "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Starting", " ", 
       RowBox[{"points", ":", " ", 
        RowBox[{"first", " ", "two"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ix1", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ix2", " ", "=", " ", 
      RowBox[{"IndexMax", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Starting", " ", 
       RowBox[{"points", ":", " ", 
        RowBox[{"third", " ", "one"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{
        RowBox[{"Range", "[", "nvts", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ix1", ",", "ix2"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"ixvtria", " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"#", ",", 
            RowBox[{"TriArea2D", "[", 
             RowBox[{"vts", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"ix1", ",", "ix2", ",", "#"}], "}"}], "]"}], "]"}], 
             "]"}]}], "}"}], "&"}], " ", "/@", " ", "ixvts"}]}], ";"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"tdval", " ", "=", " ", 
      RowBox[{"TriAreaData2D", "[", 
       RowBox[{"vts", "[", 
        RowBox[{"[", 
         RowBox[{"{", 
          RowBox[{"ix1", ",", "ix2"}], "}"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvtria", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"TriAreaEval2D", "[", 
           RowBox[{"tdval", ",", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "#", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
        " ", "ixvts"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ivtng", " ", "=", " ", 
      RowBox[{"MinBySortByFunc", "[", 
       RowBox[{"ixvtria", ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ivtps", " ", "=", " ", 
      RowBox[{"MaxBySortByFunc", "[", 
       RowBox[{"ixvtria", ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixng", " ", "=", " ", 
      RowBox[{"ivtng", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixps", " ", "=", " ", 
      RowBox[{"ivtps", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tang", " ", "=", " ", 
      RowBox[{"-", " ", 
       RowBox[{"ivtng", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"taps", " ", "=", " ", 
      RowBox[{"ivtps", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"taps", " ", "\[GreaterEqual]", " ", "tang"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ix3", " ", "=", " ", "ixps"}], ";", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"ix1", ",", "ix2", ",", "ix3"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ix3", " ", "=", " ", "ixng"}], ";", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"ix2", ",", "ix1", ",", "ix3"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{"ixvts", ",", 
        RowBox[{"{", "ix3", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "outward", " ", "vertices", " ", "for", " ", 
       "each", " ", "edge"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hlen", " ", "=", " ", 
      RowBox[{"Length", "[", "hull", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"kn", " ", "=", " ", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"seg", " ", "=", 
         RowBox[{"hull", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"k", ",", "kn"}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"extvts", " ", "[", "seg", "]"}], " ", "=", " ", 
         RowBox[{"CH2QHExternalVertices", "[", 
          RowBox[{"vts", ",", "ixvts", ",", "seg"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Search", " ", "for", " ", "an", " ", "edge", " ", "with", " ", 
         "outward", " ", "points"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"kxn", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"hlen", " ", "=", " ", 
         RowBox[{"Length", "[", "hull", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"kn", " ", "=", " ", 
            RowBox[{
             RowBox[{"Mod", "[", 
              RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"seg", " ", "=", " ", 
            RowBox[{"hull", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"k", ",", "kn"}], "}"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"evsimx", " ", "=", " ", 
            RowBox[{"extvts", "[", "seg", "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"evs", " ", "=", " ", 
            RowBox[{"evsimx", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"ixmax", " ", "=", " ", 
            RowBox[{"evsimx", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "evs", "]"}], " ", ">", " ", "0"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"kx", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"kxn", " ", "=", " ", "kn"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"kx", " ", "===", " ", "Null"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Find", " ", "a", " ", "point", " ", "on", " ", "the", " ", "hull", 
          " ", "from", " ", "it"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"ixh", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "kx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ixhn", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "kxn", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"evs", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"evs", ",", 
           RowBox[{"{", "ixmax", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"kxn", " ", "<", " ", "kx"}], ",", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"kxn", ",", "kx"}], "}"}]}], "]"}], ",", 
             RowBox[{"{", "ixmax", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"kxn", ",", 
                 RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", "kx"}], "}"}]}], "]"}], ",", 
             RowBox[{"{", "ixmax", "}"}]}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Find", " ", "the", " ", "outward", " ", "points", " ", "for", " ", 
          "the", " ", "new", " ", "hull", " ", "segments"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"seg1", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"ixh", ",", "ixmax"}], "}"}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"seg2", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"ixmax", ",", "ixhn"}], "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"extvts", "[", "seg1", "]"}], " ", "=", " ", 
         RowBox[{"CH2QHExternalVertices", "[", 
          RowBox[{"vts", ",", "evs", ",", "seg1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"extvts", "[", "seg2", "]"}], " ", "=", " ", 
         RowBox[{"CH2QHExternalVertices", "[", 
          RowBox[{"vts", ",", "evs", ",", "seg2"}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"RotateToMin", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5760871559453173`*^9, 3.576087163319709*^9}, {
   3.576087196106777*^9, 3.5760871999132032`*^9}, 3.576087366894826*^9, {
   3.576087593103817*^9, 3.576087596884666*^9}, {3.576087786862938*^9, 
   3.5760878099851713`*^9}, {3.5760987912215652`*^9, 3.576098801383127*^9}, {
   3.576099994921384*^9, 3.576100034178421*^9}, {3.576102444704525*^9, 
   3.576102462010558*^9}, {3.576106066219431*^9, 3.5761060848499947`*^9}, {
   3.576114271768013*^9, 3.576114648011324*^9}, {3.576145506155438*^9, 
   3.576145538932366*^9}, {3.576145576907448*^9, 3.576145807857596*^9}, {
   3.5761458455724497`*^9, 3.57614584872637*^9}, {3.576145890209508*^9, 
   3.57614595727253*^9}, {3.576146152864555*^9, 3.576146161254201*^9}, {
   3.576146213687154*^9, 3.576146235749601*^9}, {3.5761467893755074`*^9, 
   3.5761468140509653`*^9}, {3.576147301009488*^9, 3.576147314468515*^9}, {
   3.576147405930875*^9, 3.576147414819984*^9}, {3.576147458187669*^9, 
   3.576147461651425*^9}, {3.576147494061471*^9, 3.5761475405085783`*^9}, {
   3.57614757424796*^9, 3.576147678398075*^9}, {3.576147718635757*^9, 
   3.576147731340663*^9}, {3.576147765694377*^9, 3.5761477738067217`*^9}, {
   3.576147808928825*^9, 3.5761478449811087`*^9}, {3.5761478861183968`*^9, 
   3.57614837967557*^9}, {3.5761486167554092`*^9, 3.576148619226398*^9}, {
   3.5761487191799173`*^9, 3.57614884010264*^9}, {3.576148907531739*^9, 
   3.576148908220676*^9}, {3.5761489500546827`*^9, 3.576148952915557*^9}, {
   3.576149044818079*^9, 3.576149058818138*^9}, {3.576149889171176*^9, 
   3.576149907471566*^9}, {3.576155695915517*^9, 3.576155699306476*^9}, {
   3.576155732480452*^9, 3.576155746525326*^9}, {3.57615584083776*^9, 
   3.5761560216959887`*^9}, {3.5764037979578543`*^9, 3.576403897658107*^9}, {
   3.576403938976942*^9, 3.576403953354466*^9}, 3.576754182313003*^9, {
   3.57679655751556*^9, 3.576796602864517*^9}, {3.576796708892858*^9, 
   3.576796732426828*^9}, 3.576798604195736*^9, {3.576814873545368*^9, 
   3.576814889007862*^9}, {3.576985070275062*^9, 3.576985073757833*^9}, {
   3.576985230372758*^9, 3.5769852332826357`*^9}, {3.577873929140257*^9, 
   3.577874022137191*^9}, {3.5778740611012173`*^9, 3.577874062384556*^9}, {
   3.577874143667439*^9, 3.577874150091178*^9}, {3.5778741805162983`*^9, 
   3.5778741810970917`*^9}, {3.5778742440899763`*^9, 3.577874250988379*^9}, {
   3.5778742833306828`*^9, 3.5778743087448807`*^9}, {3.577874351731762*^9, 
   3.577874383903356*^9}, {3.5778744198210907`*^9, 3.577874471344408*^9}, {
   3.5778749034200573`*^9, 3.577874905539729*^9}, {3.577875380135881*^9, 
   3.577875416313779*^9}, {3.577875502983602*^9, 3.5778755039917307`*^9}, {
   3.57888554705659*^9, 3.578885552140255*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Incremental", "Subsubsection",
 CellChangeTimes->{{3.575989212353999*^9, 3.575989213731234*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2GenIncremental", "[", 
   RowBox[{"vts_", ",", "tridatafunc_", ",", "triorient_"}], "]"}], " ", ":=",
   " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "hull", ",", "hlen", ",", "k", ",", "kn", ",", "seg", ",", 
      "tridata", ",", "tdval", ",", "ix", ",", "bgns", ",", "ends", ",", 
      "IsOutside", ",", "AllOutside", ",", "hlk", ",", "hlkn", ",", "ixbgn", 
      ",", "ixend", ",", "kp"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "1"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"Range", "[", "nvts", "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"Range", "[", "2", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hlen", " ", "=", " ", 
      RowBox[{"Length", "[", "hull", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"kn", " ", "=", " ", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"seg", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"k", ",", "kn"}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tridata", "[", "seg", "]"}], " ", "=", " ", 
         RowBox[{"tridatafunc", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", 
            RowBox[{"seg", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"2", ",", "1"}], "}"}], "]"}], "]"}], "]"}], "]"}], 
          "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Make", " ", "triangles", " ", "between", " ", "the", " ", "point", 
        " ", "and", " ", "pairs", " ", "of", " ", "neighbors", " ", "on", " ",
         "the", " ", 
        RowBox[{"hull", ".", " ", "Do"}], " ", "any", " ", "of", " ", "them", 
        " ", "have", " ", "the", " ", "polarity", " ", "expected", " ", "for",
         " ", "a", " ", "nearby", " ", "outside", " ", 
        RowBox[{"point", "?"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"hlen", " ", "=", " ", 
         RowBox[{"Length", "[", "hull", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"bgns", " ", "=", " ", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"False", ",", "hlen"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ends", " ", "=", " ", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"False", ",", "hlen"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"IsOutside", " ", "=", " ", "False"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AllOutside", " ", "=", " ", "True"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"kn", " ", "=", " ", 
            RowBox[{
             RowBox[{"Mod", "[", 
              RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"seg", " ", "=", " ", 
            RowBox[{"hull", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"k", ",", "kn"}], "}"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"tdval", " ", "=", " ", 
            RowBox[{"tridata", "[", "seg", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"triorient", "[", 
              RowBox[{"tdval", ",", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "ix", "]"}], "]"}]}], "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{
               RowBox[{"bgns", "[", 
                RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "True"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"ends", "[", 
                RowBox[{"[", "kn", "]"}], "]"}], " ", "=", " ", "True"}], ";",
               "\[IndentingNewLine]", 
              RowBox[{"IsOutside", " ", "=", " ", "True"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"AllOutside", " ", "=", " ", "False"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"AllOutside", ",", " ", 
          RowBox[{
           RowBox[{"hull", "=", 
            RowBox[{"{", "}"}]}], ";", " ", 
           RowBox[{"Break", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"!", "IsOutside"}], ",", 
          RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"kp", " ", "=", " ", 
            RowBox[{
             RowBox[{"Mod", "[", 
              RowBox[{
               RowBox[{"k", "-", "2"}], ",", "hlen"}], "]"}], "+", "1"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"bgns", "[", 
               RowBox[{"[", "k", "]"}], "]"}], " ", "&&", " ", 
              RowBox[{"(", 
               RowBox[{"!", 
                RowBox[{"bgns", "[", 
                 RowBox[{"[", "kp", "]"}], "]"}]}], ")"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ixbgn", " ", "=", " ", "k"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Break", "[", "]"}]}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"kn", " ", "=", " ", 
            RowBox[{
             RowBox[{"Mod", "[", 
              RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ends", "[", 
               RowBox[{"[", "k", "]"}], "]"}], " ", "&&", " ", 
              RowBox[{"(", 
               RowBox[{"!", 
                RowBox[{"ends", "[", 
                 RowBox[{"[", "kn", "]"}], "]"}]}], ")"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ixend", " ", "=", " ", "k"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Break", "[", "]"}]}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Remove", " ", "the", " ", "points", " ", "that", " ", "will", " ", 
          "become", " ", "inside", " ", "and", " ", "add", " ", "the", " ", 
          "new", " ", "one", " ", "to", " ", "the", " ", 
          RowBox[{"hull", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"Which", "[", 
          RowBox[{
           RowBox[{"ixbgn", " ", "<", " ", "ixend"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"ixend", ",", 
                 RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", "ixbgn"}], "}"}]}], "]"}], ",", 
             RowBox[{"{", "ix", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"ixbgn", " ", ">", " ", "ixend"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{"hull", ",", 
               RowBox[{"{", 
                RowBox[{"ixend", ",", "ixbgn"}], "}"}]}], "]"}], ",", 
             RowBox[{"{", "ix", "}"}]}], "]"}]}], "\[IndentingNewLine]", 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"seg", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "2"}], ",", 
             RowBox[{"-", "1"}]}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tridata", "[", "seg", "]"}], " ", "=", " ", 
         RowBox[{"tridatafunc", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", 
            RowBox[{"seg", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"2", ",", "1"}], "}"}], "]"}], "]"}], "]"}], "]"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"seg", "  ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "1"}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tridata", "[", "seg", "]"}], " ", "=", " ", 
         RowBox[{"tridatafunc", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", 
            RowBox[{"seg", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"2", ",", "1"}], "}"}], "]"}], "]"}], "]"}], "]"}], 
          "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", "3", ",", "nvts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"RotateToMin", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575989112004964*^9, 3.575989123891686*^9}, {
   3.5759892162450457`*^9, 3.575989217276854*^9}, {3.575990816824333*^9, 
   3.5759908693583927`*^9}, {3.575999006722858*^9, 3.575999027337965*^9}, {
   3.5760131895512743`*^9, 3.5760131945665007`*^9}, {3.576013293278934*^9, 
   3.576013301240838*^9}, {3.576013338953743*^9, 3.576013354707448*^9}, {
   3.5760134146836157`*^9, 3.576013433338456*^9}, {3.576013481282713*^9, 
   3.576013517574024*^9}, {3.576013766529728*^9, 3.576013783760627*^9}, {
   3.5760175026707487`*^9, 3.576017554105238*^9}, {3.5760350736735563`*^9, 
   3.576035103187702*^9}, {3.576058924806303*^9, 3.5760590312124863`*^9}, {
   3.5760594877539463`*^9, 3.5760596836836033`*^9}, {3.576060539120452*^9, 
   3.576060631841042*^9}, {3.576060662749511*^9, 3.5760609116496468`*^9}, {
   3.57607432295535*^9, 3.576074323593398*^9}, {3.57608233835627*^9, 
   3.5760823794609823`*^9}, {3.5760824917134447`*^9, 3.57608249854414*^9}, {
   3.576158704631749*^9, 3.576158718736826*^9}, 3.5761587825100727`*^9, {
   3.576160690944961*^9, 3.5761606944414053`*^9}, {3.576160788940792*^9, 
   3.5761608357434263`*^9}, {3.576160879538661*^9, 3.576160954876842*^9}, {
   3.5761610409276752`*^9, 3.576161082957465*^9}, {3.576161153041895*^9, 
   3.576161153765533*^9}, {3.576985654679489*^9, 3.5769856551079483`*^9}, {
   3.577877386042697*^9, 3.5778773885125933`*^9}, {3.577877479838544*^9, 
   3.5778775287809057`*^9}, {3.577877596794798*^9, 3.577877597430101*^9}, {
   3.577877675392435*^9, 3.577877787675606*^9}, {3.577877843242386*^9, 
   3.577877859033633*^9}, {3.577877912885005*^9, 3.5778779218487673`*^9}, {
   3.5788855686863127`*^9, 3.578885572358239*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2Incremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CH2GenIncremental", "[", 
   RowBox[{"vts", ",", "TriAreaData2D", ",", "TriOrientEval2D"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5761587248111134`*^9, 3.576158765392201*^9}, {
  3.577877426222477*^9, 3.577877432573085*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2SphIncremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CH2GenIncremental", "[", 
   RowBox[{"vts", ",", "TriAreaData2Sph", ",", "TriOrientEval2Sph"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5761587248111134`*^9, 3.576158760713361*^9}, {
   3.577877439788232*^9, 3.577877446533252*^9}, 3.581582354366832*^9, {
   3.5815824595998287`*^9, 3.581582462718713*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Monotone Chain", "Subsubsection",
 CellChangeTimes->{{3.5760755374532213`*^9, 3.5760755395803423`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Each", " ", "individual", " ", "segment"}], ",", " ", 
   RowBox[{"either", " ", "upper", " ", "or", " ", 
    RowBox[{"lower", ".", " ", "Gets"}], " ", "vertices"}], ",", " ", 
   RowBox[{
    RowBox[{"indices", " ", "in", " ", "order"}], " ", "-", " ", 
    RowBox[{"either", " ", "upward", " ", "or", " ", "downward"}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.5760807079187927`*^9, 3.576080714444021*^9}, {
  3.576081441748355*^9, 3.57608146556395*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2MonotoneChainSegment", "[", 
   RowBox[{"vts_", ",", "ixs_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hseg", ",", "ix", ",", "hsend"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hseg", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"First", "[", "ixs", "]"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"hsend", " ", "=", " ", 
           RowBox[{"Length", "[", "hseg", "]"}]}], ",", 
          RowBox[{"hsend", "\[GreaterEqual]", "2"}], ",", 
          RowBox[{"hsend", "--"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"TriOrient2D", "[", 
              RowBox[{"vts", "[", 
               RowBox[{"[", 
                RowBox[{"{", 
                 RowBox[{"ix", ",", 
                  RowBox[{"hseg", "[", 
                   RowBox[{"[", "hsend", "]"}], "]"}], ",", 
                  RowBox[{"hseg", "[", 
                   RowBox[{"[", 
                    RowBox[{"hsend", "-", "1"}], "]"}], "]"}]}], "}"}], "]"}],
                "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"hseg", " ", "=", " ", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"Take", "[", 
            RowBox[{"hseg", ",", "hsend"}], "]"}], ",", "ix"}], "]"}]}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", 
         RowBox[{"Rest", "[", "ixs", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "hseg"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576080622022588*^9, 3.576080634067563*^9}, {
  3.576080699854567*^9, 3.576080766518506*^9}, {3.576081052452435*^9, 
  3.576081250760419*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2MonotoneChain", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ixvts", ",", "ixs1", ",", "ixs2", ",", "hull1", ",", 
      "hull2", ",", "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "1"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"Range", "[", "nvts", "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Increasing", " ", "and", " ", "decreasing", " ", "order"}], 
      " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"AddIndices", "[", "vts", "]"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixs1", " ", "=", " ", 
      RowBox[{"First", " ", "/@", " ", "ixvts"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixs2", " ", "=", " ", 
      RowBox[{"Reverse", "[", "ixs1", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"The", " ", "two", " ", "hull", " ", "halves"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"CH2MonotoneChainSegment", "[", 
       RowBox[{"vts", ",", "ixs1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"CH2MonotoneChainSegment", "[", 
       RowBox[{"vts", ",", "ixs2"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Most", "[", "hull1", "]"}], ",", 
        RowBox[{"Most", "[", "hull2", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"RotateToMin", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576075589568347*^9, 3.5760755930154257`*^9}, {
   3.576075666119611*^9, 3.576075667187112*^9}, {3.5760757961383867`*^9, 
   3.5760758023543463`*^9}, {3.576075895347411*^9, 3.57607592210859*^9}, 
   3.57607957618122*^9, {3.576079738575498*^9, 3.576079769822137*^9}, {
   3.576079817935113*^9, 3.576079819478572*^9}, {3.57608059213533*^9, 
   3.576080602102776*^9}, {3.576080637544301*^9, 3.5760806938019323`*^9}, {
   3.576081425090715*^9, 3.5760814286264153`*^9}, {3.576985665513335*^9, 
   3.576985733175274*^9}, {3.578885686423607*^9, 3.578885690559132*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Verification and Comparison", "Subsubsection",
 CellChangeTimes->{{3.5757865932155457`*^9, 3.5757866150754213`*^9}, {
  3.576144261062499*^9, 3.5761442634525414`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2GenLocCheck", "[", 
   RowBox[{"vts_", ",", "hull_", ",", "triorient_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "hlen", ",", "nvts", ",", "inside", ",", "k", ",", "kn", ",", "hlk", ",",
       "hlkn", ",", "ix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hlen", " ", "=", 
      RowBox[{"Length", "[", "hull", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"kn", " ", "=", " ", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{"k", ",", "hlen"}], "]"}], "+", "1"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hlk", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hlkn", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "kn", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ix", " ", "\[Equal]", " ", "hlk"}], " ", "||", " ", 
           RowBox[{"ix", " ", "\[Equal]", " ", "hlkn"}]}], ",", 
          "\[IndentingNewLine]", "True", ",", "\[IndentingNewLine]", 
          RowBox[{"triorient", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", 
             RowBox[{"{", 
              RowBox[{"hlk", ",", "hlkn", ",", "ix"}], "}"}], "]"}], "]"}], 
           "]"}]}], "\[IndentingNewLine]", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", "nvts"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input",
 CellChangeTimes->{{3.576160476801395*^9, 3.576160514621572*^9}, {
  3.577058315403681*^9, 3.577058319241228*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2LocCheck", "[", 
   RowBox[{"vts_", ",", "hull_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CH2GenLocCheck", "[", 
   RowBox[{"vts", ",", "hull", ",", "TriOrient2D"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5757866489572144`*^9, 3.575786689078209*^9}, {
  3.5757867327050543`*^9, 3.575786823688817*^9}, {3.5757869408346767`*^9, 
  3.5757871410427647`*^9}, {3.575990182446587*^9, 3.575990185070249*^9}, {
  3.576012663354772*^9, 3.576012751444072*^9}, {3.576012836102601*^9, 
  3.576012851222892*^9}, {3.576012918772579*^9, 3.576013049378448*^9}, {
  3.576013363093774*^9, 3.576013370238529*^9}, {3.576160522235409*^9, 
  3.576160531652213*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHSphLocCheck", "[", 
   RowBox[{"vts_", ",", "hull_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CH2GenLocCheck", "[", 
   RowBox[{"vts", ",", "hull", ",", "TriOrientSph"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5757866489572144`*^9, 3.575786689078209*^9}, {
  3.5757867327050543`*^9, 3.575786823688817*^9}, {3.5757869408346767`*^9, 
  3.5757871410427647`*^9}, {3.575990182446587*^9, 3.575990185070249*^9}, {
  3.576012663354772*^9, 3.576012751444072*^9}, {3.576012836102601*^9, 
  3.576012851222892*^9}, {3.576012918772579*^9, 3.576013049378448*^9}, {
  3.576013363093774*^9, 3.576013370238529*^9}, {3.576160522235409*^9, 
  3.57616054450041*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2Algorithms", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "CH2GrahamScan", ",", "CH2GiftWrapping", ",", "CH2DivideConquer", ",", 
     "CH2ChanShattering", ",", "CH2QuickHull", ",", "CH2Incremental", ",", 
     "CH2MonotoneChain"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.575990204332796*^9, 3.575990254047467*^9}, {
  3.575990450130661*^9, 3.575990453201108*^9}, {3.576074287745886*^9, 
  3.576074288225525*^9}, {3.5760814992053537`*^9, 3.5760814996311407`*^9}, {
  3.576148439842161*^9, 3.576148447192309*^9}, {3.5761705521966877`*^9, 
  3.576170560260644*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2SphAlgorithms", " ", "=", " ", 
   RowBox[{"{", "CH2SphIncremental", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581577430880744*^9, 3.581577438989724*^9}, {
  3.581577493000955*^9, 3.5815774938178253`*^9}, {3.581582670507001*^9, 
  3.5815826724557533`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3D", "Subsection",
 CellChangeTimes->{{3.576162181654368*^9, 3.576162183123013*^9}, {
  3.5790439123691607`*^9, 3.57904395222556*^9}, {3.579068836383751*^9, 
  3.579068841781406*^9}, {3.579069720598716*^9, 3.579069721334751*^9}, {
  3.5791021923858223`*^9, 3.5791021999951553`*^9}, {3.579141131403919*^9, 
  3.5791411317719393`*^9}}],

Cell[CellGroupData[{

Cell["Gift Wrapping", "Subsubsection",
 CellChangeTimes->{{3.576169978495591*^9, 3.5761699809397497`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GWMakeProjection", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"refpt", ",", "diffvec", ",", "dfcovec"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"refpt", " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"vts", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "+", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "/", "2"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"diffvec", " ", "=", " ", 
      RowBox[{
       RowBox[{"vts", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "-", 
       RowBox[{"vts", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dfcovec", " ", "=", " ", 
      RowBox[{"diffvec", "/", 
       RowBox[{"(", 
        RowBox[{"diffvec", ".", "diffvec"}], ")"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"refpt", ",", "diffvec", ",", "dfcovec"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.577045701021409*^9, 3.577045729284752*^9}, {
  3.577045842561447*^9, 3.5770458778144407`*^9}, {3.5770459814333487`*^9, 
  3.5770460006435423`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GWPartProj", "[", 
   RowBox[{"prj_", ",", "dir_"}], "]"}], " ", ":=", " ", 
  RowBox[{"dir", " ", "-", " ", 
   RowBox[{
    RowBox[{"prj", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "*", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"prj", "[", 
       RowBox[{"[", "3", "]"}], "]"}], ".", "dir"}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.578738778698804*^9, 3.578738806583222*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GWProjectOut", "[", 
   RowBox[{"prj_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CH3GWPartProj", "[", 
   RowBox[{"prj", ",", 
    RowBox[{"vt", " ", "-", " ", 
     RowBox[{"prj", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5770460077664633`*^9, 3.577046023043132*^9}, {
  3.57704612346692*^9, 3.577046153932314*^9}, {3.577046209903761*^9, 
  3.5770462560934887`*^9}, {3.5787388182164917`*^9, 3.578738830411418*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3PartialGiftWrapping", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"indef_:", "True"}], ",", 
    RowBox[{"nmax_:", "0"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ix0", ",", "ix1", ",", "vt", ",", "ixvts", ",", "ixvts1", 
      ",", "bdryedges", ",", " ", "bdryoppoverts", ",", "initial", ",", "n", 
      ",", "res", ",", "iscomplete", ",", "seg", ",", "prj", ",", "prevdir", 
      ",", "ix", ",", "newtri", ",", "triedges", ",", "trioppoverts", ",", 
      "bdrysel", ",", "trisel", ",", "k", ",", "pos", ",", "hulltris", ",", 
      "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"True", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Range", "[", "nvts", "]"}], ",", 
            RowBox[{"{", "}"}]}], "}"}]}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "a", " ", "farthest"}], "-", 
       RowBox[{"out", " ", "point"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ix0", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "closest"}], "-", "to", "-", 
       RowBox[{
       "outward", " ", "point", " ", "using", " ", "its", " ", 
        "direction"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"vt", " ", "=", " ", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "ix0", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"AddIndices", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"ixvts", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "ix0"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{"First", "[", 
           RowBox[{"Normalize", "[", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "-", "vt"}], "]"}], "]"}]}], 
         "}"}], "&"}], " ", "/@", " ", "ixvts1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ix1", " ", "=", " ", 
      RowBox[{"First", "[", 
       RowBox[{"MinBySortByFunc", "[", 
        RowBox[{"ixvts1", ",", 
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "triangles"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"bdryedges", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"bdryoppoverts", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"iscomplete", " ", "=", " ", "False"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"initial", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"n", "=", "0"}], ",", " ", 
         RowBox[{"indef", " ", "||", " ", 
          RowBox[{"(", 
           RowBox[{"n", "<", "nmax"}], ")"}]}], ",", 
         RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"Find", " ", "search", " ", "direction"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{"initial", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"seg", " ", "=", " ", 
              RowBox[{"{", 
               RowBox[{"ix0", ",", "ix1"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"prj", " ", "=", " ", 
              RowBox[{"CH3GWMakeProjection", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "seg", "]"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"prevdir", " ", "=", " ", 
              RowBox[{"Cross", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"-", "1"}], ",", "0", ",", "0"}], "}"}], ",", 
                RowBox[{"prj", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"initial", " ", "=", " ", "False"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"seg", " ", "=", " ", 
              RowBox[{"bdryedges", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"ix", " ", "=", " ", 
              RowBox[{"bdryoppoverts", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"prj", " ", "=", " ", 
              RowBox[{"CH3GWMakeProjection", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "seg", "]"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"prevdir", " ", "=", " ", 
              RowBox[{"-", " ", 
               RowBox[{"CH3GWProjectOut", "[", 
                RowBox[{"prj", ",", " ", 
                 RowBox[{"vts", "[", 
                  RowBox[{"[", "ix", "]"}], "]"}]}], "]"}]}]}]}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Select", " ", "the", " ", "vertex", " ", 
            RowBox[{"that", "'"}], "s", " ", "closest", " ", "to", " ", 
            "that", " ", 
            RowBox[{"direction", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
          
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{"Select", "[", 
            RowBox[{"ixvts", ",", 
             RowBox[{
              RowBox[{"!", 
               RowBox[{"MemberQ", "[", 
                RowBox[{"seg", ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "&"}]}], "]"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"prevdir", ".", 
                RowBox[{"Normalize", "[", 
                 RowBox[{"CH3GWProjectOut", "[", 
                  RowBox[{"prj", ",", 
                   RowBox[{"#", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}]}]}], "}"}],
              "&"}], " ", "/@", " ", "ixvts1"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"First", "[", 
            RowBox[{"MaxBySortByFunc", "[", 
             RowBox[{"ixvts1", ",", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "Compose", " ", "the", " ", "new", " ", "triangle", " ", "from", 
            " ", "that", " ", "vertex"}], " ", "*)"}], "\[IndentingNewLine]", 
          
          RowBox[{"newtri", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"seg", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"seg", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", "ix"}], "}"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Sow", "[", "newtri", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"triedges", " ", "=", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#", "[", 
                  RowBox[{"[", "2", "]"}], "]"}], ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#", "[", 
                  RowBox[{"[", "3", "]"}], "]"}], ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "}"}], "&"}], 
            " ", "@", " ", "newtri"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"trioppoverts", " ", "=", " ", "newtri"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
            RowBox[{
            "Add", " ", "its", " ", "edges", " ", "to", " ", "the", " ", 
             "list", " ", "of", " ", "edges"}], ",", " ", 
            RowBox[{"but", " ", "subtract", " ", "where", " ", "they", " ", 
             RowBox[{"duplicate", "."}]}]}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"bdrysel", " ", "=", " ", 
           RowBox[{"ConstantArray", "[", 
            RowBox[{"True", ",", 
             RowBox[{"Length", "[", "bdryedges", "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"trisel", " ", "=", " ", 
           RowBox[{"ConstantArray", "[", 
            RowBox[{"True", ",", "3"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"Do", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"pos", " ", "=", " ", 
              RowBox[{"Position", "[", 
               RowBox[{"bdryedges", ",", 
                RowBox[{"Reverse", "[", 
                 RowBox[{"triedges", "[", 
                  RowBox[{"[", "k", "]"}], "]"}], "]"}]}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Length", "[", "pos", "]"}], " ", ">", " ", "0"}], 
               ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"ix", " ", "=", " ", 
                 RowBox[{"pos", "[", 
                  RowBox[{"[", 
                   RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"bdrysel", "[", 
                  RowBox[{"[", "ix", "]"}], "]"}], " ", "=", " ", "False"}], 
                ";", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"trisel", "[", 
                  RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", 
                 "False"}]}]}], "\[IndentingNewLine]", "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"k", ",", "3"}], "}"}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"bdryedges", " ", "=", " ", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"SelWithList", "[", 
              RowBox[{"bdrysel", ",", "bdryedges"}], "]"}], ",", 
             RowBox[{"SelWithList", "[", 
              RowBox[{"trisel", ",", "triedges"}], "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"bdryoppoverts", " ", "=", " ", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"SelWithList", "[", 
              RowBox[{"bdrysel", ",", "bdryoppoverts"}], "]"}], ",", 
             RowBox[{"SelWithList", "[", 
              RowBox[{"trisel", ",", "trioppoverts"}], "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "bdryedges", "]"}], " ", "\[Equal]", " ", 
             "0"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"iscomplete", " ", "=", " ", "True"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", 
           "]"}]}]}], "\[IndentingNewLine]", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hulltris", " ", "=", " ", 
      RowBox[{"res", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH3DOutputFromTriangles", "[", "hulltris", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"iscomplete", ",", "hull"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576865114083542*^9, 3.576865128592491*^9}, {
   3.576865503400943*^9, 3.576865509102724*^9}, 3.57686554004494*^9, 
   3.576984411555727*^9, {3.5769844419397907`*^9, 3.576984446996233*^9}, {
   3.576984582932901*^9, 3.5769845831330023`*^9}, {3.576984740312188*^9, 
   3.576984791386381*^9}, {3.576984865209584*^9, 3.5769848822506037`*^9}, {
   3.576984971131645*^9, 3.576984976763644*^9}, {3.577032702004829*^9, 
   3.5770327083066196`*^9}, {3.577033693804084*^9, 3.577033775104992*^9}, {
   3.577034288912118*^9, 3.577034302029847*^9}, {3.577034857065804*^9, 
   3.577034950567184*^9}, {3.577036037499617*^9, 3.577036081817319*^9}, {
   3.577043555780568*^9, 3.577043556323996*^9}, {3.577043862078783*^9, 
   3.577043877123172*^9}, {3.577044774684071*^9, 3.5770448187773943`*^9}, {
   3.577046522790659*^9, 3.5770465411177893`*^9}, {3.577046585376145*^9, 
   3.577046620003338*^9}, {3.5770466722759867`*^9, 3.577046756858982*^9}, {
   3.577046808887129*^9, 3.577046873789773*^9}, {3.57704713060929*^9, 
   3.577047219050797*^9}, {3.5770473049866533`*^9, 3.5770473245052233`*^9}, {
   3.577047377655138*^9, 3.577047456950034*^9}, {3.5770483709282*^9, 
   3.577048378562101*^9}, {3.5770485882717037`*^9, 3.5770486175016813`*^9}, {
   3.577049245350862*^9, 3.577049258750785*^9}, {3.577049670959614*^9, 
   3.5770497230550528`*^9}, {3.577049967783308*^9, 3.577049981293902*^9}, {
   3.577050023593109*^9, 3.577050057577374*^9}, {3.577051604235832*^9, 
   3.5770516064221277`*^9}, {3.5770516572780647`*^9, 3.577051664917632*^9}, {
   3.577051849381053*^9, 3.577051856348124*^9}, {3.577052056369815*^9, 
   3.57705208030121*^9}, {3.577052110310918*^9, 3.577052118911295*^9}, {
   3.5770521914212227`*^9, 3.577052207334435*^9}, {3.577052242894895*^9, 
   3.577052265208831*^9}, {3.577052451299588*^9, 3.577052479671191*^9}, {
   3.5770525355196257`*^9, 3.577052647871091*^9}, {3.577054075593709*^9, 
   3.5770540808540583`*^9}, {3.577054259416366*^9, 3.577054266633185*^9}, {
   3.577054483296777*^9, 3.577054503830571*^9}, {3.577054549863328*^9, 
   3.577054575436812*^9}, {3.5771664336727047`*^9, 3.577166475377664*^9}, {
   3.577932543018207*^9, 3.577932560488406*^9}, {3.581373730911832*^9, 
   3.5813737934849777`*^9}, {3.581375716832012*^9, 3.581375720732512*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GiftWrapping", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"iscomplete", ",", "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"iscomplete", ",", "hull"}], "}"}], " ", "=", " ", 
      RowBox[{"CH3PartialGiftWrapping", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"iscomplete", ",", "hull", ",", 
       RowBox[{"CH3DOutputFromTriangles", "[", 
        RowBox[{"{", "}"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5768655122210007`*^9, 3.576865534735012*^9}, {
   3.576874484686337*^9, 3.5768744858445187`*^9}, {3.577054788749959*^9, 
   3.577054789166621*^9}, {3.578886303151701*^9, 3.578886349223419*^9}, 
   3.5813737996720037`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Divide and Conquer", "Subsubsection",
 CellChangeTimes->{{3.576169992861631*^9, 3.5761699965237017`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3DCFindNeighbors", "[", 
   RowBox[{"tris_", ",", "vts_", ",", "ngbrix_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"tri", ",", "vt"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Length", "[", "tris", "]"}], " ", ">", " ", "0"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Collect", " ", "each", " ", 
        RowBox[{"vertex", "'"}], "s", " ", "neighbors", " ", "for", " ", 
        "each", " ", "triangle"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Do", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Head", "[", 
                RowBox[{"ngbrix", "[", "vt", "]"}], "]"}], " ", "===", " ", 
               "ngbrix"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"ngbrix", "[", "vt", "]"}], " ", "=", " ", 
               RowBox[{"{", "}"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{
              RowBox[{"ngbrix", "[", "vt", "]"}], ",", 
              RowBox[{"Complement", "[", 
               RowBox[{"tri", ",", 
                RowBox[{"{", "vt", "}"}]}], "]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"vt", ",", "tri"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"tri", ",", "tris"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"Remove", " ", "duplicates"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Do", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"ngbrix", "[", "vt", "]"}], " ", "=", " ", 
            RowBox[{"Union", "[", 
             RowBox[{"Flatten", "[", 
              RowBox[{"ngbrix", "[", "vt", "]"}], "]"}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"vt", ",", "tri"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"tri", ",", "tris"}], "}"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"With", " ", "no", " ", "triangles"}], ",", " ", 
        RowBox[{
        "use", " ", "the", " ", "vertex", " ", "list", " ", "instead"}]}], 
       " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Do", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ngbrix", "[", "vt", "]"}], " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"vts", ",", 
           RowBox[{"{", "vt", "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"vt", ",", "vts"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.577210082500319*^9, 3.577210095717577*^9}, {
   3.577210241090886*^9, 3.577210443823126*^9}, {3.577210507321344*^9, 
   3.5772105087354507`*^9}, {3.577210840797144*^9, 3.577210841715115*^9}, {
   3.577211332514765*^9, 3.577211392438596*^9}, {3.577211845753243*^9, 
   3.5772119072031384`*^9}, 3.5772705236701736`*^9, 3.5772705885871468`*^9, 
   3.5781030395893*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3DCTrimTriangles", "[", 
   RowBox[{"tris_", ",", "rngeds_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "tri", ",", "trieds", ",", "edtri", ",", "tristat", ",", "ed", ",", 
      "eduse", ",", "rngtris", ",", "unchecked", ",", "ntris", ",", "k", ",", 
      "trix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "tris", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "rngeds", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "Get", " ", "the", " ", "triangle", " ", "and", " ", "edge", " ", 
        "references"}], ",", " ", 
       RowBox[{
       "their", " ", "references", " ", "to", " ", "each", " ", "other"}]}], 
      " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"TriEdgeRefs", "[", 
        RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "tris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Ring", " ", 
       RowBox[{"edges", ":", " ", 
        RowBox[{
         RowBox[{"don", "'"}], "t", " ", "expand", " ", "past", " ", 
         "those"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"eduse", "[", "ed", "]"}], " ", "=", " ", "True"}], ",", 
         RowBox[{"{", 
          RowBox[{"ed", ",", 
           RowBox[{"trieds", "[", "tri", "]"}]}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "tris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"eduse", "[", "ed", "]"}], " ", "=", " ", "False"}], ",", 
       RowBox[{"{", 
        RowBox[{"ed", ",", "rngeds"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
        "Find", " ", "which", " ", "existing", " ", "triangles", " ", "are", 
         " ", "accessible", " ", "from", " ", "the", " ", "ring", " ", 
         RowBox[{"edges", ".", " ", "0"}]}], " ", "=", " ", "unchecked"}], 
       ",", " ", 
       RowBox[{"1", " ", "=", " ", "checked"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"rngtris", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"edtri", "[", "ed", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ed", ",", "rngeds"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"tristat", "[", "tri", "]"}], " ", "=", " ", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "rngtris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"unchecked", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"unchecked", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"unchecked", " ", "=", " ", "False"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ntris", " ", "=", " ", 
         RowBox[{"Length", "[", "rngtris", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"rngtris", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"tristat", "[", "tri", "]"}], " ", "\[Equal]", " ", 
              "0"}], ",", " ", 
             RowBox[{
              RowBox[{"unchecked", " ", "=", " ", "True"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"(*", " ", 
               RowBox[{"Look", " ", "at", " ", "a", " ", 
                RowBox[{"triangle", "'"}], "s", " ", "neighboring", " ", 
                "triangles"}], " ", "*)"}], "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"eduse", "[", "ed", "]"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"trix", " ", "=", " ", 
                    RowBox[{"edtri", "[", 
                    RowBox[{"Reverse", "[", "ed", "]"}], "]"}]}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"tristat", "[", "trix", "]"}], "]"}], " ", "===", 
                    " ", "tristat"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"rngtris", ",", "trix"}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"tristat", "[", "trix", "]"}], " ", "=", " ", 
                    "0"}]}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{"ed", ",", 
                  RowBox[{"trieds", "[", "tri", "]"}]}], "}"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"tristat", "[", "tri", "]"}], " ", "=", " ", "1"}]}]}],
             "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "ntris"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Complement", "[", 
      RowBox[{"tris", ",", "rngtris"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5771711338525057`*^9, 3.5771711649632187`*^9}, {
   3.577171282942721*^9, 3.577171304018222*^9}, {3.577171364307446*^9, 
   3.5771713712219963`*^9}, 3.57719619688378*^9, {3.577200502331499*^9, 
   3.57720055843417*^9}, {3.577200735335083*^9, 3.577200836832486*^9}, {
   3.577201200400715*^9, 3.577201213754002*^9}, {3.577201254516172*^9, 
   3.577201260515053*^9}, {3.577201461381238*^9, 3.577201474940029*^9}, {
   3.577201537689073*^9, 3.577201547720005*^9}, {3.57720163612252*^9, 
   3.577201660592013*^9}, {3.577202108637879*^9, 3.577202161239538*^9}, {
   3.577202194575162*^9, 3.5772023723559628`*^9}, {3.5772024826212997`*^9, 
   3.5772025538711777`*^9}, {3.577202590307753*^9, 3.5772026136881733`*^9}, {
   3.57720266816428*^9, 3.5772027728130283`*^9}, {3.577202817363633*^9, 
   3.577202847580779*^9}, {3.57720301778358*^9, 3.577203066896846*^9}, 
   3.5772031141147957`*^9, {3.577203210743087*^9, 3.577203248815597*^9}, {
   3.5772033199154663`*^9, 3.5772033938368073`*^9}, {3.577204280200787*^9, 
   3.577204300095449*^9}, {3.577204394717969*^9, 3.5772044466301603`*^9}, {
   3.5772048001526546`*^9, 3.577204860003337*^9}, {3.577204930130249*^9, 
   3.577204938499301*^9}, {3.5772051007991667`*^9, 3.5772051899740353`*^9}, {
   3.577205314097542*^9, 3.577205357810549*^9}, {3.5772061574196453`*^9, 
   3.5772061822385807`*^9}, {3.577206243015979*^9, 3.5772063704205303`*^9}, {
   3.577206440315189*^9, 3.577206525543146*^9}, {3.5772065956398993`*^9, 
   3.5772066014706182`*^9}, {3.577206652885088*^9, 3.577206653213728*^9}, {
   3.5772068411031017`*^9, 3.577206850011878*^9}, {3.577209078607584*^9, 
   3.577209119888495*^9}, {3.577209240618597*^9, 3.577209268714266*^9}, {
   3.577210078780532*^9, 3.57721007940431*^9}, 3.577270527779203*^9, 
   3.577270590327886*^9, {3.578446813028645*^9, 3.578446831726253*^9}, {
   3.579022556488077*^9, 3.579022565465954*^9}, {3.579881287546735*^9, 
   3.5798812913749037`*^9}, {3.579889735205027*^9, 3.579889737051938*^9}, {
   3.57988978325353*^9, 3.579889801964028*^9}, {3.579889846204699*^9, 
   3.5798898832845697`*^9}, {3.579889916969241*^9, 3.579889950976557*^9}, {
   3.579890005841236*^9, 3.579890097644919*^9}, {3.579890141255721*^9, 
   3.579890147468305*^9}, {3.579890323330142*^9, 3.579890368358788*^9}, {
   3.57989046322036*^9, 3.579890506109626*^9}, {3.579891868442951*^9, 
   3.579892016657463*^9}, {3.579892066375791*^9, 3.579892098529313*^9}, {
   3.579892195542809*^9, 3.5798922192140408`*^9}, {3.579892263876501*^9, 
   3.579892264249419*^9}, {3.579892338583735*^9, 3.579892383616517*^9}, 
   3.5798977955789433`*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", "Recursive", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575786636696672*^9, 3.575786639748083*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3DivideConquer", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "hullvts", ",", "hulltris", ",", "hull", ",", "ixvts", ",", 
      "nspl", ",", "ixvts1", ",", "ixvts2", ",", "ixs1", ",", "ixs2", ",", 
      "vts1", ",", "vts2", ",", "hull1", ",", "hull2", ",", "hullixs1", ",", 
      "hullixs2", ",", "hullvts1", ",", "hullvts2", ",", "hulltris1", ",", 
      "hulltris2", ",", "ngbrix1", ",", "ngbrix2", ",", "projhull1", ",", 
      "projhull2", ",", "k", ",", "ixnr1", ",", "ixnr2", ",", "twtres", ",", 
      "ix1", ",", "ix2", ",", "ixin1", ",", "ixin2", ",", "blixvts", ",", 
      "numblix", ",", "initial", ",", "res", ",", "seg", ",", "prj", ",", " ",
       "prevdir", ",", "blix1", ",", "blix2", ",", "blixvts01", ",", 
      "blixvts02", ",", "ixp", ",", "ixn", ",", "blixvts1", ",", "blix", ",", 
      "ih", ",", "newtri", ",", "belttris", ",", "hrvts1", ",", "hrvts2", ",",
       "hreds1", ",", "hreds2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{"For", " ", "at", " ", "most", " ", "3", " ", "points"}], ",", 
      " ", 
      RowBox[{"find", " ", "the", " ", "hull", " ", "by", " ", "hand"}]}], 
     " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "3"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hullvts", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hulltris", " ", "=", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nvts", " ", "\[Equal]", " ", "3"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "3", ",", "2"}], "}"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"hullvts", ",", "hulltris"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "hulls", " ", "of", " ", "the", " ", "two", 
       " ", "parts"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Rotate", " ", "the", " ", "splitting", " ", "axis", " ", "for", " ", 
       "calculating", " ", "the", " ", "child", " ", 
       RowBox[{"hulls", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"AddIndices", "[", "vts", "]"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nspl", " ", "=", " ", 
      RowBox[{"Floor", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nvts", "+", "1"}], ")"}], "/", "2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts2", " ", "=", " ", 
      RowBox[{"Drop", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixs1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixs2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"CH3DivideConquer", "[", 
       RowBox[{"RotateLeft", " ", "/@", " ", "vts1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"CH3DivideConquer", "[", 
       RowBox[{"RotateLeft", " ", "/@", " ", "vts2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullixs1", " ", "=", " ", 
      RowBox[{"ixs1", "[", 
       RowBox[{"[", 
        RowBox[{"hull1", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullixs2", " ", "=", " ", 
      RowBox[{"ixs2", "[", 
       RowBox[{"[", 
        RowBox[{"hull2", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullvts1", " ", "=", " ", 
      RowBox[{"vts1", "[", 
       RowBox[{"[", 
        RowBox[{"hull1", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullvts2", " ", "=", " ", 
      RowBox[{"vts2", "[", 
       RowBox[{"[", 
        RowBox[{"hull2", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hulltris1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"ixs1", "[", 
         RowBox[{"[", "#", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"hull1", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hulltris2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"ixs2", "[", 
         RowBox[{"[", "#", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"hull2", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"CH3DCFindNeighbors", "[", 
      RowBox[{"hulltris1", ",", "hullixs1", ",", "ngbrix1"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"CH3DCFindNeighbors", "[", 
      RowBox[{"hulltris2", ",", "hullixs2", ",", "ngbrix2"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "projected", " ", "hulls"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"projhull1", " ", "=", " ", 
      RowBox[{"CH2DivideConquer", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Take", "[", 
          RowBox[{"#", ",", "2"}], "]"}], "&"}], " ", "/@", " ", "hullvts1"}],
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"projhull2", " ", "=", " ", 
      RowBox[{"CH2DivideConquer", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Take", "[", 
          RowBox[{"#", ",", "2"}], "]"}], "&"}], " ", "/@", " ", "hullvts2"}],
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "closest", " ", "points"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"projhull1", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", 
          RowBox[{"Length", "[", "hullvts1", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ixnr1", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "projhull1", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"projhull2", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ixnr2", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "projhull2", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "a", " ", "tangent", " ", "line", " ", "between", " ", 
       "the", " ", "projected", " ", "hulls"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"twtres", " ", "=", " ", 
      RowBox[{"CH2TwoHullTangent", "[", 
       RowBox[{
        RowBox[{"hullvts1", "[", 
         RowBox[{"[", "projhull1", "]"}], "]"}], ",", 
        RowBox[{"hullvts2", "[", 
         RowBox[{"[", "projhull2", "]"}], "]"}], ",", "ixnr1", ",", "ixnr2"}],
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ix1", " ", "=", " ", 
      RowBox[{"hullixs1", "[", 
       RowBox[{"[", 
        RowBox[{"projhull1", "[", 
         RowBox[{"[", 
          RowBox[{"twtres", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], "]"}], "]"}]}], ";", 
     " ", "\[IndentingNewLine]", 
     RowBox[{"ix2", " ", "=", " ", 
      RowBox[{"hullixs2", "[", 
       RowBox[{"[", 
        RowBox[{"projhull2", "[", 
         RowBox[{"[", 
          RowBox[{"twtres", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixin1", " ", "=", " ", "ix1"}], ";", " ", 
     RowBox[{"ixin2", " ", "=", " ", "ix2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "belt", " ", "between", " ", "the", " ", "two", " ", 
       "hulls"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"blixvts", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"1", ",", "#"}], "}"}], "&"}], " ", "/@", " ", 
            "hullixs1"}], ",", "hullvts1"}], "}"}], "]"}], ",", 
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"2", ",", "#"}], "}"}], "&"}], " ", "/@", " ", 
            "hullixs2"}], ",", "hullvts2"}], "}"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"numblix", " ", "=", " ", 
      RowBox[{
       RowBox[{"Length", "[", "hullixs1", "]"}], "*", 
       RowBox[{"Length", "[", "hullixs2", "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"seg", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"ix1", ",", " ", "ix2"}], "}"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"initial", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"prj", " ", "=", " ", 
            RowBox[{"CH3GWMakeProjection", "[", 
             RowBox[{"vts", "[", 
              RowBox[{"[", "seg", "]"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{"initial", ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"prevdir", " ", "=", " ", 
               RowBox[{"-", " ", 
                RowBox[{"CH3GWPartProj", "[", 
                 RowBox[{"prj", ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "]"}]}]}], ";",
               "\[IndentingNewLine]", 
              RowBox[{"initial", " ", "=", " ", "False"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"prevdir", " ", "=", " ", 
              RowBox[{"-", " ", 
               RowBox[{"CH3GWProjectOut", "[", 
                RowBox[{"prj", ",", " ", 
                 RowBox[{"vts", "[", 
                  RowBox[{"[", "ixp", "]"}], "]"}]}], "]"}]}]}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"blix1", " ", "=", " ", 
            RowBox[{"ngbrix1", "[", 
             RowBox[{"seg", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"blix2", " ", "=", " ", 
            RowBox[{"ngbrix2", "[", 
             RowBox[{"seg", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"blixvts01", " ", "=", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"1", ",", "#"}], "}"}], ",", 
                RowBox[{"vts", "[", 
                 RowBox[{"[", "#", "]"}], "]"}]}], "}"}], "&"}], " ", "/@", 
             " ", "blix1"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"blixvts02", " ", "=", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"2", ",", "#"}], "}"}], ",", 
                RowBox[{"vts", "[", 
                 RowBox[{"[", "#", "]"}], "]"}]}], "}"}], "&"}], " ", "/@", 
             " ", "blix2"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{"blixvts1", " ", "=", " ", 
              RowBox[{"Select", "[", 
               RowBox[{"blixvts", ",", 
                RowBox[{
                 RowBox[{"!", 
                  RowBox[{"MemberQ", "[", 
                   RowBox[{"seg", ",", 
                    RowBox[{"#", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}]}], "&"}]}],
                "]"}]}], ";"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"blixvts1", " ", "=", " ", 
            RowBox[{"Join", "[", 
             RowBox[{"blixvts01", ",", "blixvts02"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"blixvts1", " ", "=", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"prevdir", ".", 
                 RowBox[{"Normalize", "[", 
                  RowBox[{"CH3GWProjectOut", "[", 
                   RowBox[{"prj", ",", 
                    RowBox[{"#", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}]}]}], "}"}],
               "&"}], " ", "/@", " ", "blixvts1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"blix", " ", "=", " ", 
            RowBox[{"First", "[", 
             RowBox[{"MaxBySortByFunc", "[", 
              RowBox[{"blixvts1", ",", 
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";",
            "\[IndentingNewLine]", 
           RowBox[{"ih", " ", "=", " ", 
            RowBox[{"blix", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"ixn", " ", "=", " ", 
            RowBox[{"blix", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "Compose", " ", "the", " ", "new", " ", "triangle", " ", "from", 
             " ", "that", " ", "vertex"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"newtri", " ", "=", " ", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"seg", "[", 
               RowBox[{"[", "2", "]"}], "]"}], ",", 
              RowBox[{"seg", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", "ixn"}], "}"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Sow", "[", 
            RowBox[{"newtri", ",", "\"\<tri\>\""}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Switch", "[", 
            RowBox[{"ih", ",", "\[IndentingNewLine]", "1", ",", 
             RowBox[{
              RowBox[{"Sow", "[", 
               RowBox[{"ixn", ",", "\"\<vt1\>\""}], "]"}], ";", 
              RowBox[{"Sow", "[", 
               RowBox[{
                RowBox[{"newtri", "[", 
                 RowBox[{"[", 
                  RowBox[{"{", 
                   RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], ",", 
                "\"\<ed1\>\""}], "]"}]}], ",", "\[IndentingNewLine]", "2", 
             ",", 
             RowBox[{
              RowBox[{"Sow", "[", 
               RowBox[{"ixn", ",", "\"\<vt2\>\""}], "]"}], ";", 
              RowBox[{"Sow", "[", 
               RowBox[{
                RowBox[{"newtri", "[", 
                 RowBox[{"[", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "1"}], "}"}], "]"}], "]"}], ",", 
                "\"\<ed2\>\""}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";",
            "\[IndentingNewLine]", 
           RowBox[{"ixp", " ", "=", " ", 
            RowBox[{"seg", "[", 
             RowBox[{"[", "ih", "]"}], "]"}]}], ";", " ", 
           RowBox[{
            RowBox[{"seg", "[", 
             RowBox[{"[", "ih", "]"}], "]"}], " ", "=", " ", "ixn"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"seg", "[", 
                RowBox[{"[", "1", "]"}], "]"}], " ", "\[Equal]", " ", 
               "ixin1"}], " ", "&&", " ", 
              RowBox[{
               RowBox[{"seg", "[", 
                RowBox[{"[", "2", "]"}], "]"}], " ", "\[Equal]", " ", 
               "ixin2"}]}], ",", " ", 
             RowBox[{"Break", "[", "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", "numblix", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
        
        RowBox[{"{", 
         RowBox[{
         "\"\<tri\>\"", ",", "\"\<vt1\>\"", ",", "\"\<vt2\>\"", ",", 
          "\"\<ed1\>\"", ",", "\"\<ed2\>\""}], "}"}]}], "\[IndentingNewLine]",
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"belttris", " ", "=", " ", 
      RowBox[{"ExtractReaped", "[", 
       RowBox[{"res", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hrvts1", " ", "=", " ", 
      RowBox[{"ExtractReaped", "[", 
       RowBox[{"res", ",", "2"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hrvts2", " ", "=", " ", 
      RowBox[{"ExtractReaped", "[", 
       RowBox[{"res", ",", "3"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hreds1", " ", "=", " ", 
      RowBox[{"ExtractReaped", "[", 
       RowBox[{"res", ",", "4"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hreds2", " ", "=", " ", 
      RowBox[{"ExtractReaped", "[", 
       RowBox[{"res", ",", "5"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Remove", " ", "inward", " ", "triangles", " ", "for", " ", "both", " ",
        "hulls"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hulltris1", " ", "=", " ", 
      RowBox[{"CH3DCTrimTriangles", "[", 
       RowBox[{"hulltris1", ",", "hreds1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hulltris2", " ", "=", " ", 
      RowBox[{"CH3DCTrimTriangles", "[", 
       RowBox[{"hulltris2", ",", "hreds2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Merge", " ", "the", " ", "remaining", " ", "ones", " ", "together"}], 
      " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"CH3DOutputFromTriangles", "[", 
      RowBox[{"Join", "[", 
       RowBox[{"hulltris1", ",", "belttris", ",", "hulltris2"}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.575748121984099*^9, 3.575748133075938*^9}, {
   3.57578655472513*^9, 3.575786582390957*^9}, {3.5757873538331614`*^9, 
   3.575787401202379*^9}, {3.5757874326641903`*^9, 3.575787572516796*^9}, {
   3.575787679197785*^9, 3.575787751425363*^9}, {3.575787781803688*^9, 
   3.57578780940387*^9}, {3.575789031535413*^9, 3.5757890806103077`*^9}, {
   3.575789116911085*^9, 3.575789169414853*^9}, {3.575789291096561*^9, 
   3.575789413102585*^9}, {3.575789451627865*^9, 3.5757894572107363`*^9}, 
   3.575789492296549*^9, {3.575789688365378*^9, 3.5757897889042053`*^9}, {
   3.5757898650243673`*^9, 3.575789935440854*^9}, {3.575790070571764*^9, 
   3.575790093043057*^9}, {3.575790232950604*^9, 3.575790262168594*^9}, {
   3.575790303137864*^9, 3.575790332287765*^9}, {3.5757903735315533`*^9, 
   3.5757905746520348`*^9}, {3.5757906138218317`*^9, 3.575790620810421*^9}, {
   3.5757906855835752`*^9, 3.575790760742723*^9}, {3.575791533901527*^9, 
   3.575791535251452*^9}, {3.575791864239854*^9, 3.575791922414309*^9}, {
   3.575792073008532*^9, 3.5757920862298737`*^9}, {3.575792776685885*^9, 
   3.575792821001733*^9}, {3.575796120286574*^9, 3.575796160283103*^9}, {
   3.575796219716041*^9, 3.575796241511891*^9}, {3.575796304978487*^9, 
   3.5757963300170317`*^9}, {3.575796950870241*^9, 3.575797045448537*^9}, {
   3.575837879280663*^9, 3.575837923251533*^9}, {3.575981221608276*^9, 
   3.575981221773098*^9}, {3.575981561927153*^9, 3.575981562089891*^9}, {
   3.5759816611929407`*^9, 3.575981676221401*^9}, {3.575981809651079*^9, 
   3.575981980924674*^9}, {3.575982032807292*^9, 3.57598206231157*^9}, {
   3.575982247266596*^9, 3.5759822613789463`*^9}, {3.575982338357884*^9, 
   3.5759823624439173`*^9}, {3.5759824552495527`*^9, 3.575982537612626*^9}, {
   3.575982576129842*^9, 3.575982601422359*^9}, {3.575982641406695*^9, 
   3.575982641973921*^9}, {3.576309460401182*^9, 3.576309471037458*^9}, {
   3.576309501766316*^9, 3.576309637330697*^9}, {3.576311098693799*^9, 
   3.576311159244645*^9}, {3.5763111996144447`*^9, 3.576311216615343*^9}, {
   3.576311415654913*^9, 3.576311421370924*^9}, {3.5763114561413403`*^9, 
   3.576311575078356*^9}, {3.5763116191219893`*^9, 3.576311669662118*^9}, {
   3.576338123004759*^9, 3.576338143561308*^9}, 3.5765061307554417`*^9, {
   3.577058607805697*^9, 3.5770586961758747`*^9}, {3.57705884678146*^9, 
   3.5770588958002567`*^9}, {3.577058980865094*^9, 3.577059038939393*^9}, {
   3.57705937688946*^9, 3.577059422438108*^9}, {3.577059480413266*^9, 
   3.577059531791513*^9}, 3.577059563954064*^9, {3.5770595982476063`*^9, 
   3.577059600139077*^9}, {3.577059853656898*^9, 3.577059855634008*^9}, {
   3.577059924247554*^9, 3.577059966152567*^9}, {3.577060027149864*^9, 
   3.577060122512244*^9}, {3.5770607158915043`*^9, 3.577060726710826*^9}, {
   3.577061112841281*^9, 3.577061142066897*^9}, {3.577062166581757*^9, 
   3.577062166649909*^9}, {3.577062222404893*^9, 3.5770623661133413`*^9}, {
   3.5770625202455072`*^9, 3.5770625712622633`*^9}, {3.57706261541833*^9, 
   3.5770626648574953`*^9}, {3.577064573929178*^9, 3.5770646123649263`*^9}, {
   3.5770649836004963`*^9, 3.577064997261413*^9}, {3.5770650568375*^9, 
   3.577065064972261*^9}, {3.577116917758497*^9, 3.577116940048764*^9}, {
   3.577118826632924*^9, 3.5771188328636913`*^9}, 3.5771192603690042`*^9, {
   3.577119344361125*^9, 3.577119447502716*^9}, {3.5771194850413103`*^9, 
   3.577119496572094*^9}, {3.5771195558868523`*^9, 3.5771196202176857`*^9}, {
   3.577119690608285*^9, 3.5771197122360773`*^9}, {3.5771197468682947`*^9, 
   3.577119785084754*^9}, {3.5771227692551003`*^9, 3.577122772952106*^9}, {
   3.5771349487422523`*^9, 3.577135026020193*^9}, {3.577135095691576*^9, 
   3.577135294392886*^9}, {3.5771353439442043`*^9, 3.5771354079376383`*^9}, 
   3.5771355194313297`*^9, {3.577135654734811*^9, 3.5771356841917458`*^9}, {
   3.5771357178545723`*^9, 3.577135741238014*^9}, {3.577135805205763*^9, 
   3.577135813526555*^9}, {3.577136123655117*^9, 3.577136433705576*^9}, {
   3.577136573402012*^9, 3.577136608738051*^9}, {3.57713775018237*^9, 
   3.57713776073629*^9}, {3.577137889152966*^9, 3.57713791095333*^9}, {
   3.5771418009053392`*^9, 3.577141815337009*^9}, {3.577142046479538*^9, 
   3.577142049476334*^9}, {3.577142186918261*^9, 3.577142188268467*^9}, {
   3.577142375174691*^9, 3.577142379860219*^9}, {3.577142418131418*^9, 
   3.577142422678288*^9}, {3.577142505372512*^9, 3.577142508792551*^9}, {
   3.577148003237391*^9, 3.5771480285086813`*^9}, {3.577148095926112*^9, 
   3.577148142228459*^9}, {3.57715030095347*^9, 3.577150319189909*^9}, 
   3.577150358597875*^9, {3.577171185599073*^9, 3.57717121667354*^9}, {
   3.577171265258565*^9, 3.577171276241333*^9}, {3.57717148928187*^9, 
   3.577171500907583*^9}, {3.577195064677384*^9, 3.5771950653622026`*^9}, {
   3.57719509693853*^9, 3.577195360312306*^9}, {3.577195395922606*^9, 
   3.577195396780859*^9}, {3.5771954339907503`*^9, 3.577195434233129*^9}, {
   3.577195569088272*^9, 3.5771955905671673`*^9}, {3.577195635086709*^9, 
   3.57719574609655*^9}, {3.577195786822359*^9, 3.5771957996950493`*^9}, {
   3.577200455849468*^9, 3.5772004955324287`*^9}, {3.5772007252695723`*^9, 
   3.5772007257750587`*^9}, {3.5772013048193502`*^9, 3.577201319211537*^9}, {
   3.5772028641913853`*^9, 3.5772028696827717`*^9}, {3.577203008356668*^9, 
   3.577203010900998*^9}, {3.577208297536034*^9, 3.5772083017511663`*^9}, {
   3.577208342869388*^9, 3.577208370533112*^9}, {3.577208486589285*^9, 
   3.577208487505835*^9}, {3.577210113520215*^9, 3.57721011888026*^9}, {
   3.5772101833071547`*^9, 3.5772102006295033`*^9}, {3.57721104576762*^9, 
   3.577211061023759*^9}, {3.57721112707898*^9, 3.5772112217029133`*^9}, {
   3.577211282233612*^9, 3.577211289276669*^9}, {3.577211493211051*^9, 
   3.57721154931549*^9}, {3.577211580823291*^9, 3.5772115960135927`*^9}, {
   3.577211682539126*^9, 3.577211720986651*^9}, {3.5772117655551243`*^9, 
   3.5772117691386013`*^9}, {3.577211949309287*^9, 3.5772119536623383`*^9}, {
   3.577212135577671*^9, 3.577212136060708*^9}, {3.577270410296666*^9, 
   3.5772704972032433`*^9}, {3.57727053507367*^9, 3.57727055637685*^9}, {
   3.577270591786982*^9, 3.577270607793735*^9}, {3.5772707930090446`*^9, 
   3.577270822473317*^9}, {3.577271984640587*^9, 3.5772719948245993`*^9}, {
   3.5772720989134417`*^9, 3.57727209971799*^9}, {3.578717328901553*^9, 
   3.578717360353963*^9}, {3.578718171272751*^9, 3.578718172972144*^9}, {
   3.5787182989963903`*^9, 3.5787183124856653`*^9}, 3.578718354895844*^9, {
   3.578738884097055*^9, 3.5787389135804157`*^9}, 3.5790177151223383`*^9, {
   3.57988969846507*^9, 3.579889706207966*^9}, {3.57989246503053*^9, 
   3.579892465988001*^9}, {3.582079485894281*^9, 3.582079529334958*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Chan Shattering", "Subsubsection",
 CellChangeTimes->{{3.577032073254735*^9, 3.5770320754105864`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3CSMapVertices", "[", 
   RowBox[{"shathulls_", ",", "hull_"}], "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "]"}], " ", ">", " ", "0"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"CH3DOutputFromTriangles", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"shathulls", "[", 
        RowBox[{"[", "#", "]"}], "]"}], "&"}], " ", "/@", " ", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"RotateToMin", "[", 
       RowBox[{"shathulls", "[", 
        RowBox[{"hull", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], ",", 
      RowBox[{"{", "}"}]}], "}"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 CellChangeTimes->{
  3.578889328256905*^9, {3.5788893978774357`*^9, 3.5788894732391863`*^9}, {
   3.578889789004684*^9, 3.578889795578456*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3ChanShattering", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"CHParts_:", "CH3GiftWrapping"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"GenChanShattering", "[", 
   RowBox[{
   "vts", ",", "First", ",", "CHParts", ",", "CH3PartialGiftWrapping", ",", 
    "CH3CSMapVertices"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577155899060878*^9, 3.5771559243873777`*^9}, {
  3.5771560441185093`*^9, 3.577156083254114*^9}, {3.578889219683118*^9, 
  3.578889229050774*^9}, {3.578889315422083*^9, 3.578889315971929*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["QuickHull", "Subsubsection",
 CellChangeTimes->{{3.576170009069625*^9, 3.5761700109402933`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Finds", " ", "the", " ", "triangles", " ", "visible", " ", "from", " ", 
    "a", " ", "new", " ", "vertex", " ", "and", " ", "replaces", " ", "them", 
    " ", "with", " ", "ones", " ", "containing", " ", "the", " ", "new", " ", 
    
    RowBox[{"vertex", ".", " ", "Also"}], " ", "returns", " ", "old", " ", 
    "triangles", " ", "and", " ", "new", " ", "triangles"}], " ", "*)"}], 
  " "}]], "Input",
 CellChangeTimes->{{3.577734688582683*^9, 3.577734749611578*^9}, {
  3.5778200513424397`*^9, 3.5778200659018106`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3VisTrisFromVert", "[", 
   RowBox[{
   "vts_", ",", "origtris_", ",", "ixtri_", ",", "ix_", ",", "trieds_", ",", 
    "edtri_", ",", "tetorient_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "oldtris", ",", "tristat", ",", "unchecked", ",", "ntris", ",", "ed", 
      ",", "k", ",", "tri", ",", "trix", ",", "newtris", ",", "updtris"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
       "Find", " ", "which", " ", "existing", " ", "triangles", " ", "are", 
        " ", "visible", " ", "from", " ", "the", " ", 
        RowBox[{"point", ".", " ", "0"}]}], " ", "=", " ", "unchecked"}], ",",
       " ", 
      RowBox[{"1", " ", "=", " ", "visible"}], ",", " ", 
      RowBox[{"2", " ", "=", " ", "invisible"}]}], " ", "*)"}], 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"oldtris", " ", "=", " ", 
      RowBox[{"{", "ixtri", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"tristat", "[", "ixtri", "]"}], " ", "=", " ", "0"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"unchecked", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"unchecked", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"unchecked", " ", "=", " ", "False"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ntris", " ", "=", " ", 
         RowBox[{"Length", "[", "oldtris", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"oldtris", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"tristat", "[", "tri", "]"}], " ", "\[Equal]", " ", 
              "0"}], ",", " ", 
             RowBox[{
              RowBox[{"unchecked", " ", "=", " ", "True"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"(*", " ", 
               RowBox[{"Look", " ", "at", " ", "a", " ", 
                RowBox[{"triangle", "'"}], "s", " ", "neighboring", " ", 
                "triangles"}], " ", "*)"}], "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"trix", " ", "=", " ", 
                  RowBox[{"edtri", "[", 
                   RowBox[{"Reverse", "[", "ed", "]"}], "]"}]}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"tristat", "[", "trix", "]"}], "]"}], " ", "===", 
                    " ", "tristat"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"oldtris", ",", "trix"}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"tristat", "[", "trix", "]"}], " ", "=", " ", 
                    "0"}]}]}], "]"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{"ed", ",", 
                  RowBox[{"trieds", "[", "tri", "]"}]}], "}"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"tristat", "[", "tri", "]"}], " ", "=", " ", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"tetorient", "[", 
                  RowBox[{"vts", "[", 
                   RowBox[{"[", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"tri", "[", 
                    RowBox[{"[", "2", "]"}], "]"}], ",", 
                    RowBox[{"tri", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"tri", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], ",", "ix"}], "}"}], "]"}],
                    "]"}], "]"}], ",", "1", ",", "2"}], "]"}]}]}]}], "]"}]}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "ntris"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "horizon", " ", "edges"}], ",", " ", 
       RowBox[{
       "those", " ", "between", " ", "visible", " ", "and", " ", "invisible", 
        " ", "triangles"}], ",", " ", 
       RowBox[{"and", " ", "the", " ", "new", " ", "triangles"}]}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"newtris", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"tristat", "[", "tri", "]"}], " ", "\[Equal]", " ", "1"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{"Do", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"trix", " ", "=", " ", 
             RowBox[{"edtri", "[", 
              RowBox[{"Reverse", "[", "ed", "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"tristat", "[", "trix", "]"}], "\[Equal]", "2"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"newtris", ",", 
                RowBox[{"Append", "[", 
                 RowBox[{"ed", ",", "ix"}], "]"}]}], "]"}]}], 
             "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"ed", ",", 
             RowBox[{"trieds", "[", "tri", "]"}]}], "}"}]}], 
          "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "oldtris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"TriEdgeRefs", "[", 
        RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Remove", " ", "the", " ", "old", " ", "triangles"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"oldtris", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"oldtris", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"tristat", "[", "#", "]"}], "\[Equal]", "1"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"updtris", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Complement", "[", 
         RowBox[{"origtris", ",", "oldtris"}], "]"}], ",", "newtris"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"updtris", ",", "oldtris", ",", "newtris"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.577734751243737*^9, 3.577734830148821*^9}, {
   3.5777348942555723`*^9, 3.577734894814622*^9}, {3.577734972864871*^9, 
   3.577734973330841*^9}, {3.5777358588776293`*^9, 3.577735963106133*^9}, {
   3.577736014735372*^9, 3.5777360989303827`*^9}, {3.577736204915154*^9, 
   3.577736345795212*^9}, {3.577736400415201*^9, 3.5777364101486*^9}, {
   3.577736541879621*^9, 3.577736560378553*^9}, {3.5777968818014793`*^9, 
   3.5777969007178907`*^9}, {3.577820034174507*^9, 3.577820035566545*^9}, {
   3.577886910302047*^9, 3.577886951935711*^9}, {3.57788699814408*^9, 
   3.577887010951013*^9}, {3.577887048152553*^9, 3.577887122112378*^9}, 
   3.578930920429802*^9, {3.579096638212311*^9, 3.57909663887047*^9}, {
   3.5795988756856327`*^9, 3.579598878043269*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", " ", "indices", " ", "of", " ", "positive"}], "-", 
   RowBox[{"side", " ", "vertices", " ", "and", " ", "the", " ", "maximum"}], 
   "-", 
   RowBox[{"distance", " ", "one"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576797278797093*^9, 3.5767972875907707`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3QHExternalVertices", "[", 
   RowBox[{"vts_", ",", "vtixs_", ",", "tri_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"tdval", ",", "ixvts", ",", "ixpos", ",", "ixmax"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"tdval", " ", "=", " ", 
      RowBox[{"TetVolumeData3D", "[", 
       RowBox[{"vts", "[", 
        RowBox[{"[", 
         RowBox[{"tri", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"2", ",", "1", ",", "3"}], "}"}], "]"}], "]"}], "]"}], 
        "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"TetVolumeEval3D", "[", 
           RowBox[{"tdval", ",", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "#", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
        " ", "vtixs"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixpos", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"Select", "[", 
        RowBox[{"ixvts", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ">", "0"}], "&"}]}], "]"}]}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixmax", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "ixvts", "]"}], " ", ">", " ", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "@", " ", 
         RowBox[{"MaxBySortByFunc", "[", 
          RowBox[{"ixvts", ",", 
           RowBox[{
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", "Null"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"ixpos", ",", "ixmax"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBmIQrXrz7uovS145Oss17QDRCwwv7AHR09a57AXRIXye
B0F0wpmCUyD6TcmLcyA68Z/EeRDNFnMp7BuQvuUiHQGija5M5DNcBtQ3/y2Y
ttrpPtUISP8tewymJcrl5oFoto/f14LoN+nO+0G0nvbKQyD6guSRUyD69rI/
YPrJpsBLIPqEwx0wbfN52WmZ5a8chZbGXADREknBymZA+t7TOXogehtDhAmI
rvmyGUwHZAuKPQbSf6p/gOlNu94kq6545RhetT0FRP/T9c8E0aEOWTkg+lp4
VimIdgnuBNOxztFx7kDaY2lbJogGANTRpT8=
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3QuickHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "hullvts", ",", "hulltris", ",", "hull", ",", "ix1", ",", 
      "ix2", ",", "ixvts", ",", "ixvtria", ",", "ivtmx", ",", "ix3", ",", 
      "tdval", ",", "ixvtetv", ",", "ivtng", ",", "ivtps", ",", "ixng", ",", 
      "ixps", ",", "tvng", ",", "tvps", ",", "ix4", ",", "tri", ",", "trieds",
       ",", "edtri", ",", "extvts", ",", "k", ",", "kx", ",", "evs", ",", 
      "ixmax", ",", "oldtris", ",", "newtris", ",", "gwvts", ",", "gwtris"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "3"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hullvts", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hulltris", " ", "=", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nvts", " ", "\[Equal]", " ", "3"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "3", ",", "2"}], "}"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"hullvts", ",", "hulltris"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Starting", " ", 
       RowBox[{"points", ":", " ", 
        RowBox[{"first", " ", "two"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ix1", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ix2", " ", "=", " ", 
      RowBox[{"IndexMax", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Starting", " ", 
       RowBox[{"points", ":", " ", 
        RowBox[{"third", " ", "one"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{
        RowBox[{"Range", "[", "nvts", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"ix1", ",", "ix2"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvtria", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"TriArea3DSqr", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", 
             RowBox[{"{", 
              RowBox[{"ix1", ",", "ix2", ",", "#"}], "}"}], "]"}], "]"}], 
           "]"}]}], "}"}], "&"}], " ", "/@", " ", "ixvts"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ivtmx", " ", "=", " ", 
      RowBox[{"MaxBySortByFunc", "[", 
       RowBox[{"ixvtria", ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ix3", " ", "=", " ", 
      RowBox[{"ivtmx", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Starting", " ", 
       RowBox[{"points", ":", " ", 
        RowBox[{"fourth", " ", "one"}]}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{"ixvts", ",", 
        RowBox[{"{", "ix3", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tdval", " ", "=", " ", 
      RowBox[{"TetVolumeData3D", "[", 
       RowBox[{"vts", "[", 
        RowBox[{"[", 
         RowBox[{"{", 
          RowBox[{"ix1", ",", "ix2", ",", "ix3"}], "}"}], "]"}], "]"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvtetv", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"TetVolumeEval3D", "[", 
           RowBox[{"tdval", ",", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "#", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
        " ", "ixvts"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"ixvtetv", " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"#", ",", 
            RowBox[{"TetVolume3D", "[", 
             RowBox[{"vts", "[", 
              RowBox[{"[", 
               RowBox[{"{", 
                RowBox[{"ix1", ",", "ix2", ",", "ix3", ",", "#"}], "}"}], 
               "]"}], "]"}], "]"}]}], "}"}], "&"}], " ", "/@", " ", 
         "ixvts"}]}], ";"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ivtng", " ", "=", " ", 
      RowBox[{"MinBySortByFunc", "[", 
       RowBox[{"ixvtetv", ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ivtps", " ", "=", " ", 
      RowBox[{"MaxBySortByFunc", "[", 
       RowBox[{"ixvtetv", ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixng", " ", "=", " ", 
      RowBox[{"ivtng", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixps", " ", "=", " ", 
      RowBox[{"ivtps", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tvng", " ", "=", " ", 
      RowBox[{"-", " ", 
       RowBox[{"ivtng", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tvps", " ", "=", " ", 
      RowBox[{"ivtps", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hulltris", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"tvps", " ", "\[GreaterEqual]", " ", "tvng"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ix4", " ", "=", " ", "ixps"}], ";", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"ix1", ",", "ix2", ",", "ix3"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix1", ",", "ix3", ",", "ix4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix3", ",", "ix2", ",", "ix4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix2", ",", "ix1", ",", "ix4"}], "}"}]}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ix4", " ", "=", " ", "ixng"}], ";", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"ix2", ",", "ix1", ",", "ix3"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix3", ",", "ix1", ",", "ix4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix2", ",", "ix3", ",", "ix4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"ix1", ",", "ix2", ",", "ix4"}], "}"}]}], "}"}]}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{"ixvts", ",", 
        RowBox[{"{", "ix4", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "outward", " ", "vertices", " ", "for", " ", 
       "each", " ", "triangle"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"TriEdgeRefs", "[", 
         RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"extvts", "[", "tri", "]"}], " ", "=", "\[IndentingNewLine]", 
         RowBox[{"CH3QHExternalVertices", "[", 
          RowBox[{"vts", ",", "ixvts", ",", "tri"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "hulltris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Search", " ", "for", " ", "a", " ", "triangle", " ", "with", " ", 
         "outward", " ", "points"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"hulltris", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"evs", ",", "ixmax"}], "}"}], " ", "=", " ", 
            RowBox[{"extvts", "[", "tri", "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "evs", "]"}], " ", ">", " ", "0"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
               RowBox[{"Found", " ", "a", " ", "non"}], "-", 
               RowBox[{"empty", " ", "outside", " ", "volume"}]}], " ", 
              "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"kx", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"Length", "[", "hulltris", "]"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"kx", " ", "===", " ", "Null"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{"Get", " ", "updated", " ", "hull", " ", "triangles"}], ",",
           " ", 
          RowBox[{"old", " ", "triangles", " ", 
           RowBox[{"(", "removed", ")"}]}], ",", " ", 
          RowBox[{"and", " ", "new", " ", "triangles", " ", 
           RowBox[{"(", "added", ")"}]}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"hulltris", ",", "oldtris", ",", "newtris"}], "}"}], " ", 
         "=", " ", 
         RowBox[{"CH3VisTrisFromVert", "[", 
          RowBox[{
          "vts", ",", "hulltris", ",", "tri", ",", "ixmax", ",", "trieds", 
           ",", "edtri", ",", "TetOrient3D"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Find", " ", "the", " ", "outward", " ", "points", " ", "for", " ", 
          "the", " ", "new", " ", 
          RowBox[{"triangles", ".", " ", "Start"}], " ", "with", " ", "all", 
          " ", "those", " ", "for", " ", "the", " ", "old", " ", 
          RowBox[{"triangles", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"evs", " ", "=", " ", 
         RowBox[{"Union", " ", "@@", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"extvts", "[", "#", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
            "oldtris"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"evs", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"evs", ",", 
           RowBox[{"{", "ixmax", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"extvts", "[", "tri", "]"}], " ", "=", 
           "\[IndentingNewLine]", 
           RowBox[{"CH3QHExternalVertices", "[", 
            RowBox[{"vts", ",", "evs", ",", "tri"}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"CH3DOutputFromTriangles", "[", "hulltris", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5772715233566513`*^9, 3.5772715306380663`*^9}, {
   3.5772719027664022`*^9, 3.577271909310153*^9}, {3.57727194106173*^9, 
   3.5772719633082533`*^9}, {3.577273458163349*^9, 3.5772734699933577`*^9}, {
   3.577274758002425*^9, 3.5772747711822567`*^9}, {3.577274817726593*^9, 
   3.577274876456511*^9}, {3.5772750129269543`*^9, 3.577275132024334*^9}, {
   3.577275286142913*^9, 3.57727543592593*^9}, {3.577275575089862*^9, 
   3.5772755754957857`*^9}, {3.577275667086937*^9, 3.577275668146021*^9}, {
   3.577275716203459*^9, 3.577275747290374*^9}, {3.5772763647541733`*^9, 
   3.577276400853549*^9}, {3.577276642035842*^9, 3.577276768549839*^9}, {
   3.577276895103734*^9, 3.577277097484478*^9}, {3.577277154351307*^9, 
   3.57727720870288*^9}, {3.5772772498866253`*^9, 3.577277445520997*^9}, {
   3.5772775104707108`*^9, 3.5772775681680107`*^9}, {3.57727760298829*^9, 
   3.577277651473817*^9}, {3.57727770731744*^9, 3.5772777293139763`*^9}, {
   3.577277801553149*^9, 3.577277824198062*^9}, {3.5772779104818563`*^9, 
   3.577277915114839*^9}, {3.57727813980192*^9, 3.5772782007879972`*^9}, {
   3.577328059962373*^9, 3.577328071111112*^9}, 3.577328105369628*^9, {
   3.577330335126172*^9, 3.5773303488391323`*^9}, {3.5773362611896467`*^9, 
   3.577336273093111*^9}, {3.577338060540394*^9, 3.577338068089727*^9}, {
   3.577339348385499*^9, 3.577339487025269*^9}, {3.577339650416666*^9, 
   3.577339748227141*^9}, {3.5773397959837713`*^9, 3.5773399183375587`*^9}, {
   3.57734011561581*^9, 3.57734015140549*^9}, {3.577340207617642*^9, 
   3.57734034215666*^9}, {3.577340493957655*^9, 3.57734069368784*^9}, {
   3.577340737782763*^9, 3.577340755289433*^9}, {3.57734079804811*^9, 
   3.577340853525011*^9}, {3.577340903008871*^9, 3.577340915137217*^9}, {
   3.577340976740037*^9, 3.577340992594479*^9}, {3.577341040661716*^9, 
   3.577341050414876*^9}, {3.577341108855582*^9, 3.577341150852865*^9}, {
   3.5773412198648977`*^9, 3.5773412687330427`*^9}, {3.577341323428969*^9, 
   3.5773413273167677`*^9}, {3.577341362749485*^9, 3.577341391519814*^9}, {
   3.577341439480196*^9, 3.5773414440240097`*^9}, {3.5773415128792753`*^9, 
   3.577341528277687*^9}, {3.577341584740284*^9, 3.577341629847632*^9}, {
   3.577341723351638*^9, 3.577341754476304*^9}, {3.577347273266108*^9, 
   3.577347335115987*^9}, {3.5773473973549623`*^9, 3.57734739785504*^9}, {
   3.577375900363183*^9, 3.577375951073432*^9}, {3.577682123680624*^9, 
   3.577682131319031*^9}, {3.5776821686432943`*^9, 3.577682171572123*^9}, {
   3.577685047116561*^9, 3.577685047585422*^9}, {3.577735018466374*^9, 
   3.577735097512094*^9}, {3.577735164865494*^9, 3.577735177955162*^9}, {
   3.577735634856648*^9, 3.5777356364539843`*^9}, {3.577735669657196*^9, 
   3.57773582316146*^9}, {3.5777373090871077`*^9, 3.5777373095387793`*^9}, {
   3.577751948000215*^9, 3.577751951028061*^9}, {3.577799770948929*^9, 
   3.577799788709231*^9}, {3.577799973289678*^9, 3.5777999742160263`*^9}, {
   3.577815329363887*^9, 3.577815433987494*^9}, {3.577815482392846*^9, 
   3.577815516474477*^9}, {3.577815600627885*^9, 3.577815636370126*^9}, {
   3.577815688992552*^9, 3.5778157022355947`*^9}, 3.577815765618318*^9, {
   3.5778181333835382`*^9, 3.577818161514567*^9}, {3.577820093590819*^9, 
   3.577820158798851*^9}, {3.577820307383466*^9, 3.5778203114852457`*^9}, {
   3.57782036430247*^9, 3.577820460561009*^9}, {3.577887154858625*^9, 
   3.577887179536903*^9}, {3.5778872283870363`*^9, 3.577887303823976*^9}, {
   3.577887351100614*^9, 3.577887372525124*^9}, {3.577887409640601*^9, 
   3.577887413757224*^9}, 3.578959135568552*^9, {3.57959889158247*^9, 
   3.5795988937162952`*^9}, {3.5815465792953587`*^9, 3.581546590462749*^9}, {
   3.581546646699883*^9, 3.5815466615128517`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Incremental", "Subsubsection",
 CellChangeTimes->{{3.576169999227941*^9, 3.576170001131818*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GenIncremental", "[", 
   RowBox[{
   "vts_", ",", "tetorient_", ",", "tetdatafunc_", ",", "tetdataorient_"}], 
   "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "hullvts", ",", "hulltris", ",", "hull", ",", "tetdata", 
      ",", "trieds", ",", "edtri", ",", "tri", ",", "ix", ",", "k", ",", "kx",
       ",", "tdval", ",", "oldtris", ",", "newtris"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hullvts", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hulltris", " ", "=", " ", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"hullvts", ",", "hulltris"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"hulltris", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "3", ",", "2"}], "}"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"tetdata", "[", "tri", "]"}], " ", "=", " ", 
         RowBox[{"tetdatafunc", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", 
            RowBox[{"tri", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"2", ",", "1", ",", "3"}], "}"}], "]"}], "]"}], "]"}], 
           "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"TriEdgeRefs", "[", 
         RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "hulltris"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"hulltris", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"tdval", " ", "=", " ", 
            RowBox[{"tetdata", "[", "tri", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"tetdataorient", "[", 
              RowBox[{"tdval", ",", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "ix", "]"}], "]"}]}], "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"kx", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"Length", "[", "hulltris", "]"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"kx", " ", "===", " ", "Null"}], ",", 
          RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"hulltris", ",", "oldtris", ",", "newtris"}], "}"}], " ", 
         "=", " ", 
         RowBox[{"CH3VisTrisFromVert", "[", 
          RowBox[{
          "vts", ",", "hulltris", ",", "tri", ",", "ix", ",", "trieds", ",", 
           "edtri", ",", "tetorient"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "hulltris", "]"}], "\[Equal]", "0"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tetdata", "[", "tri", "]"}], " ", "=", " ", 
           RowBox[{"tetdatafunc", "[", 
            RowBox[{"vts", "[", 
             RowBox[{"[", 
              RowBox[{"tri", "[", 
               RowBox[{"[", 
                RowBox[{"{", 
                 RowBox[{"2", ",", "1", ",", "3"}], "}"}], "]"}], "]"}], 
              "]"}], "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", "4", ",", "nvts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"CH3DOutputFromTriangles", "[", "hulltris", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.576171150286549*^9, 3.576171213955037*^9}, {
  3.576171250443317*^9, 3.57617125236271*^9}, {3.576171282639772*^9, 
  3.5761712872994347`*^9}, {3.5777957158386383`*^9, 3.577795801563661*^9}, {
  3.577795892550198*^9, 3.5777959061333723`*^9}, {3.577795964489244*^9, 
  3.5777960326742277`*^9}, {3.5777960981143847`*^9, 3.57779625102209*^9}, {
  3.577796281086172*^9, 3.577796310283949*^9}, {3.5777966626505613`*^9, 
  3.5777966792637653`*^9}, {3.5778878829054947`*^9, 3.577888080779984*^9}, {
  3.5778881839464817`*^9, 3.5778881845200663`*^9}, {3.578931122111272*^9, 
  3.5789311414889917`*^9}, {3.5795036017582912`*^9, 3.579503614495453*^9}, {
  3.579503644607979*^9, 3.579503684471966*^9}, {3.579598904627409*^9, 
  3.579598909090702*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3Incremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CH3GenIncremental", "[", 
   RowBox[{
   "vts", ",", "TetOrient3D", ",", "TetVolumeData3D", ",", 
    "TetOrientEval3D"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577795854113966*^9, 3.57779588796032*^9}, {
  3.57788809490821*^9, 3.5778881078365507`*^9}, {3.578931105906784*^9, 
  3.578931114410562*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3SphIncremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CH3GenIncremental", "[", 
   RowBox[{
   "vts", ",", "TetOrient3Sph", ",", "TetVolumeData3Sph", ",", 
    "TetOrientEval3Sph"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577795854113966*^9, 3.57779588796032*^9}, {
  3.57788809490821*^9, 3.5778881078365507`*^9}, {3.578931105906784*^9, 
  3.578931114410562*^9}, {3.579069093893446*^9, 3.5790690961145973`*^9}, {
  3.57906920860681*^9, 3.579069268443915*^9}, {3.581582372127664*^9, 
  3.581582374866098*^9}, {3.581582619516321*^9, 3.581582629184053*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Verification and Comparison", "Subsubsection",
 CellChangeTimes->{{3.576170587403945*^9, 3.576170594241843*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3GenLocCheck", "[", 
   RowBox[{"vts_", ",", "hulltris_", ",", "tetorient_"}], "]"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "hlen", ",", "nvts", ",", "inside", ",", "k", ",", "tri", ",", "ix"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hlen", " ", "=", 
      RowBox[{"Length", "[", "hulltris", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"tri", " ", "=", "  ", 
         RowBox[{"hulltris", "[", 
          RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"MemberQ", "[", 
           RowBox[{"tri", ",", "ix"}], "]"}], ",", "\[IndentingNewLine]", 
          "True", ",", "\[IndentingNewLine]", 
          RowBox[{"tetorient", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", 
             RowBox[{"Append", "[", 
              RowBox[{"tri", ",", "ix"}], "]"}], "]"}], "]"}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", "nvts"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "hlen"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input",
 CellChangeTimes->{{3.576160476801395*^9, 3.576160514621572*^9}, {
  3.5770576308125973`*^9, 3.5770576345293617`*^9}, {3.577057732343148*^9, 
  3.577057828980538*^9}, {3.5770579444703608`*^9, 3.577057948883271*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3LocCheck", "[", 
   RowBox[{"vts_", ",", "hulltris_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CH3GenLocCheck", "[", 
   RowBox[{"vts", ",", "hulltris", ",", "TetOrient3D"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5757866489572144`*^9, 3.575786689078209*^9}, {
  3.5757867327050543`*^9, 3.575786823688817*^9}, {3.5757869408346767`*^9, 
  3.5757871410427647`*^9}, {3.575990182446587*^9, 3.575990185070249*^9}, {
  3.576012663354772*^9, 3.576012751444072*^9}, {3.576012836102601*^9, 
  3.576012851222892*^9}, {3.576012918772579*^9, 3.576013049378448*^9}, {
  3.576013363093774*^9, 3.576013370238529*^9}, {3.576160522235409*^9, 
  3.576160531652213*^9}, {3.5770580120173264`*^9, 3.5770580311452303`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHHySpLocCheck", "[", 
   RowBox[{"vts_", ",", "hulltris_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CH3GenLocCheck", "[", 
   RowBox[{"vts", ",", "hulltris", ",", "TetOrientHySp"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5757866489572144`*^9, 3.575786689078209*^9}, {
  3.5757867327050543`*^9, 3.575786823688817*^9}, {3.5757869408346767`*^9, 
  3.5757871410427647`*^9}, {3.575990182446587*^9, 3.575990185070249*^9}, {
  3.576012663354772*^9, 3.576012751444072*^9}, {3.576012836102601*^9, 
  3.576012851222892*^9}, {3.576012918772579*^9, 3.576013049378448*^9}, {
  3.576013363093774*^9, 3.576013370238529*^9}, {3.576160522235409*^9, 
  3.576160531652213*^9}, {3.5770580120173264`*^9, 3.5770580311452303`*^9}, {
  3.581578057361662*^9, 3.5815780681510963`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3Algorithms", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "CH3GiftWrapping", ",", "CH3DivideConquer", ",", "CH3ChanShattering", ",",
      "CH3QuickHull", ",", "CH3Incremental"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5770581874986887`*^9, 3.5770582088330297`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3SphAlgorithms", " ", "=", " ", 
   RowBox[{"{", "CH3SphIncremental", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5815780003089447`*^9, 3.581578005993802*^9}, {
  3.581582685746037*^9, 3.5815826927682323`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ND", "Subsection",
 CellChangeTimes->{{3.5790697009119043`*^9, 3.579069715430483*^9}, {
   3.57910219570503*^9, 3.579102210891635*^9}, 3.579141135041875*^9}],

Cell[CellGroupData[{

Cell["Gift Wrapping", "Subsubsection",
 CellChangeTimes->{{3.5812372708657827`*^9, 3.5812372731034203`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNGWMakeProjection", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"refpt", ",", "diffs", ",", "trdiffs", ",", "pjmat"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"refpt", " ", "=", " ", 
      RowBox[{"Mean", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"diffs", " ", "=", " ", 
      RowBox[{"SubFirstFromVecSet", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"trdiffs", " ", "=", " ", 
      RowBox[{"Transpose", "[", "diffs", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"pjmat", " ", "=", " ", 
      RowBox[{
       RowBox[{"IdentityMatrix", "[", 
        RowBox[{"Length", "[", 
         RowBox[{"First", "[", "vts", "]"}], "]"}], "]"}], " ", "-", " ", 
       RowBox[{"trdiffs", ".", 
        RowBox[{"Inverse", "[", 
         RowBox[{"diffs", ".", "trdiffs"}], "]"}], ".", "diffs"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"refpt", ",", "pjmat"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.577045701021409*^9, 3.577045729284752*^9}, {
  3.577045842561447*^9, 3.5770458778144407`*^9}, {3.5770459814333487`*^9, 
  3.5770460006435423`*^9}, {3.581329282856481*^9, 3.581329345241271*^9}, {
  3.581329438707962*^9, 3.581329557953767*^9}, {3.581385589480587*^9, 
  3.5813856332537527`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CHNGWPartProj", "[", 
    RowBox[{"prj_", ",", "dir_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"prj", "[", 
     RowBox[{"[", "2", "]"}], "]"}], ".", "dir"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.578738778698804*^9, 3.578738806583222*^9}, {
  3.581329286748201*^9, 3.581329287258972*^9}, {3.581329571222272*^9, 
  3.581329574001437*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNGWProjectOut", "[", 
   RowBox[{"prj_", ",", "vt_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CHNGWPartProj", "[", 
   RowBox[{"prj", ",", 
    RowBox[{"vt", " ", "-", " ", 
     RowBox[{"prj", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5770460077664633`*^9, 3.577046023043132*^9}, {
   3.57704612346692*^9, 3.577046153932314*^9}, {3.577046209903761*^9, 
   3.5770462560934887`*^9}, {3.5787388182164917`*^9, 3.578738830411418*^9}, {
   3.581329290478835*^9, 3.581329290929428*^9}, 3.581329579176321*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNPartialGiftWrapping", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"indef_:", "True"}], ",", 
    RowBox[{"nmax_:", "0"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ndims", ",", "ix0", ",", "vt", ",", "ixvts", ",", "ixvts1",
       ",", "ixvts11", ",", "ix1", ",", "face", ",", "dvts", ",", "k", ",", 
      "prj", ",", "ix", ",", "bdrysbfcs", ",", "bdryoppoverts", ",", 
      "iscomplete", ",", "initial", ",", "n", ",", "res", ",", "sbfc", ",", 
      "prevdir", ",", "fcsubfcs", ",", "fcoppoverts", ",", "bdrysel", ",", 
      "fcsel", ",", "pos", ",", "hullfcs"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vts", "]"}], " ", "\[Equal]", " ", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"False", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", "}"}]}], "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nvts", ",", "ndims"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "\[LessEqual]", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"False", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", "}"}]}], "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "<", " ", "ndims"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"True", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Range", "[", "nvts", "]"}], ",", 
            RowBox[{"{", "}"}]}], "}"}]}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"indef", " ", "||", " ", 
          RowBox[{"nmax", " ", ">", " ", "0"}]}], ")"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"False", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", "}"}]}], "}"}]}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "a", " ", "farthest"}], "-", 
       RowBox[{"out", " ", "point"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ix0", " ", "=", " ", 
      RowBox[{"IndexMin", "[", 
       RowBox[{"First", " ", "/@", " ", "vts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vt", " ", "=", " ", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "ix0", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "closest"}], "-", "to", "-", 
       RowBox[{
       "outward", " ", "point", " ", "using", " ", "its", " ", 
        "direction"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"AddIndices", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"ixvts", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "ix0"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts11", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{"First", "[", 
           RowBox[{"Normalize", "[", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "-", "vt"}], "]"}], "]"}]}], 
         "}"}], "&"}], " ", "/@", " ", "ixvts1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ix1", " ", "=", " ", 
      RowBox[{"First", "[", 
       RowBox[{"MinBySortByFunc", "[", 
        RowBox[{"ixvts11", ",", 
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"ixvts1", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "ix1"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"face", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"ix0", ",", "ix1"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "next", " ", "closest"}], "-", "to", 
       "-", 
       RowBox[{"outward", " ", "ones"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"prj", " ", "=", " ", 
         RowBox[{"CHNGWMakeProjection", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", "face", "]"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ixvts11", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"First", "[", 
              RowBox[{"Normalize", "[", 
               RowBox[{"CHNGWProjectOut", "[", 
                RowBox[{"prj", ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "]"}]}], 
            "}"}], "&"}], " ", "/@", " ", "ixvts1"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ix", " ", "=", " ", 
         RowBox[{"First", "[", 
          RowBox[{"MinBySortByFunc", "[", 
           RowBox[{"ixvts11", ",", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ixvts1", " ", "=", " ", 
         RowBox[{"Select", "[", 
          RowBox[{"ixvts1", ",", 
           RowBox[{
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "ix"}], "&"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"face", ",", "ix"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"ndims", "-", "2"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Adjust", " ", "the", " ", "orientation", " ", "of", " ", "the", " ", 
       "initial", " ", "face"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"face", " ", "=", " ", 
      RowBox[{"Sort", "[", "face", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"First", "[", 
         RowBox[{"Cross", " ", "@@", " ", 
          RowBox[{"SubFirstFromVecSet", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", "face", "]"}], "]"}], "]"}]}], "]"}], " ", "<", " ", 
        "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{"face", " ", "=", " ", 
        RowBox[{"SimplexFlip", "[", "face", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"iscomplete", " ", "=", " ", "False"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"initial", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"n", "=", "0"}], ",", " ", 
         RowBox[{"indef", " ", "||", " ", 
          RowBox[{"(", 
           RowBox[{"n", "<", "nmax"}], ")"}]}], ",", 
         RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{"initial", ",", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "Setup", " ", "for", " ", "the", " ", "initial", " ", "face"}], 
             " ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Sow", "[", "face", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"bdrysbfcs", " ", "=", " ", 
              RowBox[{"SimplexSubs", "[", "face", "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"bdryoppoverts", " ", "=", " ", 
              RowBox[{"Reverse", "[", "face", "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"initial", " ", "=", " ", "False"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"Continue", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Find", " ", "the", " ", "remaining", " ", "faces"}], " ", 
           "*)"}], "\[IndentingNewLine]", 
          RowBox[{"sbfc", " ", "=", " ", 
           RowBox[{"bdrysbfcs", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"bdryoppoverts", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"prj", " ", "=", " ", 
           RowBox[{"CHNGWMakeProjection", "[", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "sbfc", "]"}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"prevdir", " ", "=", " ", 
           RowBox[{"-", " ", 
            RowBox[{"CHNGWProjectOut", "[", 
             RowBox[{"prj", ",", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "ix", "]"}], "]"}]}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Select", " ", "the", " ", "vertex", " ", 
            RowBox[{"that", "'"}], "s", " ", "closest", " ", "to", " ", 
            "that", " ", 
            RowBox[{"direction", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
          
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{"Select", "[", 
            RowBox[{"ixvts", ",", 
             RowBox[{
              RowBox[{"!", 
               RowBox[{"MemberQ", "[", 
                RowBox[{"sbfc", ",", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "&"}]}], "]"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"ixvts1", " ", "=", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"prevdir", ".", 
                RowBox[{"Normalize", "[", 
                 RowBox[{"CHNGWProjectOut", "[", 
                  RowBox[{"prj", ",", 
                   RowBox[{"#", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}]}]}], "}"}],
              "&"}], " ", "/@", " ", "ixvts1"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"First", "[", 
            RowBox[{"MaxBySortByFunc", "[", 
             RowBox[{"ixvts1", ",", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "Compose", " ", "the", " ", "new", " ", "face", " ", "from", " ", 
            "that", " ", "vertex"}], " ", "*)"}], "\[IndentingNewLine]", 
          RowBox[{"face", " ", "=", " ", 
           RowBox[{"SimplexCanon", "[", 
            RowBox[{"Append", "[", 
             RowBox[{
              RowBox[{"SimplexFlip", "[", "sbfc", "]"}], ",", "ix"}], "]"}], 
            "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"Sow", "[", "face", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"fcsubfcs", " ", "=", " ", 
           RowBox[{"SimplexSubs", "[", "face", "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"fcoppoverts", " ", "=", " ", 
           RowBox[{"Reverse", "[", "face", "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
            RowBox[{
            "Add", " ", "its", " ", "subfaces", " ", "to", " ", "the", " ", 
             "list", " ", "of", " ", "subfaces"}], ",", " ", 
            RowBox[{"but", " ", "subtract", " ", "where", " ", "they", " ", 
             RowBox[{"duplicate", "."}]}]}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"bdrysel", " ", "=", " ", 
           RowBox[{"ConstantArray", "[", 
            RowBox[{"True", ",", 
             RowBox[{"Length", "[", "bdrysbfcs", "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"fcsel", " ", "=", " ", 
           RowBox[{"ConstantArray", "[", 
            RowBox[{"True", ",", 
             RowBox[{"Length", "[", "face", "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Do", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"pos", " ", "=", " ", 
              RowBox[{"Position", "[", 
               RowBox[{"bdrysbfcs", ",", 
                RowBox[{"SimplexFlip", "[", 
                 RowBox[{"fcsubfcs", "[", 
                  RowBox[{"[", "k", "]"}], "]"}], "]"}]}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Length", "[", "pos", "]"}], " ", ">", " ", "0"}], 
               ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"ix", " ", "=", " ", 
                 RowBox[{"pos", "[", 
                  RowBox[{"[", 
                   RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"bdrysel", "[", 
                  RowBox[{"[", "ix", "]"}], "]"}], " ", "=", " ", "False"}], 
                ";", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"fcsel", "[", 
                  RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", 
                 "False"}]}]}], "\[IndentingNewLine]", "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"k", ",", 
              RowBox[{"Length", "[", "face", "]"}]}], "}"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"bdrysbfcs", " ", "=", " ", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"SelWithList", "[", 
              RowBox[{"bdrysel", ",", "bdrysbfcs"}], "]"}], ",", 
             RowBox[{"SelWithList", "[", 
              RowBox[{"fcsel", ",", "fcsubfcs"}], "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"bdryoppoverts", " ", "=", " ", 
           RowBox[{"Join", "[", 
            RowBox[{
             RowBox[{"SelWithList", "[", 
              RowBox[{"bdrysel", ",", "bdryoppoverts"}], "]"}], ",", 
             RowBox[{"SelWithList", "[", 
              RowBox[{"fcsel", ",", "fcoppoverts"}], "]"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "bdrysbfcs", "]"}], " ", "\[Equal]", " ", 
             "0"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"iscomplete", " ", "=", " ", "True"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", 
           "]"}]}]}], "\[IndentingNewLine]", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullfcs", " ", "=", " ", 
      RowBox[{"res", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"iscomplete", ",", 
       RowBox[{"CHNDOutputFromSimplexes", "[", "hullfcs", "]"}]}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.581375112120625*^9, 3.581375148588161*^9}, {
   3.581375181723575*^9, 3.5813752579137163`*^9}, {3.581375421801217*^9, 
   3.5813754408125143`*^9}, {3.581375506861071*^9, 3.581375530632083*^9}, {
   3.581375593613615*^9, 3.581375656423942*^9}, {3.58137573451338*^9, 
   3.581375738119916*^9}, {3.5813758605655203`*^9, 3.581375871873097*^9}, {
   3.581375943188662*^9, 3.581375984435001*^9}, 3.5813760844325047`*^9, {
   3.581376188372218*^9, 3.581376255002459*^9}, {3.581376372591445*^9, 
   3.581376402751897*^9}, {3.581376443636101*^9, 3.581376455983747*^9}, {
   3.58137656128541*^9, 3.581376583178956*^9}, {3.581376953394163*^9, 
   3.581377061302505*^9}, {3.581377099218789*^9, 3.58137717689079*^9}, {
   3.581377309232258*^9, 3.581377390009479*^9}, {3.5813774256608953`*^9, 
   3.581377428389275*^9}, {3.5813774645511208`*^9, 3.5813774724482927`*^9}, {
   3.5813839977495127`*^9, 3.581384038583454*^9}, {3.5813856617750607`*^9, 
   3.581385744790763*^9}, {3.581387459973802*^9, 3.5813874877199717`*^9}, {
   3.581387520889*^9, 3.581387554996626*^9}, {3.581387983065596*^9, 
   3.581387992894238*^9}, {3.581389227754714*^9, 3.581389229764696*^9}, {
   3.581389359544228*^9, 3.581389364051148*^9}, {3.581389635990954*^9, 
   3.581389710443096*^9}, {3.581390924121817*^9, 3.58139104240936*^9}, {
   3.5813910961853857`*^9, 3.5813911108327093`*^9}, {3.5813913386713448`*^9, 
   3.581391340384801*^9}, {3.581391914313984*^9, 3.581391928211814*^9}, {
   3.581391991762169*^9, 3.581392045835424*^9}, {3.581392166021839*^9, 
   3.581392267256588*^9}, {3.5813923042439404`*^9, 3.5813923835699778`*^9}, {
   3.581392571815893*^9, 3.581392663321177*^9}, {3.581392762757498*^9, 
   3.581392821716477*^9}, {3.5813933010073147`*^9, 3.581393345953425*^9}, {
   3.5813933893222313`*^9, 3.581393407102302*^9}, {3.581393443063552*^9, 
   3.5813935188193283`*^9}, {3.581393723708027*^9, 3.5813937720388603`*^9}, {
   3.581393838494894*^9, 3.5813938405099907`*^9}, {3.581394143632573*^9, 
   3.581394146202332*^9}, {3.581394580808338*^9, 3.5813945937036037`*^9}, {
   3.5813947346693287`*^9, 3.581394763416754*^9}, {3.5813954447639437`*^9, 
   3.581395458326008*^9}, {3.581395921146534*^9, 3.581395932173934*^9}, {
   3.581509183385284*^9, 3.5815091871679277`*^9}, {3.5815118459059963`*^9, 
   3.581511846470339*^9}, {3.5815122174384127`*^9, 3.581512273180311*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNGiftWrapping", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"iscomplete", ",", "hull"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"iscomplete", ",", "hull"}], "}"}], " ", "=", " ", 
      RowBox[{"CHNPartialGiftWrapping", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"iscomplete", ",", "hull", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", "}"}]}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5768655122210007`*^9, 3.576865534735012*^9}, {
   3.576874484686337*^9, 3.5768744858445187`*^9}, {3.577054788749959*^9, 
   3.577054789166621*^9}, {3.578886303151701*^9, 3.578886349223419*^9}, 
   3.5813737996720037`*^9, {3.581393960574637*^9, 3.581393986820835*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "136666", "]"}], ";", 
  RowBox[{"tp3", " ", "=", " ", 
   RowBox[{"MakePoints", "[", 
    RowBox[{"50", ",", "3"}], "]"}]}], ";", 
  RowBox[{"SeedRandom", "[", "136666", "]"}], ";", 
  RowBox[{"tp4", " ", "=", " ", 
   RowBox[{"MakePoints", "[", 
    RowBox[{"50", ",", "4"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581389234568434*^9, 3.581389287865527*^9}, {
  3.581394028667939*^9, 3.581394033006586*^9}, {3.581394910242098*^9, 
  3.581394913564912*^9}, {3.581395105396986*^9, 3.581395109139408*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Chan Shattering", "Subsubsection",
 CellChangeTimes->{{3.58123740377127*^9, 3.581237405833888*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNCSMapVertices", "[", 
   RowBox[{"shathulls_", ",", "hull_"}], "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "]"}], " ", ">", " ", "0"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"CHNDOutputFromSimplexes", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"shathulls", "[", 
        RowBox[{"[", "#", "]"}], "]"}], "&"}], " ", "/@", " ", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"SimplexCanon", "[", 
       RowBox[{"shathulls", "[", 
        RowBox[{"hull", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], ",", 
      RowBox[{"{", "}"}]}], "}"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 CellChangeTimes->{
  3.578889328256905*^9, {3.5788893978774357`*^9, 3.5788894732391863`*^9}, {
   3.578889789004684*^9, 3.578889795578456*^9}, {3.581395174805313*^9, 
   3.5813951783705683`*^9}, {3.581395378238381*^9, 3.581395432612812*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNChanShattering", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"CHParts_:", "CHNGiftWrapping"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"GenChanShattering", "[", 
   RowBox[{
   "vts", ",", "First", ",", "CHParts", ",", "CHNPartialGiftWrapping", ",", 
    "CHNCSMapVertices"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577155899060878*^9, 3.5771559243873777`*^9}, {
  3.5771560441185093`*^9, 3.577156083254114*^9}, {3.578889219683118*^9, 
  3.578889229050774*^9}, {3.578889315422083*^9, 3.578889315971929*^9}, {
  3.581395183096648*^9, 3.581395183752795*^9}, {3.581398225187531*^9, 
  3.581398237169237*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["QuickHull", "Subsubsection",
 CellChangeTimes->{{3.5790702203496017`*^9, 3.579070222667605*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Find", " ", "the", " ", "faces", " ", "visible", " ", "from", " ", "the", 
   " ", "a", " ", "new", " ", "vertex", " ", "and", " ", "replace", " ", 
   "them", " ", "with", " ", "ones", " ", "containing", " ", "the", " ", 
   "new", " ", 
   RowBox[{"vertex", ".", " ", "Also"}], " ", "returns", " ", "the", " ", 
   "old", " ", "faces", " ", "and", " ", "the", " ", "new", " ", 
   RowBox[{"faces", "."}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5790959395319757`*^9, 3.579095955538993*^9}, {
  3.579096005882608*^9, 3.579096035883107*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNVisFacesFromVert", "[", 
   RowBox[{
   "vts_", ",", "origfaces_", ",", "startface_", ",", "ix_", ",", 
    "fctosbfcs_", ",", "sbfctofcs_", ",", "orient_"}], "]"}], " ", ":=", " ", 
  
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "oldfaces", ",", "facestat", ",", "unchecked", ",", "nfaces", ",", 
      "sbfc", ",", "k", ",", "face", ",", "facex", ",", "newfaces", ",", 
      "updfaces"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"oldfaces", " ", "=", " ", 
      RowBox[{"{", "startface", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
        "Find", " ", "which", " ", "existing", " ", "faces", " ", "are", " ", 
         "visible", " ", "from", " ", "the", " ", 
         RowBox[{"vertex", ".", " ", "0"}]}], " ", "=", " ", "unchecked"}], 
       ",", " ", 
       RowBox[{"1", " ", "=", " ", "visible"}], ",", " ", 
       RowBox[{"2", " ", "=", " ", "invisible"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"facestat", "[", "startface", "]"}], " ", "=", " ", "0"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"unchecked", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"unchecked", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"unchecked", " ", "=", " ", "False"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"nfaces", " ", "=", " ", 
         RowBox[{"Length", "[", "oldfaces", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"face", " ", "=", " ", 
            RowBox[{"oldfaces", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"facestat", "[", "face", "]"}], " ", "\[Equal]", " ", 
              "0"}], ",", " ", 
             RowBox[{
              RowBox[{"unchecked", " ", "=", " ", "True"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"(*", " ", 
               RowBox[{"Look", " ", "at", " ", "a", " ", 
                RowBox[{"face", "'"}], "s", " ", "neighboring", " ", 
                "faces"}], " ", "*)"}], "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"facex", " ", "=", " ", 
                  RowBox[{"sbfctofcs", "[", 
                   RowBox[{"SimplexFlip", "[", "sbfc", "]"}], "]"}]}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"facestat", "[", "facex", "]"}], "]"}], " ", "===",
                     " ", "facestat"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"oldfaces", ",", "facex"}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"facestat", "[", "facex", "]"}], " ", "=", " ", 
                    "0"}]}]}], "]"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{"sbfc", ",", 
                  RowBox[{"fctosbfcs", "[", "face", "]"}]}], "}"}]}], "]"}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"facestat", "[", "face", "]"}], " ", "=", " ", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"orient", "[", 
                  RowBox[{"vts", "[", 
                   RowBox[{"[", 
                    RowBox[{"Append", "[", 
                    RowBox[{
                    RowBox[{"SimplexFlip", "[", "face", "]"}], ",", "ix"}], 
                    "]"}], "]"}], "]"}], "]"}], ",", "1", ",", "2"}], 
                "]"}]}]}]}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "nfaces"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "horizon", " ", "subfaces"}], ",", 
       " ", 
       RowBox[{
       "those", " ", "between", " ", "visible", " ", "and", " ", "invisible", 
        " ", "faces"}], ",", " ", 
       RowBox[{"and", " ", "the", " ", "new", " ", "faces"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"newfaces", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"facestat", "[", "face", "]"}], " ", "\[Equal]", " ", "1"}],
          ",", "\[IndentingNewLine]", 
         RowBox[{"Do", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"facex", " ", "=", " ", 
             RowBox[{"sbfctofcs", "[", 
              RowBox[{"SimplexFlip", "[", "sbfc", "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"facestat", "[", "facex", "]"}], " ", "\[Equal]", " ", 
               "2"}], ",", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"newfaces", ",", 
                RowBox[{"SimplexCanon", "[", 
                 RowBox[{"Append", "[", 
                  RowBox[{"sbfc", ",", "ix"}], "]"}], "]"}]}], "]"}]}], 
             "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"sbfc", ",", 
             RowBox[{"fctosbfcs", "[", "face", "]"}]}], "}"}]}], 
          "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"face", ",", "oldfaces"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"SimplexSubRefs", "[", 
        RowBox[{"face", ",", " ", "fctosbfcs", ",", " ", "sbfctofcs"}], "]"}],
        " ", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"face", ",", "newfaces"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Remove", " ", "the", " ", "old", " ", "faces"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"oldfaces", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"oldfaces", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"facestat", "[", "#", "]"}], "\[Equal]", "1"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"updfaces", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Complement", "[", 
         RowBox[{"origfaces", ",", "oldfaces"}], "]"}], ",", "newfaces"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"updfaces", ",", "oldfaces", ",", "newfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5790960526237907`*^9, 3.579096134209607*^9}, {
   3.579096171776676*^9, 3.579096362630061*^9}, {3.579096488624277*^9, 
   3.5790965959233313`*^9}, {3.579096651227673*^9, 3.5790966627641*^9}, {
   3.5790966984169693`*^9, 3.579096791635332*^9}, {3.579096867573968*^9, 
   3.579096931993291*^9}, {3.5790969974319267`*^9, 3.579097027505761*^9}, 
   3.5790970739932537`*^9, {3.57909717080589*^9, 3.579097426518169*^9}, {
   3.57909746322502*^9, 3.5790975453751707`*^9}, {3.5793358286661577`*^9, 
   3.579335862802433*^9}, {3.579335993516274*^9, 3.579335998713616*^9}, {
   3.579336070071204*^9, 3.579336075741413*^9}, {3.579336119435607*^9, 
   3.579336122361332*^9}, {3.579336174256178*^9, 3.57933617886802*^9}, {
   3.579336228965053*^9, 3.579336237504695*^9}, {3.579336296098968*^9, 
   3.5793363413053713`*^9}, {3.5793366732592363`*^9, 3.579336722321844*^9}, {
   3.579345296664233*^9, 3.579345301451729*^9}, {3.5793455128219748`*^9, 
   3.579345518675687*^9}, 3.579600291697723*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", " ", "indices", " ", "of", " ", "positive"}], "-", 
   RowBox[{"side", " ", "vertices", " ", "and", " ", "the", " ", "maximum"}], 
   "-", 
   RowBox[{"distance", " ", "one"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576797278797093*^9, 3.5767972875907707`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNQHExternalVertices", "[", 
   RowBox[{"vts_", ",", "vtixs_", ",", "face_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"fdval", ",", "ixvts", ",", "ixpos", ",", "ixmax"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"fdval", " ", "=", " ", 
      RowBox[{"SimplexVolData", "[", 
       RowBox[{"vts", "[", 
        RowBox[{"[", 
         RowBox[{"SimplexFlip", "[", "face", "]"}], "]"}], "]"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"#", ",", 
          RowBox[{"SimplexVolEval", "[", 
           RowBox[{"fdval", ",", 
            RowBox[{"vts", "[", 
             RowBox[{"[", "#", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
        " ", "vtixs"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixpos", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"Select", "[", 
        RowBox[{"ixvts", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ">", "0"}], "&"}]}], "]"}]}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixmax", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "ixvts", "]"}], " ", ">", " ", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "@", " ", 
         RowBox[{"MaxBySortByFunc", "[", 
          RowBox[{"ixvts", ",", 
           RowBox[{
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", "Null"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"ixpos", ",", "ixmax"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQA2IQrXrz7uovS145Oss17QDRCwwv7AHR09a57AXRIXye
B0F0wpmCUyD6TcmLcyA68Z/EeRDNFnMp7BuQvuUiHQGija5M5DNcBtQ3/y2Y
ttrpPtUISP8tewymJcrl5oFoto/f14LoN+nO+0G0nvbKQyD6guSRUyD69rI/
YPrJpsBLIPqEwx0wbfN52WmZ5a8chZbGXADREknBymZA+t7TOXogehtDhAmI
rvmyGUwHZAuKPQbSf6p/gOlNu94kq6545RhetT0FRP/T9c8E0aEOWTkg+lp4
VimIdgnuBNOxztFx7kDaY2lbJojO1UlwmrYKGD7hcV4gOurR9sN8q185FrXN
Pwail6wpv+W0/pWjko7ubRANACw0vfI=
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNQuickHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ndims", ",", "hullvts", ",", "hullfcs", ",", "hull", ",", 
      "ixvts", ",", "ix", ",", "ixinit", ",", "vtstrim", ",", "k", ",", 
      "ixvtarea", ",", "face", ",", "fctosbfcs", ",", "sbfctofcs", ",", 
      "extsfs", ",", "kx", ",", "evs", ",", "ixmax", ",", "oldfcs", ",", 
      "newfcs"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vts", "]"}], " ", "\[Equal]", " ", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nvts", ",", "ndims"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "\[LessEqual]", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "ndims"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hullvts", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hullfcs", " ", "=", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nvts", " ", "\[Equal]", " ", "ndims"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"hullvts", ",", 
             RowBox[{"SimplexFlip", "[", "hullvts", "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", "}"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"hullvts", ",", "hullfcs"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixinit", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"First", " ", "two", " ", "points"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"vtstrim", " ", "=", " ", 
      RowBox[{"First", " ", "/@", " ", "vts"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"ix", " ", "=", " ", 
      RowBox[{"IndexMin", "[", "vtstrim", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"ixinit", ",", "ix"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ix", " ", "=", " ", 
      RowBox[{"IndexMax", "[", "vtstrim", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"ixinit", ",", "ix"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"Complement", "[", 
       RowBox[{"ixvts", ",", "ixinit"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"(*", " ", 
      RowBox[{"Remaining", " ", "points"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"vtstrim", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"Take", "[", 
            RowBox[{"#", ",", "k"}], "]"}], "&"}], " ", "/@", " ", "vts"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"ixvtarea", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"#", ",", 
             RowBox[{"Abs", "[", 
              RowBox[{"SimplexVolume", "[", 
               RowBox[{"vtstrim", "[", 
                RowBox[{"[", 
                 RowBox[{"Append", "[", 
                  RowBox[{"ixinit", ",", "#"}], "]"}], "]"}], "]"}], "]"}], 
              "]"}]}], "}"}], "&"}], " ", "/@", " ", "ixvts"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ix", " ", "=", " ", 
         RowBox[{
          RowBox[{"MaxBySortByFunc", "[", 
           RowBox[{"ixvtarea", ",", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"ixinit", ",", "ix"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"ixvts", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"ixvts", ",", 
           RowBox[{"{", "ix", "}"}]}], "]"}]}]}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", "2", ",", "ndims"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixinit", " ", "=", " ", 
      RowBox[{"Sort", "[", "ixinit", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"SimplexOrient", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "ixinit", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"ixinit", " ", "=", " ", 
        RowBox[{"SimplexFlip", "[", "ixinit", "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullfcs", " ", "=", " ", 
      RowBox[{"SimplexSubs", "[", "ixinit", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "subfaces", " ", "for", " ", "each", " ", 
       "face"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"SimplexSubRefs", "[", 
         RowBox[{"face", ",", "fctosbfcs", ",", "sbfctofcs"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"extsfs", "[", "face", "]"}], " ", "=", " ", 
         RowBox[{"CHNQHExternalVertices", "[", 
          RowBox[{"vts", ",", "ixvts", ",", "face"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"face", ",", "hullfcs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Search", " ", "for", " ", "a", " ", "face", " ", "with", " ", 
         "vertices", " ", "outside", " ", "of", " ", "it"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"face", " ", "=", " ", 
            RowBox[{"hullfcs", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"evs", ",", "ixmax"}], "}"}], " ", "=", " ", 
            RowBox[{"extsfs", "[", "face", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "evs", "]"}], " ", ">", " ", "0"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
               RowBox[{"Found", " ", "a", " ", "non"}], "-", 
               RowBox[{"empty", " ", "outside", " ", "volume"}]}], " ", 
              "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"kx", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"Length", "[", "hullfcs", "]"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"kx", " ", "===", " ", "Null"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{"Get", " ", "updated", " ", "hull", " ", "faces"}], ",", 
          " ", 
          RowBox[{"old", " ", "faces", " ", 
           RowBox[{"(", "removed", ")"}]}], ",", " ", 
          RowBox[{"and", " ", "new", " ", "faces", " ", 
           RowBox[{"(", "added", ")"}]}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"hullfcs", ",", " ", "oldfcs", ",", "newfcs"}], "}"}], " ", 
         "=", " ", 
         RowBox[{"CHNVisFacesFromVert", "[", 
          RowBox[{
          "vts", ",", " ", "hullfcs", ",", " ", "face", ",", " ", "ixmax", 
           ",", " ", "fctosbfcs", ",", " ", "sbfctofcs", ",", " ", 
           "SimplexOrient"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Find", " ", "the", " ", "outward", " ", "points", " ", "for", " ", 
          "the", " ", "new", " ", 
          RowBox[{"faces", ".", " ", "Start"}], " ", "with", " ", "all", " ", 
          "those", " ", "for", " ", "the", " ", "old", " ", 
          RowBox[{"faces", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"evs", " ", "=", " ", 
         RowBox[{"Union", " ", "@@", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"extsfs", "[", "#", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
            "oldfcs"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"evs", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"evs", ",", 
           RowBox[{"{", "ixmax", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"extsfs", "[", "face", "]"}], " ", "=", " ", 
           RowBox[{"CHNQHExternalVertices", "[", 
            RowBox[{"vts", ",", "evs", ",", "face"}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"face", ",", "newfcs"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"CHNDOutputFromSimplexes", "[", "hullfcs", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579099836194688*^9, 3.579100018323752*^9}, {
   3.579100078719198*^9, 3.579100079224411*^9}, {3.579100163711027*^9, 
   3.5791003170802402`*^9}, {3.579100365850952*^9, 3.5791005447143373`*^9}, {
   3.579100578556157*^9, 3.57910062022289*^9}, {3.579100652498815*^9, 
   3.579100780540134*^9}, {3.579100850440393*^9, 3.5791008944205713`*^9}, {
   3.579100939810104*^9, 3.579101001557868*^9}, {3.579101052401557*^9, 
   3.579101107922846*^9}, {3.5791011421646442`*^9, 3.579101154752204*^9}, {
   3.579101188997686*^9, 3.5791011901386757`*^9}, {3.579101244809539*^9, 
   3.579101387770412*^9}, {3.579101430428266*^9, 3.579101472597349*^9}, {
   3.579101609530457*^9, 3.579101654459176*^9}, {3.579101749991275*^9, 
   3.579101806268581*^9}, {3.579102359099642*^9, 3.579102387790381*^9}, {
   3.579102425543029*^9, 3.5791025356163588`*^9}, {3.579102566079179*^9, 
   3.5791025728549*^9}, {3.579102628051218*^9, 3.579102688224749*^9}, {
   3.579142535139606*^9, 3.579142538003293*^9}, {3.579144298681755*^9, 
   3.579144312376338*^9}, {3.579144411784391*^9, 3.579144416873748*^9}, {
   3.57914446544023*^9, 3.5791445840283537`*^9}, {3.5791446540813723`*^9, 
   3.579144733814413*^9}, {3.579144795816972*^9, 3.579144799465344*^9}, {
   3.579144853314831*^9, 3.579144870286797*^9}, {3.5791459271085987`*^9, 
   3.5791459393704348`*^9}, {3.5793370767823467`*^9, 3.579337077247365*^9}, {
   3.579337378272992*^9, 3.579337382150055*^9}, {3.579338827039712*^9, 
   3.5793388435276423`*^9}, {3.5793388846605787`*^9, 3.5793388863174*^9}, {
   3.5793391165179043`*^9, 3.579339155663494*^9}, {3.5793393178958597`*^9, 
   3.579339326737082*^9}, {3.579339367992535*^9, 3.579339381617453*^9}, {
   3.579339422908177*^9, 3.579339432821226*^9}, {3.579340340455347*^9, 
   3.5793403907606287`*^9}, {3.5793404549861097`*^9, 3.579340460944669*^9}, {
   3.579340495994516*^9, 3.57934049815162*^9}, {3.579340546130481*^9, 
   3.5793405891613007`*^9}, {3.57934063745851*^9, 3.579340641198112*^9}, {
   3.5793422472975683`*^9, 3.579342283667192*^9}, {3.5793424556563673`*^9, 
   3.5793424561376743`*^9}, {3.579344462385189*^9, 3.579344495543729*^9}, 
   3.579600297034449*^9, 3.581395361299643*^9, {3.5815467804730797`*^9, 
   3.5815468216853533`*^9}, {3.5815468886191874`*^9, 3.581546890820716*^9}, {
   3.5815469406186132`*^9, 3.581546954169673*^9}, {3.581547141128627*^9, 
   3.5815471453768263`*^9}, {3.581547268365838*^9, 3.5815473042390842`*^9}, {
   3.581547346804387*^9, 3.58154734727705*^9}, {3.581547405589374*^9, 
   3.581547415792809*^9}, {3.5815483489214373`*^9, 3.581548357021593*^9}, {
   3.5815484332411213`*^9, 3.581548445350876*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Incremental", "Subsubsection",
 CellChangeTimes->{{3.579070230190919*^9, 3.57907023174893*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNGenIncremental", "[", 
   RowBox[{
   "vts_", ",", "extradims_", ",", "orient_", ",", "datafunc_", ",", 
    "dataorient_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ndims", ",", "hullvts", ",", "hullfcs", ",", "hull", ",", 
      "facedata", ",", "face", ",", "fctosbfcs", ",", "sbfctofcs", ",", "ix", 
      ",", "kx", ",", "k", ",", "oldfcs", ",", "newfcs"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "vts", "]"}], " ", "\[Equal]", " ", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nvts", ",", "ndims"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ndims", " ", "-=", " ", "extradims"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "\[LessEqual]", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "<", " ", "ndims"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hullvts", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hullfcs", " ", "=", " ", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"hullvts", ",", "hullfcs"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hull", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"hullvts", " ", "=", " ", 
      RowBox[{"Range", "[", "ndims", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hullfcs", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"hullvts", ",", 
        RowBox[{"SimplexFlip", "[", "hullvts", "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"facedata", "[", "face", "]"}], " ", "=", " ", 
         RowBox[{"datafunc", "[", 
          RowBox[{"vts", "[", 
           RowBox[{"[", 
            RowBox[{"SimplexFlip", "[", "face", "]"}], "]"}], "]"}], "]"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"SimplexSubRefs", "[", 
         RowBox[{"face", ",", " ", "fctosbfcs", ",", " ", "sbfctofcs"}], 
         "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"face", ",", "hullfcs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"face", " ", "=", " ", 
            RowBox[{"hullfcs", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"dataorient", "[", 
              RowBox[{
               RowBox[{"facedata", "[", "face", "]"}], ",", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "ix", "]"}], "]"}]}], "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"kx", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"Length", "[", "hullfcs", "]"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"kx", " ", "===", " ", "Null"}], ",", 
          RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"hullfcs", ",", " ", "oldfcs", ",", "newfcs"}], "}"}], " ", 
         "=", " ", 
         RowBox[{"CHNVisFacesFromVert", "[", 
          RowBox[{
          "vts", ",", " ", "hullfcs", ",", " ", "face", ",", " ", "ix", ",", 
           " ", "fctosbfcs", ",", " ", "sbfctofcs", ",", " ", "orient"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "hullfcs", "]"}], "\[Equal]", "0"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"facedata", "[", "face", "]"}], " ", "=", " ", 
           RowBox[{"datafunc", "[", 
            RowBox[{"vts", "[", 
             RowBox[{"[", 
              RowBox[{"SimplexFlip", "[", "face", "]"}], "]"}], "]"}], 
            "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"face", ",", "newfcs"}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", 
         RowBox[{"ndims", "+", "1"}], ",", "nvts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"CHNDOutputFromSimplexes", "[", "hullfcs", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579141223272593*^9, 3.5791412389592333`*^9}, {
   3.579141470581057*^9, 3.579141497717958*^9}, {3.579141844879208*^9, 
   3.579142156920068*^9}, {3.579142203770859*^9, 3.579142363547842*^9}, {
   3.579142433102378*^9, 3.5791425455240097`*^9}, {3.579142990359614*^9, 
   3.579143014469714*^9}, {3.57914531897815*^9, 3.579145332483588*^9}, {
   3.579145955424349*^9, 3.579145955842692*^9}, {3.579146137559847*^9, 
   3.579146160048162*^9}, {3.579146191805361*^9, 3.579146193569096*^9}, {
   3.5791463331212387`*^9, 3.5791463511827993`*^9}, {3.5791463914537373`*^9, 
   3.579146402251934*^9}, {3.5791464647552958`*^9, 3.57914648150937*^9}, {
   3.579146519041133*^9, 3.579146530174006*^9}, {3.579146566730715*^9, 
   3.579146599298759*^9}, {3.579146869411261*^9, 3.5791468812365513`*^9}, {
   3.5791469426236467`*^9, 3.579146942991501*^9}, {3.579335532583075*^9, 
   3.579335544485281*^9}, {3.579335580933865*^9, 3.579335588903659*^9}, {
   3.579336748680052*^9, 3.579336753403883*^9}, {3.5793370448862877`*^9, 
   3.579337056949683*^9}, {3.579345383659553*^9, 3.579345433827134*^9}, {
   3.579347702279351*^9, 3.5793477070986643`*^9}, {3.579503569477725*^9, 
   3.5795035851368637`*^9}, {3.5795037081347218`*^9, 3.579503712384383*^9}, 
   3.579600305006011*^9, {3.581395323784947*^9, 3.581395329598291*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNIncremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CHNGenIncremental", "[", 
   RowBox[{
   "vts", ",", "0", ",", "SimplexOrient", ",", "SimplexVolData", ",", 
    "SimplexOrientEval"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577795854113966*^9, 3.57779588796032*^9}, {
  3.57788809490821*^9, 3.5778881078365507`*^9}, {3.578931105906784*^9, 
  3.578931114410562*^9}, {3.5791426758028708`*^9, 3.579142727929441*^9}, {
  3.579143020379878*^9, 3.5791430206498833`*^9}, {3.579146646124132*^9, 
  3.579146647028595*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNSphIncremental", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"CHNGenIncremental", "[", 
   RowBox[{
   "vts", ",", "1", ",", "SphSimplexOrient", ",", "SphSimplexVolData", ",", 
    "SphSimplexOrientEval"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.577795854113966*^9, 3.57779588796032*^9}, {
   3.57788809490821*^9, 3.5778881078365507`*^9}, {3.578931105906784*^9, 
   3.578931114410562*^9}, {3.579069093893446*^9, 3.5790690961145973`*^9}, {
   3.57906920860681*^9, 3.579069268443915*^9}, {3.579142682419087*^9, 
   3.579142694468863*^9}, {3.579142755098205*^9, 3.579142772377696*^9}, {
   3.579143023154986*^9, 3.5791430234900503`*^9}, 3.579146654019068*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Verification and Comparison", "Subsubsection",
 CellChangeTimes->{3.579095435744019*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNAlgorithms", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "CHNGiftWrapping", ",", "CHNChanShattering", ",", "CHNQuickHull", ",", 
     "CHNIncremental"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581397532082965*^9, 3.581397567018704*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CHNSphAlgorihms", " ", "=", " ", 
   RowBox[{"{", "CHNSphIncremental", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581578160318675*^9, 3.581578199223027*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Delaunay Triangulation / Tessellation", "Section",
 CellChangeTimes->{{3.57959449558987*^9, 3.579594502166646*^9}, 
   3.5799075062904387`*^9, {3.582535622132268*^9, 3.58253562333639*^9}}],

Cell[CellGroupData[{

Cell["2D - Divide and Conquer", "Subsection",
 CellChangeTimes->{{3.582161300433289*^9, 3.5821613104425297`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Recursive", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.582161403770174*^9, 3.5821614091611977`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2DivideConquer", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", " ", "hull", ",", "tris", ",", "hldt", ",", "ixvts", ",", 
      "nspl", ",", "ixvts1", ",", "ixvts2", ",", "ixs1", ",", "ixs2", ",", 
      "vts1", ",", "vts2", ",", "hldt1", ",", "hldt2", ",", "hull1", ",", " ",
       "hull2", ",", " ", "tris1", ",", "tris2", ",", "tri", ",", "trieds", 
      ",", "edtri", ",", "ixnr1", ",", "ixnr2", ",", "ix11", ",", "ix12", ",",
       "ix21", ",", "ix22", ",", "hli1", ",", "hli2", ",", "hiix1", ",", 
      "hiix2", ",", "rmtris1", ",", "rmtris2", ",", "adtris", ",", "adv1", 
      ",", "adv2", ",", "crix1", ",", "crix2", ",", "nxix1", ",", "nxix2", 
      ",", "rmtri1", ",", "rmtri2", ",", "hlad1", ",", "hlad2", ",", "ccdt1", 
      ",", "ccdt2", ",", "seg", ",", "ix"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{"For", " ", "at", " ", "most", " ", "3", " ", "points"}], ",", 
      " ", 
      RowBox[{"find", " ", "the", " ", "hull", " ", "by", " ", "hand"}]}], 
     " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "3"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"Range", "[", "nvts", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"tris", " ", "=", " ", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"nvts", " ", "\[Equal]", " ", "3"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", 
              RowBox[{"TriOrient2D", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "hull", "]"}], "]"}], "]"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"hull", " ", "=", " ", 
              RowBox[{"{", 
               RowBox[{"1", ",", "3", ",", "2"}], "}"}]}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"tris", " ", "=", " ", 
            RowBox[{"{", "hull", "}"}]}]}]}], "\[IndentingNewLine]", "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"hldt", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", "hull", "}"}], ",", " ", "tris", ",", " ", 
           RowBox[{"{", "}"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "hldt", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "hulls", " ", "of", " ", "the", " ", "two", 
       " ", "parts"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"ixvts", " ", "=", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"AddIndices", "[", "vts", "]"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nspl", " ", "=", " ", 
      RowBox[{"Floor", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nvts", "+", "1"}], ")"}], "/", "2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ixvts1", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixvts2", " ", "=", " ", 
      RowBox[{"Drop", "[", 
       RowBox[{"ixvts", ",", "nspl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixs1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ixs2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts1"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vts2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "&"}], " ", "/@", " ", "ixvts2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "Rotate", " ", "the", " ", "points", " ", "90", "d", " ", "for", " ", 
        "calculating", " ", "the", " ", "child", " ", "hulls"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
       "so", " ", "as", " ", "to", " ", "alternate", " ", "the", " ", 
        "directions", " ", "of", " ", "splitting"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"hldt1", " ", "=", " ", 
      RowBox[{"DT2DivideConquer", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"-", "1"}]}], "}"}], "*", 
           RowBox[{"Reverse", "[", "#", "]"}]}], ")"}], "&"}], " ", "/@", " ",
         "vts1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hldt2", " ", "=", " ", 
      RowBox[{"DT2DivideConquer", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"-", "1"}]}], "}"}], "*", 
           RowBox[{"Reverse", "[", "#", "]"}]}], ")"}], "&"}], " ", "/@", " ",
         "vts2"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"hldt1", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"hldt2", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tris1", " ", "=", " ", 
      RowBox[{"hldt1", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tris2", " ", "=", " ", 
      RowBox[{"hldt2", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "closest", " ", "points"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"hull1", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", 
          RowBox[{"Length", "[", "ixs1", "]"}]}], ",", "\[IndentingNewLine]", 
         
         RowBox[{
          RowBox[{"ixnr1", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "hull1", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"hull2", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "\[Equal]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ixnr2", " ", "=", " ", "k"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "hull2", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Get", " ", "the", " ", "hulls", " ", "and", " ", "triangles", " ", 
       "into", " ", "the", " ", "input", " ", 
       RowBox[{"points", "'"}], " ", "indexing"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"ixs1", "[", 
       RowBox[{"[", "hull1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"ixs2", "[", 
       RowBox[{"[", "hull2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tris1", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"RotateToMin", "[", 
         RowBox[{"ixs1", "[", 
          RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "tris1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tris2", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"RotateToMin", "[", 
         RowBox[{"ixs2", "[", 
          RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "tris2"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"TriEdgeRefs", "[", 
        RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "tris1"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"TriEdgeRefs", "[", 
        RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"tri", ",", "tris2"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "tangent", " ", "lines", " ", "between", " ", "the", " ", 
       "hulls"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ix11", ",", "ix12"}], "}"}], " ", "=", " ", 
      RowBox[{"CH2TwoHullTangent", "[", 
       RowBox[{
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull1", "]"}], "]"}], ",", 
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull2", "]"}], "]"}], ",", "ixnr1", ",", "ixnr2"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ix22", ",", "ix21"}], "}"}], " ", "=", " ", 
      RowBox[{"CH2TwoHullTangent", "[", 
       RowBox[{
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull2", "]"}], "]"}], ",", 
        RowBox[{"vts", "[", 
         RowBox[{"[", "hull1", "]"}], "]"}], ",", "ixnr2", ",", "ixnr1"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "the", " ", "inside", " ", "hulls", " ", "for", " ", 
       "making", " ", "the", " ", "inside", " ", "triangles"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "If", " ", "the", " ", "top", " ", "and", " ", "bottom", " ", 
        "tangents", " ", "meet", " ", "at", " ", "the", " ", "same", " ", 
        "point"}], ",", "\[IndentingNewLine]", 
       RowBox[{
       "then", " ", "the", " ", "hull", " ", "segment", " ", "starts", " ", 
        "and", " ", "ends", " ", "at", " ", "that", " ", "point"}]}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hli1", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix11", " ", ">", " ", "ix21"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull1", ",", 
          RowBox[{"{", 
           RowBox[{"ix21", ",", "ix11"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"ix21", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix11"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hli2", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix22", " ", ">", " ", "ix12"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull2", ",", 
          RowBox[{"{", 
           RowBox[{"ix12", ",", "ix22"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"ix12", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix22"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Stitch", " ", "the", " ", "hulls", " ", "together"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "If", " ", "the", " ", "top", " ", "and", " ", "bottom", " ", 
        "tangents", " ", "meet", " ", "at", " ", "the", " ", "same", " ", 
        "point"}], ",", "\[IndentingNewLine]", 
       RowBox[{
       "then", " ", "the", " ", "hull", " ", "segment", " ", "has", " ", 
        "zero", " ", "size"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hull1", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix21", " ", "\[GreaterEqual]", " ", "ix11"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull1", ",", 
          RowBox[{"{", 
           RowBox[{"ix11", ",", "ix21"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"ix11", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull1", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix21"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hull2", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ix12", " ", "\[GreaterEqual]", " ", "ix22"}], ",", 
        RowBox[{"Take", "[", 
         RowBox[{"hull2", ",", 
          RowBox[{"{", 
           RowBox[{"ix22", ",", "ix12"}], "}"}]}], "]"}], ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"ix22", ",", 
              RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"Take", "[", 
           RowBox[{"hull2", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "ix12"}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"RotateToMin", "[", 
       RowBox[{"Join", "[", 
        RowBox[{"hull1", ",", "hull2"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Do", " ", "the", " ", "merged", " ", "triangles", " ", "from", " ", 
       "bottom", " ", "to", " ", 
       RowBox[{"top", ".", "\[IndentingNewLine]", "Make"}], " ", "the", " ", 
       "internal", " ", "hulls", " ", "both", " ", "top", " ", "to", " ", 
       "bottom"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"hli1", " ", "=", " ", 
      RowBox[{"Reverse", "[", "hli1", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hiix1", " ", "=", " ", 
      RowBox[{"Length", "[", "hli1", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hiix2", " ", "=", " ", 
      RowBox[{"Length", "[", "hli2", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rmtris1", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rmtris2", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"adtris", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"adv1", " ", "=", " ", 
         RowBox[{"hiix1", " ", ">", " ", "1"}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"adv2", " ", "=", " ", 
         RowBox[{"hiix2", " ", ">", " ", "1"}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"(*", " ", 
         RowBox[{"Unable", " ", "to", " ", "advance", " ", 
          RowBox[{"further", "?"}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"!", "adv1"}], ")"}], " ", "&&", " ", 
           RowBox[{"(", 
            RowBox[{"!", "adv2"}], ")"}]}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"crix1", " ", "=", " ", 
         RowBox[{"hli1", "[", 
          RowBox[{"[", "hiix1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"crix2", " ", "=", " ", 
         RowBox[{"hli2", "[", 
          RowBox[{"[", "hiix2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Left", "-", 
          RowBox[{"side", " ", 
           RowBox[{"advance", "?"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"adv1", ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"hlad1", " ", "=", " ", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"rmtri1", " ", "=", " ", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"nxix1", " ", "=", " ", 
            RowBox[{"hli1", "[", 
             RowBox[{"[", 
              RowBox[{"hiix1", "-", "1"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"TriOrient2D", "[", 
              RowBox[{"vts", "[", 
               RowBox[{"[", 
                RowBox[{"{", 
                 RowBox[{"crix1", ",", "crix2", ",", "nxix1"}], "}"}], "]"}], 
               "]"}], "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "Look", " ", "inward", " ", "from", " ", "the", " ", "edge", 
               " ", "to", " ", "find", " ", "a", " ", "suitable", " ", "next",
                " ", "point"}], " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{"While", "[", 
              RowBox[{"True", ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"ccdt1", " ", "=", " ", 
                 RowBox[{"CircumData", "[", 
                  RowBox[{"vts", "[", 
                   RowBox[{"[", 
                    RowBox[{"{", 
                    RowBox[{"crix1", ",", "crix2", ",", "nxix1"}], "}"}], 
                    "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
                RowBox[{"seg", " ", "=", " ", 
                 RowBox[{"{", 
                  RowBox[{"crix1", ",", "nxix1"}], "}"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"tri", " ", "=", " ", 
                 RowBox[{"edtri", "[", "seg", "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"Head", "[", "tri", "]"}], " ", "===", " ", 
                   "edtri"}], ",", 
                  RowBox[{"Break", "[", "]"}]}], "]"}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"ix", " ", "=", " ", 
                 RowBox[{"First", "[", 
                  RowBox[{"Complement", "[", 
                   RowBox[{"tri", ",", "seg"}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"!", 
                   RowBox[{"TriOrient2D", "[", 
                    RowBox[{"vts", "[", 
                    RowBox[{"[", 
                    RowBox[{"{", 
                    RowBox[{"crix1", ",", "crix2", ",", "ix"}], "}"}], "]"}], 
                    "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}],
                 ";", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"CircumInside", "[", 
                   RowBox[{"ccdt1", ",", 
                    RowBox[{"vts", "[", 
                    RowBox[{"[", "ix", "]"}], "]"}]}], "]"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"AppendTo", "[", 
                    RowBox[{"hlad1", ",", "ix"}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"AppendTo", "[", 
                    RowBox[{"rmtri1", ",", "tri"}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"nxix1", " ", "=", " ", "ix"}]}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", 
                 "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"adv1", " ", "=", " ", "False"}]}], 
            "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Right", "-", 
          RowBox[{"side", " ", 
           RowBox[{"advance", "?"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"adv2", ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"hlad2", " ", "=", " ", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"rmtri2", " ", "=", " ", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"nxix2", " ", "=", " ", 
            RowBox[{"hli2", "[", 
             RowBox[{"[", 
              RowBox[{"hiix2", "-", "1"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"TriOrient2D", "[", 
              RowBox[{"vts", "[", 
               RowBox[{"[", 
                RowBox[{"{", 
                 RowBox[{"crix1", ",", "crix2", ",", "nxix2"}], "}"}], "]"}], 
               "]"}], "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "Look", " ", "inward", " ", "from", " ", "the", " ", "edge", 
               " ", "to", " ", "find", " ", "a", " ", "suitable", " ", "next",
                " ", "point"}], " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{"While", "[", 
              RowBox[{"True", ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"ccdt2", " ", "=", " ", 
                 RowBox[{"CircumData", "[", 
                  RowBox[{"vts", "[", 
                   RowBox[{"[", 
                    RowBox[{"{", 
                    RowBox[{"crix1", ",", "crix2", ",", "nxix2"}], "}"}], 
                    "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
                RowBox[{"seg", " ", "=", " ", 
                 RowBox[{"{", 
                  RowBox[{"nxix2", ",", "crix2"}], "}"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"tri", " ", "=", " ", 
                 RowBox[{"edtri", "[", "seg", "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"Head", "[", "tri", "]"}], " ", "===", " ", 
                   "edtri"}], ",", 
                  RowBox[{"Break", "[", "]"}]}], "]"}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"ix", " ", "=", " ", 
                 RowBox[{"First", "[", 
                  RowBox[{"Complement", "[", 
                   RowBox[{"tri", ",", "seg"}], "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"!", 
                   RowBox[{"TriOrient2D", "[", 
                    RowBox[{"vts", "[", 
                    RowBox[{"[", 
                    RowBox[{"{", 
                    RowBox[{"crix1", ",", "crix2", ",", "ix"}], "}"}], "]"}], 
                    "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}],
                 ";", "\[IndentingNewLine]", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"CircumInside", "[", 
                   RowBox[{"ccdt2", ",", 
                    RowBox[{"vts", "[", 
                    RowBox[{"[", "ix", "]"}], "]"}]}], "]"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"AppendTo", "[", 
                    RowBox[{"hlad2", ",", "ix"}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"AppendTo", "[", 
                    RowBox[{"rmtri2", ",", "tri"}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"nxix2", " ", "=", " ", "ix"}]}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", 
                 "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"adv2", " ", "=", " ", "False"}]}], 
            "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Both", "-", 
          RowBox[{"sides", " ", 
           RowBox[{"advance", "?", " ", "Find"}], " ", "out", " ", "which", 
           " ", "one"}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"adv1", " ", "&&", " ", "adv2"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Which", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"CircumInside", "[", 
              RowBox[{"ccdt1", ",", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "nxix2", "]"}], "]"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"adv2", " ", "=", " ", "False"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"!", 
             RowBox[{"CircumInside", "[", 
              RowBox[{"ccdt2", ",", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "nxix1", "]"}], "]"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"adv1", " ", "=", " ", "False"}], ",", 
            "\[IndentingNewLine]", "True", ",", "\[IndentingNewLine]", 
            RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{"adv1", ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Advance", " ", "on", " ", "left", " ", "side"}], " ", 
           "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "hlad1", "]"}], " ", ">", " ", "0"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"hli1", " ", "=", " ", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"Take", "[", 
                  RowBox[{"hli1", ",", 
                   RowBox[{"hiix1", "-", "1"}]}], "]"}], ",", "hlad1"}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"hiix1", " ", "=", " ", 
               RowBox[{"Length", "[", "hli1", "]"}]}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"hiix1", " ", "-=", " ", "1"}]}], "\[IndentingNewLine]", 
            "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "rmtri1", "]"}], " ", ">", " ", "0"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{"rmtris1", " ", "=", " ", 
              RowBox[{"Join", "[", 
               RowBox[{"rmtris1", ",", "rmtri1"}], "]"}]}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"RotateToMin", "[", 
             RowBox[{"{", 
              RowBox[{"crix1", ",", "crix2", ",", "nxix1"}], "}"}], "]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"adtris", ",", "tri"}], "]"}]}], ",", 
          "\[IndentingNewLine]", "adv2", ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Advance", " ", "on", " ", "right", " ", "side"}], " ", 
           "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "hlad2", "]"}], " ", ">", " ", "0"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"hli2", " ", "=", " ", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"Take", "[", 
                  RowBox[{"hli2", ",", 
                   RowBox[{"hiix2", "-", "1"}]}], "]"}], ",", "hlad2"}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"hiix2", " ", "=", " ", 
               RowBox[{"Length", "[", "hli2", "]"}]}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"hiix2", " ", "-=", " ", "1"}]}], "\[IndentingNewLine]", 
            "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "rmtri2", "]"}], " ", ">", " ", "0"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{"rmtris2", " ", "=", " ", 
              RowBox[{"Join", "[", 
               RowBox[{"rmtris2", ",", "rmtri2"}], "]"}]}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"tri", " ", "=", " ", 
            RowBox[{"RotateToMin", "[", 
             RowBox[{"{", 
              RowBox[{"crix1", ",", "crix2", ",", "nxix2"}], "}"}], "]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"adtris", ",", "tri"}], "]"}]}], ",", 
          "\[IndentingNewLine]", "True", ",", "\[IndentingNewLine]", 
          RowBox[{"Break", "[", "]"}]}], "\[IndentingNewLine]", "]"}], 
        ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"tris", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Complement", "[", 
         RowBox[{"tris1", ",", "rmtris1"}], "]"}], ",", 
        RowBox[{"Complement", "[", 
         RowBox[{"tris2", ",", "rmtris2"}], "]"}], ",", "adtris"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"tris", " ", "=", " ", 
      RowBox[{"Sort", "[", "tris", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "hull", "}"}], ",", " ", "tris", ",", " ", 
       RowBox[{"{", "}"}]}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.575748121984099*^9, 3.575748133075938*^9}, {
   3.57578655472513*^9, 3.575786582390957*^9}, {3.5757873538331614`*^9, 
   3.575787401202379*^9}, {3.5757874326641903`*^9, 3.575787572516796*^9}, {
   3.575787679197785*^9, 3.575787751425363*^9}, {3.575787781803688*^9, 
   3.57578780940387*^9}, {3.575789031535413*^9, 3.5757890806103077`*^9}, {
   3.575789116911085*^9, 3.575789169414853*^9}, {3.575789291096561*^9, 
   3.575789413102585*^9}, {3.575789451627865*^9, 3.5757894572107363`*^9}, 
   3.575789492296549*^9, {3.575789688365378*^9, 3.5757897889042053`*^9}, {
   3.5757898650243673`*^9, 3.575789935440854*^9}, {3.575790070571764*^9, 
   3.575790093043057*^9}, {3.575790232950604*^9, 3.575790262168594*^9}, {
   3.575790303137864*^9, 3.575790332287765*^9}, {3.5757903735315533`*^9, 
   3.5757905746520348`*^9}, {3.5757906138218317`*^9, 3.575790620810421*^9}, {
   3.5757906855835752`*^9, 3.575790760742723*^9}, {3.575791533901527*^9, 
   3.575791535251452*^9}, {3.575791864239854*^9, 3.575791922414309*^9}, {
   3.575792073008532*^9, 3.5757920862298737`*^9}, {3.575792776685885*^9, 
   3.575792821001733*^9}, {3.575796120286574*^9, 3.575796160283103*^9}, {
   3.575796219716041*^9, 3.575796241511891*^9}, {3.575796304978487*^9, 
   3.5757963300170317`*^9}, {3.575796950870241*^9, 3.575797045448537*^9}, {
   3.575837879280663*^9, 3.575837923251533*^9}, {3.575981221608276*^9, 
   3.575981221773098*^9}, {3.575981561927153*^9, 3.575981562089891*^9}, {
   3.5759816611929407`*^9, 3.575981676221401*^9}, {3.575981809651079*^9, 
   3.575981980924674*^9}, {3.575982032807292*^9, 3.57598206231157*^9}, {
   3.575982247266596*^9, 3.5759822613789463`*^9}, {3.575982338357884*^9, 
   3.5759823624439173`*^9}, {3.5759824552495527`*^9, 3.575982537612626*^9}, {
   3.575982576129842*^9, 3.575982601422359*^9}, {3.575982641406695*^9, 
   3.575982641973921*^9}, {3.576309460401182*^9, 3.576309471037458*^9}, {
   3.576309501766316*^9, 3.576309637330697*^9}, {3.576311098693799*^9, 
   3.576311159244645*^9}, {3.5763111996144447`*^9, 3.576311216615343*^9}, {
   3.576311415654913*^9, 3.576311421370924*^9}, {3.5763114561413403`*^9, 
   3.576311575078356*^9}, {3.5763116191219893`*^9, 3.576311669662118*^9}, {
   3.576338123004759*^9, 3.576338143561308*^9}, 3.5765061307554417`*^9, {
   3.577059116148549*^9, 3.577059154308537*^9}, {3.577270918407454*^9, 
   3.577270935943115*^9}, {3.577270990784835*^9, 3.577270996988584*^9}, {
   3.577272031886281*^9, 3.577272048025474*^9}, {3.578885406969839*^9, 
   3.5788854279589233`*^9}, {3.578885613162485*^9, 3.578885613814839*^9}, {
   3.582078792221448*^9, 3.5820788319588127`*^9}, {3.582078946396646*^9, 
   3.582078963276052*^9}, {3.5820790578151913`*^9, 3.5820791267001743`*^9}, {
   3.5820792408755817`*^9, 3.582079283660409*^9}, {3.582079319993956*^9, 
   3.582079348431094*^9}, {3.582079384236154*^9, 3.582079394744967*^9}, {
   3.58216141659807*^9, 3.582161533519068*^9}, {3.5821616206333523`*^9, 
   3.582161696379876*^9}, {3.5821617291558733`*^9, 3.5821618967293043`*^9}, {
   3.5821621128949127`*^9, 3.582162254346355*^9}, {3.582162289836523*^9, 
   3.582162319231469*^9}, {3.582162936389138*^9, 3.582163021596834*^9}, {
   3.5821630881772547`*^9, 3.582163102473995*^9}, {3.582163144033794*^9, 
   3.582163152265976*^9}, {3.582163189339347*^9, 3.582163211597866*^9}, {
   3.5821632603207293`*^9, 3.5821632703311987`*^9}, {3.58216342285987*^9, 
   3.5821634749853487`*^9}, {3.5821635678819437`*^9, 3.582163571756875*^9}, {
   3.5821636129145803`*^9, 3.582163666464246*^9}, {3.582163727506151*^9, 
   3.58216378658142*^9}, {3.582184960558655*^9, 3.5821850103315163`*^9}, {
   3.5821850500452957`*^9, 3.582185076678797*^9}, {3.5821851250822268`*^9, 
   3.582185206848526*^9}, {3.582185341340424*^9, 3.582185438370557*^9}, {
   3.582185496626938*^9, 3.582185512280993*^9}, {3.582185549540083*^9, 
   3.582185632252297*^9}, {3.582185671411772*^9, 3.58218580768755*^9}, {
   3.582185861548584*^9, 3.5821858814821033`*^9}, {3.582185998748336*^9, 
   3.582186006846937*^9}, {3.582186053907426*^9, 3.582186065192993*^9}, {
   3.582186164081121*^9, 3.5821864004680233`*^9}, {3.582186432554592*^9, 
   3.582186458345702*^9}, {3.582186529994879*^9, 3.582186576359414*^9}, {
   3.582186727684414*^9, 3.582186730839232*^9}, {3.582186767373576*^9, 
   3.582186789511376*^9}, {3.5821869075062513`*^9, 3.582186930259553*^9}, {
   3.5821870034379807`*^9, 3.5821870948436213`*^9}, {3.58218770883315*^9, 
   3.582187742441875*^9}, {3.582187799465025*^9, 3.5821878193093576`*^9}, {
   3.582187850994348*^9, 3.582187857495839*^9}, {3.582187976820147*^9, 
   3.582187991033901*^9}, {3.582188033823423*^9, 3.582188046854244*^9}, {
   3.582188155641461*^9, 3.582188254747267*^9}, {3.582188354836452*^9, 
   3.5821883567898893`*^9}, {3.582188387802153*^9, 3.582188412243191*^9}, {
   3.5821884563598957`*^9, 3.582188563228972*^9}, {3.582188618572918*^9, 
   3.582188737311026*^9}, {3.582188824261922*^9, 3.5821890031140137`*^9}, 
   3.582189035891766*^9, {3.582189484967066*^9, 3.582189493188443*^9}, {
   3.5821895782047377`*^9, 3.582189593582123*^9}, {3.582189695113276*^9, 
   3.58218969807786*^9}, {3.582190900715905*^9, 3.5821910049336643`*^9}, {
   3.582191045611267*^9, 3.582191048968793*^9}, {3.582194356904917*^9, 
   3.582194382852585*^9}, {3.5821971765891027`*^9, 3.582197196565135*^9}, {
   3.5821974444329653`*^9, 3.582197458005731*^9}, {3.582197589666979*^9, 
   3.582197611572957*^9}, 3.5821996953043633`*^9, {3.582335991128694*^9, 
   3.5823360105463743`*^9}, {3.5823360827290173`*^9, 3.582336109009563*^9}, {
   3.582336308220809*^9, 3.582336311102746*^9}, {3.582342599898045*^9, 
   3.5823426038240137`*^9}, {3.5825079184210873`*^9, 3.58250795930221*^9}, {
   3.582508188688608*^9, 3.5825082687113447`*^9}, {3.582508308789509*^9, 
   3.5825083163096457`*^9}, {3.582508364357234*^9, 3.582508527535914*^9}, {
   3.5825085709017773`*^9, 3.582508619407737*^9}, {3.582508876065917*^9, 
   3.582508889862591*^9}, {3.582509299610179*^9, 3.58250933729681*^9}, {
   3.582509367926437*^9, 3.582509402581333*^9}, {3.582509466038434*^9, 
   3.5825094665270357`*^9}, {3.582509497383215*^9, 3.582509497959778*^9}, {
   3.582509780184504*^9, 3.5825098377688637`*^9}, 3.5825098795234337`*^9, {
   3.582510046101057*^9, 3.582510048562183*^9}, {3.5825102847693863`*^9, 
   3.582510290410612*^9}, {3.58251311203472*^9, 3.582513136795985*^9}, {
   3.58251318815694*^9, 3.582513189162957*^9}, {3.5825133743074007`*^9, 
   3.5825134063448973`*^9}, {3.5825134791380787`*^9, 3.582513487257182*^9}, {
   3.582513522044189*^9, 3.582513566624708*^9}, {3.582513613108279*^9, 
   3.58251361347099*^9}, {3.5825136605095997`*^9, 3.582513665317088*^9}, {
   3.58251371140825*^9, 3.582513759439439*^9}, {3.58251388310795*^9, 
   3.5825139109738398`*^9}, {3.582513950944413*^9, 3.582513958305488*^9}, {
   3.5825140800959*^9, 3.5825141322688437`*^9}, {3.582514187951976*^9, 
   3.582514224468692*^9}, {3.582514281227838*^9, 3.5825142815621433`*^9}, {
   3.5825143226499443`*^9, 3.582514344351201*^9}, {3.582514816536549*^9, 
   3.5825148203760843`*^9}, {3.5825409783248653`*^9, 3.58254098239212*^9}, {
   3.582541058077898*^9, 3.5825410586393833`*^9}, {3.582541198230607*^9, 
   3.5825412556135273`*^9}, {3.582542800022154*^9, 3.582542800531509*^9}, {
   3.582547788263611*^9, 3.5825478040788803`*^9}, {3.582548123292501*^9, 
   3.582548132900176*^9}, {3.582548571653658*^9, 3.5825485721015453`*^9}, {
   3.5825531835354013`*^9, 3.5825532438127728`*^9}, {3.582553357479814*^9, 
   3.582553369827639*^9}, 3.58255340798713*^9, {3.582553517781823*^9, 
   3.5825535645906363`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2D, Spherical - Incremental, Flipping", "Subsection",
 CellChangeTimes->{{3.57959451184096*^9, 3.5795945396475697`*^9}, {
  3.5796058660134897`*^9, 3.579605866281661*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Assembles", " ", "hull", " ", "from", " ", "segments"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579608009016799*^9, 3.579608016778125*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2AssembleHull", "[", "segs_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"first", ",", "next", ",", "seg", ",", "vt", ",", "res"}], "}"}],
     ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "segs", "]"}], " ", "\[Equal]", " ", "0"}], 
       ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"first", " ", "=", " ", 
      RowBox[{"Min", "[", 
       RowBox[{"Flatten", "[", "segs", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"next", "[", 
         RowBox[{"seg", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "]"}], " ", "=", " ", 
        RowBox[{"seg", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"seg", ",", "segs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vt", " ", "=", " ", "first"}], ";", "\[IndentingNewLine]", 
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{"While", "[", 
        RowBox[{"True", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Sow", "[", "vt", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"vt", " ", "=", " ", 
           RowBox[{"next", "[", "vt", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"vt", " ", "\[Equal]", " ", "first"}], ",", 
            RowBox[{"Break", "[", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
        "]"}], "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"res", "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.579608027901626*^9, 3.579608093420395*^9}, {
  3.579608241402937*^9, 3.57960837050294*^9}, {3.579899570503228*^9, 
  3.579899597976327*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", " ", "convex", " ", "hull"}], ",", " ", "triangles"}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579607640830434*^9, 3.579607651758263*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2GenInitialTriangulation", "[", 
   RowBox[{
   "vts_", ",", "trieds_", ",", "edtri_", ",", "orient_", ",", "datafunc_", 
    ",", "dataorient_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "tri", ",", "tris", ",", "hullsegs", ",", "sidetest", ",", 
      "ed", ",", "ix", ",", "vt", ",", "k", ",", "kx", ",", "newtris"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Degenerate", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nvts", " ", "=", " ", 
      RowBox[{"Length", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nvts", " ", "\[LessEqual]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Range", "[", "nvts", "]"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Initial", " ", "triangle"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"tri", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"orient", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "tri", "]"}], "]"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"tri", " ", "=", " ", 
        RowBox[{"{", 
         RowBox[{"1", ",", "3", ",", "2"}], "}"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tris", " ", "=", " ", 
      RowBox[{"{", "tri", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"TriEdgeRefs", "[", 
      RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"sidetest", "[", "tri", "]"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"datafunc", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"TriEdges", "[", "tri", "]"}]}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"hullsegs", " ", "=", " ", 
      RowBox[{"Reverse", " ", "/@", " ", 
       RowBox[{"TriEdges", "[", "tri", "]"}]}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "What", " ", "external", " ", "triangles", " ", "contain", " ", "this",
         " ", "vertex"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"vt", " ", "=", " ", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "ix", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"newtris", " ", "=", " ", 
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"RotateToMin", "[", 
              RowBox[{"Append", "[", 
               RowBox[{"#", ",", "ix"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
            "hullsegs"}], ",", 
           RowBox[{
            RowBox[{"orient", "[", 
             RowBox[{"vts", "[", 
              RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "newtris", "]"}], " ", ">", " ", "0"}], ",",
           "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"On", " ", "the", " ", "outside"}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Do", "[", 
            RowBox[{
             RowBox[{"TriEdgeRefs", "[", 
              RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Do", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"sidetest", "[", "tri", "]"}], " ", "=", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"datafunc", "[", 
                  RowBox[{"vts", "[", 
                   RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", 
                " ", 
                RowBox[{"TriEdges", "[", "tri", "]"}]}], ")"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"tris", " ", "=", " ", 
            RowBox[{"Join", "[", 
             RowBox[{"tris", ",", "newtris"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"hullsegs", " ", "=", " ", 
            RowBox[{"Join", "[", 
             RowBox[{"hullsegs", ",", 
              RowBox[{"Flatten", "[", 
               RowBox[{
                RowBox[{"Table", "[", 
                 RowBox[{
                  RowBox[{"Reverse", " ", "/@", " ", 
                   RowBox[{"TriEdges", "[", "tri", "]"}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ",", 
                "1"}], "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"hullsegs", " ", "=", " ", 
            RowBox[{"Select", "[", 
             RowBox[{"hullsegs", ",", 
              RowBox[{
               RowBox[{
                RowBox[{"Head", "[", 
                 RowBox[{"edtri", "[", "#", "]"}], "]"}], "===", "edtri"}], 
               "&"}]}], "]"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"On", " ", "the", " ", "inside"}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "Search", " ", "in", " ", "reverse", " ", "to", " ", "check", " ", 
            "most", " ", "recent", " ", "triangles", " ", "first"}], " ", 
           "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"kx", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
           
           RowBox[{"Do", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"tri", " ", "=", " ", 
               RowBox[{"tris", "[", 
                RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
              
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"And", " ", "@@", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"dataorient", "[", 
                    RowBox[{"#", ",", "vt"}], "]"}], "&"}], " ", "/@", " ", 
                   RowBox[{"sidetest", "[", "tri", "]"}]}], ")"}]}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"kx", " ", "=", " ", "k"}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", 
               "]"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{"k", ",", 
               RowBox[{"Length", "[", "tris", "]"}], ",", "1", ",", 
               RowBox[{"-", "1"}]}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"kx", " ", "=!=", " ", "Null"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{"Inside", " ", "a", " ", 
               RowBox[{"triangle", ":", " ", 
                RowBox[{"split", " ", "it", " ", "in", " ", "three"}]}]}], 
              " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tri", " ", "=", " ", 
               RowBox[{"tris", "[", 
                RowBox[{"[", "kx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"newtris", " ", "=", " ", 
               RowBox[{
                RowBox[{
                 RowBox[{"RotateToMin", "[", 
                  RowBox[{"Append", "[", 
                   RowBox[{"#", ",", "ix"}], "]"}], "]"}], "&"}], " ", "/@", 
                " ", 
                RowBox[{"TriEdges", "[", "tri", "]"}]}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{"TriEdgeRefs", "[", 
                 RowBox[{"tri", ",", "trieds", ",", "edtri"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"sidetest", "[", "tri", "]"}], " ", "=", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"datafunc", "[", 
                    RowBox[{"vts", "[", 
                    RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", 
                   " ", 
                   RowBox[{"TriEdges", "[", "tri", "]"}]}], ")"}]}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{"tri", ",", "newtris"}], "}"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"tris", "[", 
                RowBox[{"[", "kx", "]"}], "]"}], " ", "=", " ", 
               RowBox[{"First", "[", "newtris", "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"tris", " ", "=", " ", 
               RowBox[{"Join", "[", 
                RowBox[{"tris", ",", 
                 RowBox[{"Rest", "[", "newtris", "]"}]}], "]"}]}]}]}], 
            "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"ix", ",", "4", ",", "nvts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", "Finally", " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"DT2AssembleHull", "[", 
        RowBox[{"Reverse", " ", "/@", " ", "hullsegs"}], "]"}], ",", "tris"}],
       "}"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5796063512855597`*^9, 3.579606361417362*^9}, {
   3.579606551012455*^9, 3.579606598364398*^9}, {3.579606749823884*^9, 
   3.579606784391337*^9}, {3.579606829695773*^9, 3.5796068351996202`*^9}, {
   3.579606904847234*^9, 3.579606907800363*^9}, {3.5796069658713284`*^9, 
   3.57960697440173*^9}, {3.579607088526928*^9, 3.579607089378531*^9}, 
   3.5796071554616823`*^9, {3.579607349247498*^9, 3.579607357708149*^9}, {
   3.579607543371738*^9, 3.5796076364366503`*^9}, {3.579608400071253*^9, 
   3.579608481518299*^9}, {3.579608611754086*^9, 3.579608637703988*^9}, {
   3.5796088031625643`*^9, 3.579608894605483*^9}, {3.5796089322051773`*^9, 
   3.579608953766452*^9}, {3.579610052605291*^9, 3.5796103067148657`*^9}, {
   3.579610348939172*^9, 3.579610457319659*^9}, {3.579610504727736*^9, 
   3.579610547958503*^9}, {3.5796106028526783`*^9, 3.579610607806767*^9}, {
   3.579610703774761*^9, 3.579610770980101*^9}, {3.579610810970675*^9, 
   3.5796109308731737`*^9}, {3.5796109613873053`*^9, 3.579610999797845*^9}, {
   3.579611053490999*^9, 3.579611068470128*^9}, {3.579611148180848*^9, 
   3.579611172444878*^9}, {3.579611227515313*^9, 3.5796112330365133`*^9}, {
   3.579611807497981*^9, 3.579611840365067*^9}, {3.579611889497382*^9, 
   3.579611929505889*^9}, {3.579611998200184*^9, 3.579612009980508*^9}, {
   3.579629337655058*^9, 3.5796293413105927`*^9}, 3.57963292224863*^9, {
   3.579634424252461*^9, 3.579634515044135*^9}, {3.579634589096731*^9, 
   3.579634604069799*^9}, {3.579634774698875*^9, 3.579634776583987*^9}, {
   3.579899625099494*^9, 3.579899627595477*^9}, 3.581584799820998*^9, {
   3.581585066484758*^9, 3.5815850877303257`*^9}, {3.582086119750041*^9, 
   3.582086239757587*^9}, {3.582090805276917*^9, 3.582090806696844*^9}, {
   3.5820908553085546`*^9, 3.582090870494133*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2GenEdgeFlipping", "[", 
   RowBox[{
   "vts_", ",", "origtris_", ",", " ", "trieds_", ",", "edtri_", ",", 
    "orient_", ",", "ccdata_", ",", "ccinside_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "tris", ",", "triixs", ",", "edcnts", ",", "eds", ",", "neds", ",", 
      "edixs", ",", "edtests", ",", "edcnt", ",", "isinted", ",", 
      RowBox[{"totflips", "=", 
       RowBox[{"{", "}"}]}], ",", "numflips", ",", "k", ",", "ed", ",", 
      "tri1", ",", "tri2", ",", "ix1", ",", "ix2", ",", "ccdtri", ",", "ins1",
       ",", "ins2", ",", "k1", ",", "k2", ",", "tor1", ",", "tor2"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Index", " ", "the", " ", "triangles"}], " ", "*)"}], 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"tris", " ", "=", " ", "origtris"}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"triixs", "[", 
         RowBox[{"tris", "[", 
          RowBox[{"[", "k", "]"}], "]"}], "]"}], " ", "=", " ", "k"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "tris", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "edges"}], ",", " ", 
       RowBox[{"index", " ", "them"}], ",", " ", 
       RowBox[{
       "and", " ", "mark", " ", "them", " ", "as", " ", "to", " ", "be", " ", 
        
        RowBox[{"examined", ".", " ", "Their"}], " ", "vertices", " ", "are", 
        " ", "in", " ", "sorted", " ", "order", " ", "to", " ", "resolve", 
        " ", "ambiguity"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"edcnts", " ", "=", " ", 
      RowBox[{"Tally", "[", 
       RowBox[{"Sort", " ", "/@", " ", 
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"TriEdges", " ", "/@", " ", "tris"}], ",", "1"}], "]"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"eds", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", "edcnts"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"neds", " ", "=", " ", 
      RowBox[{"Length", "[", "eds", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"edixs", "[", 
         RowBox[{"eds", "[", 
          RowBox[{"[", "k", "]"}], "]"}], "]"}], " ", "=", " ", "k"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "neds"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"edtests", " ", "=", " ", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"True", ",", "neds"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "which", " ", "ones", " ", "are", " ", "interior", " ", 
       "edges"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"isinted", " ", "=", " ", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"False", ",", "neds"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"isinted", "[", 
         RowBox[{"[", 
          RowBox[{"edixs", "[", 
           RowBox[{"edcnt", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], "]"}], " ", "=", 
        " ", 
        RowBox[{
         RowBox[{"edcnt", "[", 
          RowBox[{"[", "2", "]"}], "]"}], ">", "1"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"edcnt", ",", "edcnts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Do", " ", "as", " ", "many", " ", "passes", " ", "as", " ", 
       "necessary"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"numflips", " ", "=", " ", "0"}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", 
              RowBox[{"isinted", "[", 
               RowBox[{"[", "k", "]"}], "]"}]}], ",", 
             RowBox[{"Continue", "[", "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", 
              RowBox[{"edtests", "[", 
               RowBox[{"[", "k", "]"}], "]"}]}], ",", 
             RowBox[{"Continue", "[", "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "Get", " ", "the", " ", "edge", " ", "and", " ", "its", " ", 
             "triangles", " ", "and", " ", "their", " ", "vertices"}], " ", 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{"ed", " ", "=", " ", 
            RowBox[{"eds", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"tri1", " ", "=", " ", 
            RowBox[{"edtri", "[", "ed", "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"tri2", " ", "=", " ", 
            RowBox[{"edtri", "[", 
             RowBox[{"Reverse", "[", "ed", "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ix1", " ", "=", " ", 
            RowBox[{"First", "[", 
             RowBox[{"Complement", "[", 
              RowBox[{"tri1", ",", "ed"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ix2", " ", "=", " ", 
            RowBox[{"First", "[", 
             RowBox[{"Complement", "[", 
              RowBox[{"tri2", ",", "ed"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "Find", " ", "the", " ", "circumcircles", " ", "of", " ", "the", 
             " ", 
             RowBox[{"triangles", ".", " ", "Are"}], " ", "the", " ", 
             "opposite", " ", "vertices", " ", "of", " ", "the", " ", "other",
              " ", "triangles", " ", "in", " ", 
             RowBox[{"them", "?"}]}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Head", "[", 
               RowBox[{"ccdtri", "[", "tri1", "]"}], "]"}], " ", "===", " ", 
              "ccdtri"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ccdtri", "[", "tri1", "]"}], " ", "=", " ", 
              RowBox[{"ccdata", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "tri1", "]"}], "]"}], "]"}]}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Head", "[", 
               RowBox[{"ccdtri", "[", "tri2", "]"}], "]"}], " ", "===", " ", 
              "ccdtri"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ccdtri", "[", "tri2", "]"}], " ", "=", " ", 
              RowBox[{"ccdata", "[", 
               RowBox[{"vts", "[", 
                RowBox[{"[", "tri2", "]"}], "]"}], "]"}]}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"ins1", " ", "=", " ", 
            RowBox[{"ccinside", "[", 
             RowBox[{
              RowBox[{"ccdtri", "[", "tri1", "]"}], ",", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "ix2", "]"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ins2", " ", "=", " ", 
            RowBox[{"ccinside", "[", 
             RowBox[{
              RowBox[{"ccdtri", "[", "tri2", "]"}], ",", 
              RowBox[{"vts", "[", 
               RowBox[{"[", "ix1", "]"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"ins1", " ", "||", " ", "ins2"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "Compose", " ", "the", " ", "new", " ", "triangles", " ", "and",
                " ", "check", " ", "them"}], " ", "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"k1", " ", "=", " ", 
               RowBox[{"triixs", "[", "tri1", "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"k2", " ", "=", " ", 
               RowBox[{"triixs", "[", "tri2", "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"tri1", " ", "=", " ", 
               RowBox[{"RotateToMin", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"ed", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], ",", "ix2", ",", "ix1"}], 
                 "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"tri2", " ", "=", " ", 
               RowBox[{"RotateToMin", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"ed", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], ",", "ix1", ",", "ix2"}], 
                 "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"tor1", " ", "=", " ", 
               RowBox[{"orient", "[", 
                RowBox[{"vts", "[", 
                 RowBox[{"[", "tri1", "]"}], "]"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"tor2", " ", "=", " ", 
               RowBox[{"orient", "[", 
                RowBox[{"vts", "[", 
                 RowBox[{"[", "tri2", "]"}], "]"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"tor1", " ", "&&", " ", "tor2"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"(*", " ", 
                 RowBox[{
                 "Put", " ", "in", " ", "the", " ", "new", " ", "edge", " ", 
                  "and", " ", "triangles"}], " ", "*)"}], 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"numflips", "++"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"ed", " ", "=", " ", 
                  RowBox[{"Sort", "[", 
                   RowBox[{"{", 
                    RowBox[{"ix1", ",", "ix2"}], "}"}], "]"}]}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"eds", "[", 
                   RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "ed"}], ";",
                  "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"edixs", "[", "ed", "]"}], " ", "=", " ", "k"}], 
                 ";", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"tris", "[", 
                   RowBox[{"[", "k1", "]"}], "]"}], " ", "=", " ", "tri1"}], 
                 ";", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"tris", "[", 
                   RowBox[{"[", "k2", "]"}], "]"}], " ", "=", " ", "tri2"}], 
                 ";", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"triixs", "[", "tri1", "]"}], " ", "=", " ", "k1"}],
                  ";", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"triixs", "[", "tri2", "]"}], " ", "=", " ", "k2"}],
                  ";", "\[IndentingNewLine]", 
                 RowBox[{"TriEdgeRefs", "[", 
                  RowBox[{"tri1", ",", "trieds", ",", "edtri"}], "]"}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{"TriEdgeRefs", "[", 
                  RowBox[{"tri2", ",", "trieds", ",", "edtri"}], "]"}], ";", 
                 "\[IndentingNewLine]", 
                 RowBox[{"(*", " ", 
                  RowBox[{
                  "Mark", " ", "all", " ", "affected", " ", "edges", " ", 
                   "as", " ", "to", " ", "be", " ", "examined"}], " ", "*)"}],
                  "\[IndentingNewLine]", 
                 RowBox[{"Do", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"edtests", "[", 
                    RowBox[{"[", 
                    RowBox[{"edixs", "[", 
                    RowBox[{"Sort", "[", "ed", "]"}], "]"}], "]"}], "]"}], 
                    " ", "=", " ", "True"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"ed", ",", 
                    RowBox[{"trieds", "[", "tri1", "]"}]}], "}"}]}], "]"}], 
                 ";", "\[IndentingNewLine]", 
                 RowBox[{"Do", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"edtests", "[", 
                    RowBox[{"[", 
                    RowBox[{"edixs", "[", 
                    RowBox[{"Sort", "[", "ed", "]"}], "]"}], "]"}], "]"}], 
                    " ", "=", " ", "True"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"ed", ",", 
                    RowBox[{"trieds", "[", "tri2", "]"}]}], "}"}]}], 
                  "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "This", " ", "edge", " ", "does", " ", "not", " ", "need", " ", 
               "further", " ", "examination", " ", "in", " ", "this", " ", 
               "step"}], " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"edtests", "[", 
               RowBox[{"[", "k", "]"}], "]"}], " ", "=", " ", "False"}]}], 
            "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "neds"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"totflips", ",", "numflips"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"numflips", " ", "\[Equal]", " ", "0"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"{", 
        RowBox[{"16", "*", "neds"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"tris", ",", "totflips"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.579612063877063*^9, 3.57961210774358*^9}, {
   3.579612470062376*^9, 3.579612526028146*^9}, {3.5796126566230516`*^9, 
   3.579612744029209*^9}, {3.5796128220677547`*^9, 3.5796128478282557`*^9}, {
   3.579612947799955*^9, 3.579613016424615*^9}, {3.579613223571537*^9, 
   3.579613332692357*^9}, {3.5796134234053307`*^9, 3.5796134620870037`*^9}, {
   3.579613590970634*^9, 3.5796136532907667`*^9}, {3.5796138709230623`*^9, 
   3.579613903790411*^9}, {3.579623039796219*^9, 3.579623053210868*^9}, {
   3.5796285472756367`*^9, 3.579628633670333*^9}, {3.579628670303154*^9, 
   3.5796286711793957`*^9}, {3.5796287056954193`*^9, 3.579628779387539*^9}, {
   3.579628882274994*^9, 3.5796289091395817`*^9}, {3.5796289418551893`*^9, 
   3.579628997623238*^9}, {3.579629042921915*^9, 3.5796293656936407`*^9}, {
   3.579629399309988*^9, 3.579629481326762*^9}, {3.579629519555032*^9, 
   3.579629520009121*^9}, {3.5796295533356857`*^9, 3.57962956430243*^9}, {
   3.5796295963380947`*^9, 3.579629601458084*^9}, {3.579629679188582*^9, 
   3.579629811069087*^9}, {3.579630621783502*^9, 3.579630641096575*^9}, {
   3.5796307039295607`*^9, 3.579630771131385*^9}, {3.579630886956382*^9, 
   3.579630912744175*^9}, {3.5796327669397097`*^9, 3.579632788532535*^9}, {
   3.5796328218061028`*^9, 3.579632835829561*^9}, {3.579632912561301*^9, 
   3.5796329423507977`*^9}, {3.5796329876897497`*^9, 3.579632999976437*^9}, {
   3.57963384900515*^9, 3.579633882901999*^9}, {3.579634011485435*^9, 
   3.579634015499977*^9}, {3.579634290928968*^9, 3.579634297288521*^9}, {
   3.579634341054105*^9, 3.579634345143917*^9}, {3.579634498077837*^9, 
   3.579634498573203*^9}, {3.5796348642486486`*^9, 3.579634871064163*^9}, {
   3.579634961288479*^9, 3.5796350019432077`*^9}, {3.579635045038991*^9, 
   3.57963510428784*^9}, {3.579635155958005*^9, 3.579635159494656*^9}, {
   3.579635199772851*^9, 3.579635201187011*^9}, {3.579635253663102*^9, 
   3.5796352610485086`*^9}, {3.579635319018127*^9, 3.579635322876069*^9}, {
   3.579635801762*^9, 3.579635805734398*^9}, {3.579635903815271*^9, 
   3.579635908833308*^9}, {3.5796361434207897`*^9, 3.5796364323898287`*^9}, {
   3.579636490285606*^9, 3.579636522006236*^9}, {3.579636811036285*^9, 
   3.5796368166349487`*^9}, {3.579636865639873*^9, 3.5796369304754267`*^9}, {
   3.579639069902684*^9, 3.57963911509347*^9}, {3.579639256156414*^9, 
   3.579639264282255*^9}, 3.5797308439297867`*^9, {3.5797310912707148`*^9, 
   3.579731093462118*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2GenIncrFlipBase", "[", 
   RowBox[{
   "vts_", ",", "orient_", ",", "datafunc_", ",", "dataorient_", ",", 
    "ccdata_", ",", "ccinside_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "trieds", ",", " ", "edtri", ",", " ", "hull", ",", " ", "tris", ",", 
      "totflips"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"hull", ",", " ", "tris"}], "}"}], " ", "=", " ", 
      RowBox[{"DT2GenInitialTriangulation", "[", 
       RowBox[{
       "vts", ",", "trieds", ",", "edtri", ",", "orient", ",", "datafunc", 
        ",", "dataorient"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tris", ",", "totflips"}], "}"}], " ", "=", " ", 
      RowBox[{"DT2GenEdgeFlipping", "[", 
       RowBox[{
       "vts", ",", "tris", ",", "trieds", ",", "edtri", ",", "orient", ",", 
        "ccdata", ",", "ccinside"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "hull", "}"}], ",", " ", 
       RowBox[{"Sort", "[", "tris", "]"}], ",", 
       RowBox[{"{", "}"}], ",", "totflips"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579606615594164*^9, 3.5796066578202543`*^9}, {
  3.579606845046483*^9, 3.579606858212*^9}, {3.579606981140279*^9, 
  3.5796069931546173`*^9}, {3.579607395440187*^9, 3.5796073987569027`*^9}, {
  3.5796120410323753`*^9, 3.5796120483863297`*^9}, {3.579612121683819*^9, 
  3.5796121733616056`*^9}, {3.579612233819352*^9, 3.579612236930735*^9}, {
  3.579612514158173*^9, 3.579612514861474*^9}, {3.579628795747093*^9, 
  3.579628822385714*^9}, {3.579630844272147*^9, 3.5796308552208347`*^9}, {
  3.5796346660545473`*^9, 3.579634669692005*^9}, {3.579634801183489*^9, 
  3.579634804487493*^9}, {3.579637123547018*^9, 3.579637123959113*^9}, {
  3.579638983436491*^9, 3.579638985106333*^9}, {3.5796391067901707`*^9, 
  3.5796391309325333`*^9}, {3.579734120794805*^9, 3.579734123369955*^9}, {
  3.5798341745075817`*^9, 3.579834176160347*^9}, {3.582086360869001*^9, 
  3.582086361596058*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2ReindexRes", "[", 
   RowBox[{"ixs_", ",", "xlss_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Sort", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"RotateToMin", "[", 
      RowBox[{"ixs", "[", 
       RowBox[{"[", "xlst", "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"xlst", ",", "xlss"}], "}"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58208699993311*^9, 3.582087015166513*^9}, {
  3.58208706071808*^9, 3.58208711625528*^9}, {3.582088066370071*^9, 
  3.582088068305874*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2GenIncrFlip", "[", 
   RowBox[{
   "vts_", ",", "orient_", ",", "datafunc_", ",", "dataorient_", ",", 
    "ccdata_", ",", "ccinside_", ",", "sorttype_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "sorted", ",", "ixvts", ",", "ixs", ",", "svts", ",", "sortkeys", ",", 
      "avg", ",", "dtres"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"sorted", " ", "=", " ", 
      RowBox[{"Switch", "[", 
       RowBox[{
       "sorttype", ",", "\[IndentingNewLine]", "\"\<lin\>\"", ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{"AddIndices", "[", "vts", "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"Append", "[", 
             RowBox[{"#", ",", 
              RowBox[{"First", "[", 
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "]"}], "&"}], " ", "/@",
            " ", "ixvts"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{"SortBy", "[", 
           RowBox[{"ixvts", ",", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "3", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ixs", ",", "svts", ",", "sortkeys"}], "}"}], " ", "=", 
          " ", 
          RowBox[{"Transpose", "[", "ixvts", "]"}]}], ";", 
         "\[IndentingNewLine]", "True"}], ",", "\[IndentingNewLine]", 
        "\"\<rad\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{"AddIndices", "[", "vts", "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"avg", " ", "=", " ", 
          RowBox[{"Mean", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"Append", "[", 
             RowBox[{"#", ",", 
              RowBox[{"VecSqr", "[", 
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "2", "]"}], "]"}], "-", "avg"}], "]"}]}], "]"}],
             "&"}], " ", "/@", " ", "ixvts"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ixvts", " ", "=", " ", 
          RowBox[{"SortBy", "[", 
           RowBox[{"ixvts", ",", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "3", "]"}], "]"}], "&"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ixs", ",", "svts", ",", "sortkeys"}], "}"}], " ", "=", 
          " ", 
          RowBox[{"Transpose", "[", "ixvts", "]"}]}], ";", 
         "\[IndentingNewLine]", "True"}], ",", "\[IndentingNewLine]", "_", 
        ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"svts", " ", "=", " ", "vts"}], ";", "\[IndentingNewLine]", 
         "False"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dtres", " ", "=", " ", 
      RowBox[{"DT2GenIncrFlipBase", "[", 
       RowBox[{
       "svts", ",", "orient", ",", "datafunc", ",", "dataorient", ",", 
        "ccdata", ",", "ccinside"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"sorted", ",", "\[IndentingNewLine]", 
       RowBox[{"dtres", " ", "=", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"DT2ReindexRes", "[", 
           RowBox[{"ixs", ",", 
            RowBox[{"dtres", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
          RowBox[{"DT2ReindexRes", "[", 
           RowBox[{"ixs", ",", 
            RowBox[{"dtres", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
          RowBox[{"dtres", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", 
          RowBox[{"dtres", "[", 
           RowBox[{"[", "4", "]"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "dtres"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.582086356673369*^9, 3.5820864046488237`*^9}, {
   3.582086735389504*^9, 3.582086790374008*^9}, {3.582086885348442*^9, 
   3.5820869070481987`*^9}, {3.582087128628974*^9, 3.582087273430279*^9}, {
   3.582087341138793*^9, 3.582087472428772*^9}, 3.582087514493923*^9, {
   3.582087546348558*^9, 3.582087576446823*^9}, {3.582087609218834*^9, 
   3.582087664105925*^9}, {3.582087713707087*^9, 3.582087721051272*^9}, {
   3.5820877977957067`*^9, 3.582087850257597*^9}, {3.5820879483613234`*^9, 
   3.5820879614110603`*^9}, {3.582088010623435*^9, 3.582088019329235*^9}, {
   3.58208804999953*^9, 3.582088056207363*^9}, {3.5821875545077143`*^9, 
   3.582187556477581*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2IncrFlip", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"sorttype_:", "Null"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"DT2GenIncrFlip", "[", 
   RowBox[{
   "vts", ",", "TriOrient2D", ",", "TriAreaData2D", ",", "TriOrientEval2D", 
    ",", "CircumData", ",", "CircumInside", ",", "sorttype"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.579606998050056*^9, 3.579607035824561*^9}, {
  3.579628828856094*^9, 3.579628834109796*^9}, {3.57963900419353*^9, 
  3.579639016670793*^9}, {3.58208679846756*^9, 3.582086801556068*^9}, {
  3.5820868556547213`*^9, 3.582086860733007*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2IncrFlipLin", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DT2IncrFlip", "[", 
   RowBox[{"vts", ",", "\"\<lin\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820838262643213`*^9, 3.5820838583576393`*^9}, {
  3.582083902329357*^9, 3.58208390890202*^9}, {3.582083943959168*^9, 
  3.58208394495013*^9}, {3.582083976286541*^9, 3.582083977078333*^9}, {
  3.5820840131821613`*^9, 3.582084018806662*^9}, {3.5820881129078817`*^9, 
  3.582088122802895*^9}, {3.5820888248067913`*^9, 3.582088832457014*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2IncrFlipRad", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DT2IncrFlip", "[", 
   RowBox[{"vts", ",", "\"\<rad\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820838262643213`*^9, 3.5820838583576393`*^9}, {
  3.582083902329357*^9, 3.58208390890202*^9}, {3.582083943959168*^9, 
  3.58208394495013*^9}, {3.582083976286541*^9, 3.582083977078333*^9}, {
  3.5820840131821613`*^9, 3.582084018806662*^9}, {3.582088117186002*^9, 
  3.582088125874095*^9}, {3.582088841010404*^9, 3.5820888476414337`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2SphIncrFlip", "[", 
   RowBox[{"vts_", ",", 
    RowBox[{"sorttype_:", "Null"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"DT2GenIncrFlip", "[", 
   RowBox[{
   "vts", ",", "TriOrient2Sph", ",", "TriAreaData2Sph", ",", 
    "TriOrientEval2Sph", ",", "SphCircumData", ",", "SphCircumInside", ",", 
    "sorttype"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579606998050056*^9, 3.57960707043288*^9}, {
   3.579628840508144*^9, 3.579628845125901*^9}, 3.579638775655077*^9, {
   3.579639006841715*^9, 3.5796390245109463`*^9}, {3.5796392090336933`*^9, 
   3.5796392097054663`*^9}, {3.5815837222646914`*^9, 3.581583729372957*^9}, 
   3.582084251163822*^9, {3.582086864816305*^9, 3.5820868751757803`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2SphIncrFlipLin", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DT2SphIncrFlip", "[", 
   RowBox[{"vts", ",", "\"\<lin\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820838262643213`*^9, 3.5820838583576393`*^9}, {
  3.582083902329357*^9, 3.58208390890202*^9}, {3.582083943959168*^9, 
  3.58208394495013*^9}, {3.582083976286541*^9, 3.582083977078333*^9}, {
  3.5820840131821613`*^9, 3.582084018806662*^9}, {3.5820881129078817`*^9, 
  3.582088122802895*^9}, {3.5820888248067913`*^9, 3.582088832457014*^9}, {
  3.582088925300234*^9, 3.58208893629034*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2SphIncrFlipRad", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DT2SphIncrFlip", "[", 
   RowBox[{"vts", ",", "\"\<rad\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820838262643213`*^9, 3.5820838583576393`*^9}, {
  3.582083902329357*^9, 3.58208390890202*^9}, {3.582083943959168*^9, 
  3.58208394495013*^9}, {3.582083976286541*^9, 3.582083977078333*^9}, {
  3.5820840131821613`*^9, 3.582084018806662*^9}, {3.582088117186002*^9, 
  3.582088125874095*^9}, {3.582088841010404*^9, 3.5820888476414337`*^9}, {
  3.5820889287065372`*^9, 3.582088933361917*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["General - Convex Hull", "Subsection",
 CellChangeTimes->{{3.5798803674647417`*^9, 3.579880376637347*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Split", " ", "a", " ", "hull", " ", "using", " ", "a", " ", 
   RowBox[{"ring", ".", " ", "This"}], " ", "will", " ", "work", " ", "for", 
   " ", "any", " ", "simplex", " ", "dimension"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579889123888165*^9, 3.579889129170595*^9}, {
  3.579889183515114*^9, 3.579889193234797*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSplitHull", "[", 
   RowBox[{"faces_", ",", "ringsubfcs_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "face", ",", "fctosbfcs", ",", "sbfctofcs", ",", "sbfc", ",", "sbfcuse", 
      ",", "rngfaces", ",", "facestat", ",", "unchecked", ",", "k", ",", 
      "nfaces", ",", "facex"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "faces", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "ringsubfcs", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{"faces", ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
       "Get", " ", "the", " ", "face", " ", "and", " ", "subface", " ", 
        "references"}], ",", " ", 
       RowBox[{
       "their", " ", "references", " ", "to", " ", "each", " ", "other"}]}], 
      " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"SimplexSubRefs", "[", 
        RowBox[{"face", ",", "fctosbfcs", ",", "sbfctofcs"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"face", ",", "faces"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Ring", " ", 
       RowBox[{"edges", ":", " ", 
        RowBox[{
         RowBox[{"don", "'"}], "t", " ", "expand", " ", "past", " ", 
         "those"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"sbfcuse", "[", "sbfc", "]"}], " ", "=", " ", "True"}], ",", 
         RowBox[{"{", 
          RowBox[{"sbfc", ",", 
           RowBox[{"fctosbfcs", "[", "face", "]"}]}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"face", ",", "faces"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"sbfcuse", "[", "sbfc", "]"}], " ", "=", " ", "False"}], ",", 
       
       RowBox[{"{", 
        RowBox[{"sbfc", ",", "ringsubfcs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
        "Find", " ", "which", " ", "existing", " ", "faces", " ", "are", " ", 
         "accessible", " ", "from", " ", "the", " ", "ring", " ", 
         RowBox[{"subfaces", ".", " ", "0"}]}], " ", "=", " ", "unchecked"}], 
       ",", " ", 
       RowBox[{"1", " ", "=", " ", "checked"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"rngfaces", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"sbfctofcs", "[", "sbfc", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"sbfc", ",", "ringsubfcs"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"facestat", "[", "face", "]"}], " ", "=", " ", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"face", ",", "rngfaces"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"unchecked", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"unchecked", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"unchecked", " ", "=", " ", "False"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"nfaces", " ", "=", " ", 
         RowBox[{"Length", "[", "rngfaces", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"face", " ", "=", " ", 
            RowBox[{"rngfaces", "[", 
             RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"facestat", "[", "face", "]"}], " ", "\[Equal]", " ", 
              "0"}], ",", " ", 
             RowBox[{
              RowBox[{"unchecked", " ", "=", " ", "True"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"(*", " ", 
               RowBox[{"Look", " ", "at", " ", "a", " ", 
                RowBox[{"triangle", "'"}], "s", " ", "neighboring", " ", 
                "triangles"}], " ", "*)"}], "\[IndentingNewLine]", 
              RowBox[{"Do", "[", 
               RowBox[{
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"sbfcuse", "[", "sbfc", "]"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"facex", " ", "=", " ", 
                    RowBox[{"sbfctofcs", "[", 
                    RowBox[{"SimplexFlip", "[", "sbfc", "]"}], "]"}]}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"facestat", "[", "facex", "]"}], "]"}], " ", "===",
                     " ", "facestat"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"rngfaces", ",", "facex"}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"facestat", "[", "facex", "]"}], " ", "=", " ", 
                    "0"}]}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{"sbfc", ",", 
                  RowBox[{"fctosbfcs", "[", "face", "]"}]}], "}"}]}], "]"}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"facestat", "[", "face", "]"}], " ", "=", " ", 
               "1"}]}]}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"k", ",", "nfaces"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Union", "[", "rngfaces", "]"}], ",", 
       RowBox[{"Complement", "[", 
        RowBox[{"faces", ",", "rngfaces"}], "]"}]}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5771711338525057`*^9, 3.5771711649632187`*^9}, {
   3.577171282942721*^9, 3.577171304018222*^9}, {3.577171364307446*^9, 
   3.5771713712219963`*^9}, 3.57719619688378*^9, {3.577200502331499*^9, 
   3.57720055843417*^9}, {3.577200735335083*^9, 3.577200836832486*^9}, {
   3.577201200400715*^9, 3.577201213754002*^9}, {3.577201254516172*^9, 
   3.577201260515053*^9}, {3.577201461381238*^9, 3.577201474940029*^9}, {
   3.577201537689073*^9, 3.577201547720005*^9}, {3.57720163612252*^9, 
   3.577201660592013*^9}, {3.577202108637879*^9, 3.577202161239538*^9}, {
   3.577202194575162*^9, 3.5772023723559628`*^9}, {3.5772024826212997`*^9, 
   3.5772025538711777`*^9}, {3.577202590307753*^9, 3.5772026136881733`*^9}, {
   3.57720266816428*^9, 3.5772027728130283`*^9}, {3.577202817363633*^9, 
   3.577202847580779*^9}, {3.57720301778358*^9, 3.577203066896846*^9}, 
   3.5772031141147957`*^9, {3.577203210743087*^9, 3.577203248815597*^9}, {
   3.5772033199154663`*^9, 3.5772033938368073`*^9}, {3.577204280200787*^9, 
   3.577204300095449*^9}, {3.577204394717969*^9, 3.5772044466301603`*^9}, {
   3.5772048001526546`*^9, 3.577204860003337*^9}, {3.577204930130249*^9, 
   3.577204938499301*^9}, {3.5772051007991667`*^9, 3.5772051899740353`*^9}, {
   3.577205314097542*^9, 3.577205357810549*^9}, {3.5772061574196453`*^9, 
   3.5772061822385807`*^9}, {3.577206243015979*^9, 3.5772063704205303`*^9}, {
   3.577206440315189*^9, 3.577206525543146*^9}, {3.5772065956398993`*^9, 
   3.5772066014706182`*^9}, {3.577206652885088*^9, 3.577206653213728*^9}, {
   3.5772068411031017`*^9, 3.577206850011878*^9}, {3.577209078607584*^9, 
   3.577209119888495*^9}, {3.577209240618597*^9, 3.577209268714266*^9}, {
   3.577210078780532*^9, 3.57721007940431*^9}, 3.577270527779203*^9, 
   3.577270590327886*^9, {3.578446813028645*^9, 3.578446831726253*^9}, {
   3.579022556488077*^9, 3.579022565465954*^9}, {3.579881287546735*^9, 
   3.5798812913749037`*^9}, {3.579881565589872*^9, 3.579881566035306*^9}, {
   3.579889114678527*^9, 3.579889179821355*^9}, {3.579889218594359*^9, 
   3.5798892298221207`*^9}, {3.579889669614513*^9, 3.579889678890533*^9}, {
   3.579893887318584*^9, 3.5798939148336143`*^9}, {3.5798973210322113`*^9, 
   3.5798973298675413`*^9}, {3.579897439517995*^9, 3.579897461565442*^9}, 
   3.5798974984462767`*^9, {3.579897559971669*^9, 3.5798975874417753`*^9}, {
   3.579897672945581*^9, 3.579897707806304*^9}, {3.579897821963451*^9, 
   3.5798978351945343`*^9}, {3.579897870306786*^9, 3.5798979096148777`*^9}, {
   3.57989797456149*^9, 3.5798979765148*^9}, {3.57989805430189*^9, 
   3.579898060100383*^9}, {3.5798981020422773`*^9, 3.579898279809264*^9}, 
   3.579899131623044*^9, {3.579899876617345*^9, 3.579899890163064*^9}, {
   3.579959972749531*^9, 3.579959978082636*^9}, {3.5821152943126297`*^9, 
   3.582115297379486*^9}, {3.582115354951941*^9, 3.582115355997444*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Append", " ", "the", " ", "square", " ", "to", " ", "put", " ", "the", 
    " ", "vertices", " ", "on", " ", "a", " ", 
    RowBox[{"parabola", "/", "paraboloid"}], " ", "in", " ", "a", " ", "1", 
    " ", "higher", " ", "dimensional", " ", 
    RowBox[{"space", ".", " ", "Also"}], " ", "make", " ", "all", " ", "the", 
    " ", "variation", " ", "between"}], " ", "-", 
   RowBox[{"1", " ", "and", " ", "1"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579882957054661*^9, 3.579883006131444*^9}, {
  3.579883075702318*^9, 3.5798830845394173`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTParabExtend", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vt", ",", "vtsmean", ",", "vtsnorm", ",", "adjvts"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vtsmean", " ", "=", " ", 
      RowBox[{"Mean", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"adjvts", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"vt", " ", "-", " ", "vtsmean"}], ",", 
        RowBox[{"{", 
         RowBox[{"vt", ",", "vts"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vtsnorm", " ", "=", " ", 
      RowBox[{"Max", "[", 
       RowBox[{"Norm", " ", "/@", " ", "adjvts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"vtsnorm", " ", "\[Equal]", " ", "0"}], ",", " ", 
       RowBox[{"vtsnorm", " ", "=", " ", "1"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"adjvts", " ", "/=", " ", "vtsnorm"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Append", "[", 
        RowBox[{"vt", ",", 
         RowBox[{"vt", ".", "vt"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"vt", ",", "adjvts"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579882824188225*^9, 3.579882947022499*^9}, {
  3.579883024825725*^9, 3.57988303171627*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Which", " ", "function", " ", "to", " ", "use"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579883744507985*^9, 3.579883757346435*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTBoundaryCH2", " ", "=", " ", "CH2QuickHull"}], ";"}]], "Input",
 CellChangeTimes->{{3.57988350063932*^9, 3.579883504254953*^9}, {
  3.579883570087994*^9, 3.579883576239534*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTBoundaryCH3", " ", "=", " ", "CH3QuickHull"}], ";"}]], "Input",
 CellChangeTimes->{{3.579883578003933*^9, 3.579883584151802*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTBoundaryCHN", " ", "=", " ", "CHNQuickHull"}], ";"}]], "Input",
 CellChangeTimes->{{3.579883585932123*^9, 3.579883593192005*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphBoundaryCH2", " ", "=", " ", "CH2SphIncremental"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.579883639577334*^9, 3.579883667274931*^9}, 
   3.581663392873925*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphBoundaryCHN", " ", "=", " ", "CHNSphIncremental"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.579883639577334*^9, 3.579883667274931*^9}, {
  3.579883716723769*^9, 3.579883723961718*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTTessCH3", " ", "=", " ", "CH3QuickHull"}], ";"}]], "Input",
 CellChangeTimes->{{3.579883578003933*^9, 3.579883584151802*^9}, 
   3.579883786390828*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTTessCHN", " ", "=", " ", "CHNQuickHull"}], ";"}]], "Input",
 CellChangeTimes->{{3.579883585932123*^9, 3.579883593192005*^9}, 
   3.579883789285591*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Returns", " ", "the", " ", "hull", " ", "and", " ", "the", " ", "faces", 
   " ", "in", " ", "the", " ", "hull"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579883447867334*^9, 3.5798834546227427`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTBoundaryCH", "[", 
   RowBox[{"vts_", ",", "ndims_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hull", ",", " ", "hullfaces"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "\[Equal]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"DTBoundaryCH2", "[", "vts", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hullfaces", " ", "=", " ", 
         RowBox[{"Transpose", "[", 
          RowBox[{"{", 
           RowBox[{"hull", ",", 
            RowBox[{"RotateLeft", "[", "hull", "]"}]}], "}"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", "hull", "}"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"More", " ", "than", " ", "2", " ", "dimensions"}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"ndims", "\[Equal]", "3"}], ",", " ", 
           RowBox[{"DTBoundaryCH3", "[", "vts", "]"}], ",", 
           RowBox[{"DTBoundaryCHN", "[", "vts", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hullfaces", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ";"}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"hull", ",", " ", "hullfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579882255424571*^9, 3.579882280373467*^9}, {
  3.579883132317174*^9, 3.5798831346515408`*^9}, {3.579883342821569*^9, 
  3.579883343550078*^9}, {3.579883398792429*^9, 3.5798834012866*^9}, {
  3.579883460943226*^9, 3.57988348748036*^9}, {3.579883607299666*^9, 
  3.579883620073835*^9}, {3.579883889310893*^9, 3.5798839806977367`*^9}, {
  3.579884025613214*^9, 3.5798840298596077`*^9}, {3.579884221238796*^9, 
  3.57988424997294*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphBoundaryCH", "[", 
   RowBox[{"vts_", ",", "ndims_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hull", ",", " ", "hullfaces"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "\[Equal]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"DTSphBoundaryCH2", "[", "vts", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hullfaces", " ", "=", " ", 
         RowBox[{"Transpose", "[", 
          RowBox[{"{", 
           RowBox[{"hull", ",", 
            RowBox[{"RotateLeft", "[", "hull", "]"}]}], "}"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"{", "hull", "}"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"More", " ", "than", " ", "2", " ", "dimensions"}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hull", " ", "=", " ", 
         RowBox[{"DTSphBoundaryCHN", "[", "vts", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"hullfaces", " ", "=", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ";"}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"hull", ",", " ", "hullfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579882255424571*^9, 3.579882280373467*^9}, {
  3.579883132317174*^9, 3.5798831346515408`*^9}, {3.579883342821569*^9, 
  3.579883343550078*^9}, {3.579883398792429*^9, 3.5798834012866*^9}, {
  3.579883460943226*^9, 3.57988348748036*^9}, {3.579883607299666*^9, 
  3.579883620073835*^9}, {3.579883889310893*^9, 3.579884036186145*^9}, {
  3.579907550373028*^9, 3.579907554437479*^9}, {3.579908887376212*^9, 
  3.5799089522485027`*^9}, {3.579909021322381*^9, 3.579909021705752*^9}, {
  3.5819168458678207`*^9, 3.581916889450873*^9}, {3.581917195735209*^9, 
  3.581917201420627*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTTessCH", "[", 
   RowBox[{"vts_", ",", "ndims_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hull", ",", "hullfaces"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ndims", " ", "\[Equal]", "3"}], ",", " ", 
        RowBox[{"DTTessCH3", "[", "vts", "]"}], ",", " ", 
        RowBox[{"DTTessCHN", "[", "vts", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullfaces", " ", "=", " ", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"hull", ",", " ", "hullfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5798840900645123`*^9, 3.579884119462654*^9}, {
  3.579884210616287*^9, 3.579884211751564*^9}, {3.579884262096332*^9, 
  3.5798842762173767`*^9}, {3.5798843323626347`*^9, 3.579884339658428*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTFlipTessCH", "[", 
   RowBox[{"vts_", ",", "ndims_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hull", ",", "hullfaces"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ndims", " ", "\[Equal]", "3"}], ",", " ", 
        RowBox[{"DTTessCH3", "[", "vts", "]"}], ",", " ", 
        RowBox[{"DTTessCHN", "[", "vts", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"hull", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"Sort", "[", 
         RowBox[{"SimplexFlip", " ", "/@", " ", 
          RowBox[{"hull", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hullfaces", " ", "=", " ", 
      RowBox[{"hull", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"hull", ",", " ", "hullfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5798840900645123`*^9, 3.579884119462654*^9}, {
  3.579884210616287*^9, 3.579884211751564*^9}, {3.579884262096332*^9, 
  3.5798842762173767`*^9}, {3.5798843323626347`*^9, 3.579884339658428*^9}, {
  3.581917115194235*^9, 3.58191713625838*^9}, {3.582115374745388*^9, 
  3.582115380023322*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTGenConvexHull", "[", 
   RowBox[{
   "vts_", ",", "extradims_", ",", "BoundaryCH_", ",", "ExtendVts_", ",", 
    "TessCH_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nvts", ",", "ndims", ",", "bdryhull", ",", "bdryhullfaces", ",", 
      "extvts", ",", "tesshull", ",", "tesshullfaces", ",", "dtfaces", ",", 
      "frfaces"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nvts", ",", " ", "ndims"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ndims", " ", "-=", " ", "extradims"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ndims", " ", "<", " ", "2"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"bdryhull", ",", " ", "bdryhullfaces"}], "}"}], " ", "=", " ", 
      
      RowBox[{"BoundaryCH", "[", 
       RowBox[{"vts", ",", "ndims"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"extvts", " ", "=", " ", 
      RowBox[{"ExtendVts", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tesshull", ",", "tesshullfaces"}], "}"}], " ", "=", " ", 
      RowBox[{"TessCH", "[", 
       RowBox[{"extvts", ",", 
        RowBox[{"ndims", "+", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"dtfaces", ",", "frfaces"}], "}"}], " ", "=", " ", 
      RowBox[{"DTSplitHull", "[", 
       RowBox[{"tesshullfaces", ",", "bdryhullfaces"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"bdryhull", ",", "dtfaces", ",", "frfaces"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBmIQ/SBm6UeRNa8cE9bs/ASi/75M4BEF0vu8i8B0+T5N
WxCd0f4aTMdIabqBaL+tgh4gWv376wQQfZtdKBFEn6j2ywLzOeSzQXSqxsQS
EC3y9mQpiDZrvH8KRAvdO3oZRN94Gv4CbF6f9ksQLa2j+hlEG8mmfgXRc861
+ImB7C/9EQyiNdl940F0T/u5JBC9Zx+zuTjInS+KwHSNkUGDLJB22POhE0Rn
iB/YHbL+leMaYeU9IPqjtd8xEN1kY3gcRH+wrdr/DUi7HP0Mpt0beY+DaDPH
2WDaQCRhXfiGV45Tfi4B0wCqO5qS
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTConvexHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DTGenConvexHull", "[", 
   RowBox[{
   "vts", ",", "0", ",", "DTBoundaryCH", ",", "DTParabExtend", ",", " ", 
    "DTTessCH"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579881981212759*^9, 3.579881998792632*^9}, {
  3.579882290958774*^9, 3.5798822999470778`*^9}, {3.579882422668997*^9, 
  3.579882423012821*^9}, {3.579882683263753*^9, 3.5798826931783743`*^9}, {
  3.579882724519802*^9, 3.579882725736064*^9}, {3.579882781784687*^9, 
  3.579882833769723*^9}, {3.579883347990575*^9, 3.579883353119163*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphConvexHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"DTGenConvexHull", "[", 
   RowBox[{
   "vts", ",", "1", ",", "DTSphBoundaryCH", ",", "Identity", ",", " ", 
    "DTFlipTessCH"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.579882000967956*^9, 3.579882020588151*^9}, {
   3.579882306142108*^9, 3.579882309083415*^9}, {3.579882425565309*^9, 
   3.57988242616448*^9}, {3.579882740085596*^9, 3.57988275736854*^9}, {
   3.5798827904191504`*^9, 3.57988279509697*^9}, {3.5798833574627647`*^9, 
   3.579883362247061*^9}, 3.581917178913219*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Display", "Section",
 CellChangeTimes->{{3.575709158029788*^9, 3.575709159036175*^9}, {
  3.576156147086072*^9, 3.576156161084736*^9}}],

Cell[CellGroupData[{

Cell["Shared", "Subsection",
 CellChangeTimes->{{3.576156182278923*^9, 3.576156183557104*^9}, {
  3.576159908277432*^9, 3.576159909747416*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Generation", " ", "of", " ", "data", " ", "to", " ", 
   RowBox[{"process", ":", " ", 
    RowBox[{
    "n", " ", "points", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     "Gaussian", " ", "distribution", " ", "in", " ", 
     RowBox[{"(", "dim", ")"}], " ", "dimensions"}]}]}], " ", "*)"}]], "Input",\

 CellChangeTimes->{{3.5761563534898443`*^9, 3.57615637529698*^9}, {
  3.576156848058879*^9, 3.576156861446116*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakePoints", "[", 
   RowBox[{"n_", ",", "dim_"}], "]"}], " ", ":=", " ", 
  RowBox[{"RandomReal", "[", 
   RowBox[{
    RowBox[{"NormalDistribution", "[", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575709307685255*^9, 3.575709359599738*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Need", " ", "to", " ", "do", " ", "this", " ", "since", " ", "vector"}], 
   " ", "+", " ", 
   RowBox[{"matrix", " ", 
    RowBox[{"won", "'"}], "t", " ", "add", " ", "properly"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.576157099397078*^9, 3.576157125328499*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DoOffset", "[", 
   RowBox[{"vts_", ",", "ofst_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"ofst", "+", "#"}], ")"}], "&"}], " ", "/@", " ", 
   "vts"}]}]], "Input",
 CellChangeTimes->{{3.576157131629532*^9, 3.576157149049096*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Adjust", " ", "their", " ", "distance", " ", "from", " ", "the", " ", 
    "origin", " ", "by", " ", "a", " ", "power", " ", 
    RowBox[{"law", ":", " ", "d"}]}], " ", "\[Rule]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "^", 
      RowBox[{
       RowBox[{"(", "pwr", ")"}], " ", "--"}]}], " ", "pwr"}], " ", "<", " ", 
    
    RowBox[{
    "1", " ", "makes", " ", "a", " ", "shell", " ", "and", " ", "pwr"}], " ", 
    ">", " ", 
    RowBox[{"1", " ", "makes", " ", "a", " ", "core"}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.576156383464576*^9, 3.57615642045056*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PowerAdjust", "[", 
   RowBox[{"vts_", ",", "pwr_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vtnorms", ",", "vtmax"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vtnorms", " ", "=", " ", 
      RowBox[{"Norm", " ", "/@", " ", "vts"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vtmax", " ", "=", " ", 
      RowBox[{"Max", "[", "vtnorms", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vts", "*", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"vtnorms", "/", "vtmax"}], ")"}], "^", 
       RowBox[{"(", 
        RowBox[{"pwr", "-", "1"}], ")"}]}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.576144609605637*^9, 3.5761446976612597`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"For", " ", "graphics"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.576156433279159*^9, 3.5761564352972*^9}, {
  3.576156842581966*^9, 3.576156845485334*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"LabeledPoints", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Text", "[", 
     RowBox[{
      RowBox[{"ToString", "[", "k", "]"}], ",", 
      RowBox[{"vts", "[", 
       RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"Length", "[", "vts", "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.575787968928523*^9, 3.575788018037524*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"HollowPolygon", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Line", "[", 
   RowBox[{"MakeLoop", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.576159498702217*^9, 3.576159507431415*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "SphericalRegion", " ", "is", " ", "True", " ", "so", " ", "that", " ", 
   "one", " ", "can", " ", "easily", " ", "rotate", " ", "it"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.576159337215766*^9, 3.5761593650320797`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"show3d", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"x", ",", 
    RowBox[{"SphericalRegion", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5761592491623697`*^9, 3.576159260473995*^9}, {
  3.576160216483481*^9, 3.5761602241061077`*^9}, {3.576862547146288*^9, 
  3.576862549175387*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2D", "Subsection",
 CellChangeTimes->{{3.57615616925488*^9, 3.576156169653016*^9}}],

Cell[CellGroupData[{

Cell["Data", "Subsubsection",
 CellChangeTimes->{{3.5761567867819977`*^9, 3.576156787764155*^9}, {
  3.5761599116917343`*^9, 3.576159912252182*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp2", " ", "=", " ", 
   RowBox[{"PowerAdjust", "[", 
    RowBox[{
     RowBox[{"MakePoints", "[", 
      RowBox[{"300", ",", "2"}], "]"}], ",", "1"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576338361643358*^9, 3.576338361918153*^9}, {
   3.5763384714927893`*^9, 3.576338471698634*^9}, 3.5763386668081303`*^9, 
   3.576403627412294*^9, 3.577875705787766*^9, {3.579598276940899*^9, 
   3.5795982777031097`*^9}, 3.579610644416547*^9, {3.579955292012211*^9, 
   3.5799552921747913`*^9}, {3.581236752283203*^9, 3.581236752641964*^9}, {
   3.5812367880203657`*^9, 3.581236788275729*^9}, {3.581236854216243*^9, 
   3.581236854593597*^9}, {3.581236893335787*^9, 3.581236893683935*^9}, {
   3.5825543664778557`*^9, 3.5825543680771112`*^9}, 3.582554405440098*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp2x", " ", "=", " ", 
   RowBox[{"Take", "[", 
    RowBox[{"tp2", ",", "30"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581236960859357*^9, 3.5812369673958197`*^9}, {
   3.5825149960214233`*^9, 3.5825150167713327`*^9}, 3.5825150648381557`*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Graphics", "[", 
   RowBox[{"LabeledPoints", "[", "tp2", "]"}], "]"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.575787858142146*^9, 3.575787861436318*^9}, 
   3.575788024182509*^9, {3.5761489363393307`*^9, 3.57614894081273*^9}, {
   3.576156601611534*^9, 3.576156609699395*^9}, 3.5761588740922813`*^9, {
   3.581236822038559*^9, 3.581236825407803*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Subsubsection",
 CellChangeTimes->{{3.575709860292879*^9, 3.5757098607150784`*^9}, {
   3.575717369582897*^9, 3.5757173726436157`*^9}, 3.576156152967698*^9, 
   3.576158874091501*^9}],

Cell[BoxData[
 RowBox[{"ResTimingCount", "[", 
  RowBox[{"CH2Algorithms", ",", "tp2"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577864166395*^9, 3.581577869507538*^9}}],

Cell[BoxData[
 RowBox[{"SmallResCounts", "[", 
  RowBox[{"CH2Algorithms", ",", "20", ",", "tp2"}], "]"}]], "Input",
 CellChangeTimes->{{3.5815771740779552`*^9, 3.581577185044374*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2Display", "[", 
   RowBox[{"vts_", ",", "CH_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "hull", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Graphics", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{"vts", ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"HollowPolygon", "[", "hull", "]"}], ",", "Black", ",", 
          RowBox[{"Point", "[", "vts", "]"}]}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5757172983676577`*^9, 3.575717336624107*^9}, {
   3.575717415688117*^9, 3.575717475300501*^9}, 3.576158874107559*^9, {
   3.5761594938780403`*^9, 3.576159519475691*^9}, {3.576159551212329*^9, 
   3.576159551635791*^9}}],

Cell[BoxData[
 RowBox[{"CH2Display", "[", 
  RowBox[{"tp2", ",", "CH2GrahamScan"}], "]"}]], "Input",
 CellChangeTimes->{{3.575717534290081*^9, 3.575717544512763*^9}, 
   3.575723363003695*^9, 3.575982076294798*^9, 3.575988962172961*^9, 
   3.576066133044301*^9, 3.5761588741084414`*^9, {3.576415571530641*^9, 
   3.5764156071139517`*^9}, {3.5764156796558104`*^9, 3.576415727774911*^9}, {
   3.5764157771594067`*^9, 3.576415779109041*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Delaunay Triangulation", "Subsubsection",
 CellChangeTimes->{{3.579598258514204*^9, 3.579598263055517*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2Algorithms", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "DT2DivideConquer", ",", "DT2IncrFlip", ",", "DT2IncrFlipLin", ",", 
     "DT2IncrFlipRad", ",", "DTConvexHull"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.582083599291912*^9, 3.5820836791956367`*^9}, 
   3.582083824803515*^9, {3.582083862799141*^9, 3.582083885815756*^9}, {
   3.582083959712982*^9, 3.582083964032537*^9}, {3.5820881347901487`*^9, 
   3.582088139923765*^9}, {3.5820888024511633`*^9, 3.582088811496951*^9}, {
   3.5825132280270452`*^9, 3.5825132289787083`*^9}}],

Cell[BoxData[
 RowBox[{"DTResTimingCount", "[", 
  RowBox[{"DT2Algorithms", ",", "tp2x"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577864166395*^9, 3.581577869507538*^9}, {
  3.5820837156531887`*^9, 3.582083721037146*^9}, {3.582088862611155*^9, 
  3.582088862872994*^9}}],

Cell[BoxData[
 RowBox[{"DTSmallResCounts", "[", 
  RowBox[{"DT2Algorithms", ",", "20", ",", "tp2x"}], "]"}]], "Input",
 CellChangeTimes->{{3.5815771740779552`*^9, 3.581577185044374*^9}, 
   3.582083717219101*^9, 3.5820841852592783`*^9, {3.582088882971323*^9, 
   3.582088883297097*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2Display", "[", 
   RowBox[{"vts_", ",", "DT_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "dtres", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dtres", " ", "=", " ", 
      RowBox[{"DT", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "dtres", "]"}], " ", "\[GreaterEqual]", " ", 
        "4"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"dtres", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{"vts", ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"HollowPolygon", " ", "/@", " ", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ",", "Blue", ",", 
          RowBox[{"HollowPolygon", "[", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1"}], "]"}], "]"}], "]"}], ",", "Black", ",", 
          
          RowBox[{"Point", "[", "vts", "]"}]}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579605908394616*^9, 3.5796059264517508`*^9}, {
  3.5796062871384497`*^9, 3.5796063234325037`*^9}, {3.5796072235167713`*^9, 
  3.579607281614471*^9}, {3.579607373680635*^9, 3.5796074052948217`*^9}, {
  3.5796391450288877`*^9, 3.579639187292923*^9}, {3.579834251650813*^9, 
  3.5798342531599283`*^9}}],

Cell[BoxData[
 RowBox[{"DT2Display", "[", 
  RowBox[{"tp2x", ",", "DT2IncrFlipRad"}], "]"}]], "Input",
 CellChangeTimes->{{3.579607283884561*^9, 3.579607298564485*^9}, {
   3.5796084972069674`*^9, 3.579608507423292*^9}, {3.579610616736229*^9, 
   3.579610621797922*^9}, {3.579610686336588*^9, 3.579610686638493*^9}, {
   3.579611939094083*^9, 3.579611949508024*^9}, 3.579612754904625*^9, {
   3.579612789063198*^9, 3.579612799501573*^9}, {3.579630651487812*^9, 
   3.57963065631961*^9}, {3.579634788541349*^9, 3.579634793170336*^9}, {
   3.5796365629414053`*^9, 3.579636563373106*^9}, {3.579636742217145*^9, 
   3.579636742543785*^9}, {3.579636969178905*^9, 3.579636969864539*^9}, {
   3.57963920325642*^9, 3.579639206216099*^9}, {3.579899185441368*^9, 
   3.579899211974063*^9}, 3.581237011545148*^9, {3.582554726673233*^9, 
   3.5825547273687773`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"From", " ", 
    RowBox[{"http", ":"}]}], "//", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"en", ".", "wikipedia", ".", "org"}], "/", "wiki"}], "/", 
     "Delaunay_tessellation"}], "_field", "_estimator"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.579901211438795*^9, 3.579901214636695*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTFE", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "hull", ",", "dltris", ",", "frtris", ",", "flips", ",", "tri", ",", 
      "tria", ",", "vtta", ",", "vt", ",", "vttamin", ",", "vtclrs"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"hull", ",", "dltris", ",", "frtris", ",", "flips"}], "}"}], 
      " ", "=", " ", 
      RowBox[{"DT2IncrFlipRad", "[", "vts", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tria", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"TriArea2D", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "tri", "]"}], "]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"tri", ",", "dltris"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vtta", " ", "=", " ", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"Length", "[", "vts", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"vtta", "[", 
           RowBox[{"[", "vt", "]"}], "]"}], " ", "+=", " ", 
          RowBox[{"tria", "[", 
           RowBox[{"[", "k", "]"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"vt", ",", 
           RowBox[{"dltris", "[", 
            RowBox[{"[", "k", "]"}], "]"}]}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "dltris", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vttamin", " ", "=", " ", 
      RowBox[{"Min", "[", "vtta", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vtclrs", " ", "=", " ", 
      RowBox[{"GrayLevel", " ", "/@", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"vttamin", "/", "vtta"}], ")"}], "^", "0.3"}], ")"}]}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Graphics", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{"vts", ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Polygon", "[", 
           RowBox[{"tri", ",", 
            RowBox[{"VertexColors", "\[Rule]", 
             RowBox[{"vtclrs", "[", 
              RowBox[{"[", "tri", "]"}], "]"}]}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tri", ",", "dltris"}], "}"}]}], "]"}]}], "]"}], "]"}]}]}],
    "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5799012164568167`*^9, 3.5799012266503973`*^9}, {
  3.579901267091992*^9, 3.579901343703841*^9}, {3.579901386716704*^9, 
  3.5799014221670103`*^9}, {3.579901455792045*^9, 3.579901482535673*^9}, {
  3.579901517092559*^9, 3.579901572579946*^9}, {3.57990163429737*^9, 
  3.579901648768651*^9}, {3.579901714369678*^9, 3.579901746813633*^9}, {
  3.579901849820834*^9, 3.579901909078424*^9}, {3.579901960926055*^9, 
  3.5799021120429583`*^9}, {3.579902247087102*^9, 3.579902289171801*^9}, {
  3.5821282704877853`*^9, 3.582128271086149*^9}}],

Cell[BoxData[
 RowBox[{"DTFE", "[", "tp2x", "]"}]], "Input",
 CellChangeTimes->{{3.579901576238845*^9, 3.579901581893211*^9}, {
   3.579901622589942*^9, 3.579901625165547*^9}, {3.5799021657893744`*^9, 
   3.579902182580666*^9}, 3.581237025863208*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Needs", "[", "\"\<ComputationalGeometry`\>\"", "]"}], ";", " ", 
   RowBox[{"PlanarGraphPlot", "[", "tp2", "]"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.579639287572351*^9, 3.5796393607704372`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3D", "Subsection",
 CellChangeTimes->{{3.576861962860302*^9, 3.5768619631673193`*^9}}],

Cell[CellGroupData[{

Cell["Data", "Subsubsection",
 CellChangeTimes->{{3.576861969265642*^9, 3.576861969743217*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp3", " ", "=", " ", 
   RowBox[{"PowerAdjust", "[", 
    RowBox[{
     RowBox[{"MakePoints", "[", 
      RowBox[{"30", ",", "3"}], "]"}], ",", "0.3"}], "]"}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576338361643358*^9, 3.576338361918153*^9}, {
   3.5763384714927893`*^9, 3.576338471698634*^9}, 3.5763386668081303`*^9, 
   3.576403627412294*^9, {3.576862027662146*^9, 3.576862033337348*^9}, {
   3.5769827590688753`*^9, 3.576982767519785*^9}, 3.5770548599433823`*^9, {
   3.5770549743932858`*^9, 3.577054975071498*^9}, 3.5770565785352783`*^9, {
   3.577116852313116*^9, 3.577116852633822*^9}, {3.57711912825035*^9, 
   3.577119128728258*^9}, {3.577145116840229*^9, 3.577145172699586*^9}, {
   3.577145206622233*^9, 3.577145207283553*^9}, {3.579954913396944*^9, 
   3.57995491369061*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Subsubsection",
 CellChangeTimes->{{3.5768619754955063`*^9, 3.5768619774961567`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3NAlgorithms", " ", "=", " ", 
   RowBox[{"Join", "[", 
    RowBox[{"CH3Algorithms", ",", "CHNAlgorithms"}], "]"}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.581577296694406*^9, 3.581577311893735*^9}}],

Cell[BoxData[
 RowBox[{"ResTimingCount", "[", 
  RowBox[{"CH3NAlgorithms", ",", "tp3"}], "]"}]], "Input",
 CellChangeTimes->{{3.58157828508276*^9, 3.5815782881765223`*^9}}],

Cell[BoxData[
 RowBox[{"SmallResCounts", "[", 
  RowBox[{"CH3NAlgorithms", ",", "10", ",", "tp3"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577233318672*^9, 3.5815772420629187`*^9}, 
   3.581577326250668*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3Display", "[", 
   RowBox[{"vts_", ",", "CH_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "hull", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"show3d", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{"vts", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Point", "[", 
           RowBox[{"Range", "[", 
            RowBox[{"Length", "[", "vts", "]"}], "]"}], "]"}], ",", 
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Polygon", "[", 
           RowBox[{"hull", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5757172983676577`*^9, 3.575717336624107*^9}, {
   3.575717415688117*^9, 3.575717475300501*^9}, 3.576158874107559*^9, {
   3.5761594938780403`*^9, 3.576159519475691*^9}, {3.576159551212329*^9, 
   3.576159551635791*^9}, {3.576983335946645*^9, 3.576983336094603*^9}, {
   3.576983753561534*^9, 3.576983764851574*^9}, {3.577054746898119*^9, 
   3.577054760911982*^9}, {3.577054895682948*^9, 3.5770549115772247`*^9}, {
   3.5770550795994*^9, 3.577055079802231*^9}, {3.577137777293982*^9, 
   3.577137777547448*^9}, {3.577137930743063*^9, 3.5771379656941338`*^9}, {
   3.577141921872857*^9, 3.5771419345048733`*^9}, {3.5771424630527887`*^9, 
   3.577142493932459*^9}, {3.577142524750989*^9, 3.577142531573049*^9}, 
   3.57714264740851*^9, {3.577171659839892*^9, 3.577171685263989*^9}, {
   3.577194895801746*^9, 3.57719489605536*^9}, {3.5772066202667217`*^9, 
   3.577206620447434*^9}, {3.577206668328171*^9, 3.577206675899835*^9}, {
   3.577206726025701*^9, 3.577206726612623*^9}, {3.577213178857772*^9, 
   3.5772131960952883`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CH3LocCheck", "[", 
    RowBox[{"tp3", ",", 
     RowBox[{
      RowBox[{"CH3GiftWrapping", "[", "tp3", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], "]"}], " ", "//", " ", "Flatten"}], 
  " ", "//", " ", "Union"}]], "Input",
 CellChangeTimes->{{3.577143075367487*^9, 3.5771430823402033`*^9}, {
  3.577143127311151*^9, 3.577143128520952*^9}, {3.577145098306827*^9, 
  3.5771451019662933`*^9}, {3.577275545089271*^9, 3.577275559217908*^9}, {
  3.577275602912006*^9, 3.57727561836981*^9}, {3.5772756779304733`*^9, 
  3.577275706747252*^9}, {3.57727575991418*^9, 3.577275762755806*^9}, {
  3.5777998617559032`*^9, 3.577799899729084*^9}, {3.577815557104301*^9, 
  3.577815562439878*^9}}],

Cell[BoxData[
 RowBox[{"CH3Display", "[", 
  RowBox[{"tp3", ",", "CH3GiftWrapping"}], "]"}]], "Input",
 CellChangeTimes->{{3.575717534290081*^9, 3.575717544512763*^9}, 
   3.575723363003695*^9, 3.575982076294798*^9, 3.575988962172961*^9, 
   3.576066133044301*^9, 3.5761588741084414`*^9, {3.576415571530641*^9, 
   3.5764156071139517`*^9}, {3.5764156796558104`*^9, 3.576415727774911*^9}, {
   3.5764157771594067`*^9, 3.576415779109041*^9}, {3.576983787802824*^9, 
   3.5769837941698914`*^9}, {3.5770547276450653`*^9, 
   3.5770547289639053`*^9}, {3.577136478908942*^9, 3.577136482945867*^9}, {
   3.5771716131044273`*^9, 3.57717161861519*^9}, {3.57720655674531*^9, 
   3.5772065571522627`*^9}, {3.5772067578206244`*^9, 3.577206793650483*^9}, {
   3.577206859716997*^9, 3.577206871163455*^9}, {3.577213205703899*^9, 
   3.577213219933374*^9}, {3.579022642350668*^9, 3.579022642931418*^9}, {
   3.579892404769425*^9, 3.579892420853634*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Delaunay Tessellation", "Subsubsection",
 CellChangeTimes->{{3.579955380776334*^9, 3.579955384647108*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT3Display", "[", 
   RowBox[{"vts_", ",", "DT_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "dtres", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dtres", " ", "=", " ", 
      RowBox[{"DT", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "dtres", "]"}], " ", "\[GreaterEqual]", " ", 
        "4"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"dtres", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"show3d", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{"vts", ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"HollowPolygon", "  ", "/@", " ", 
              RowBox[{"SimplexSubs", "[", "#", "]"}]}], ")"}], "&"}], "/@", 
           " ", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ",", "Blue", ",", 
          RowBox[{"HollowPolygon", " ", "/@", " ", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "2"}], "]"}], "]"}]}], ",", "Black", ",", 
          RowBox[{"Point", "[", "vts", "]"}]}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579605908394616*^9, 3.5796059264517508`*^9}, {
  3.5796062871384497`*^9, 3.5796063234325037`*^9}, {3.5796072235167713`*^9, 
  3.579607281614471*^9}, {3.579607373680635*^9, 3.5796074052948217`*^9}, {
  3.5796391450288877`*^9, 3.579639187292923*^9}, {3.579834251650813*^9, 
  3.5798342531599283`*^9}, {3.57995544797571*^9, 3.579955558551714*^9}, {
  3.579955660972047*^9, 3.579955662352312*^9}, {3.579955712636674*^9, 
  3.579955724721059*^9}, {3.581584050466673*^9, 3.581584069301388*^9}}],

Cell[BoxData[
 RowBox[{"DT3Display", "[", 
  RowBox[{"tp3", ",", "DTConvexHull"}], "]"}]], "Input",
 CellChangeTimes->{{3.579955550510859*^9, 3.579955582806818*^9}, {
  3.579955647820961*^9, 3.5799556784913263`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4D", "Subsection",
 CellChangeTimes->{{3.58139768410876*^9, 3.5813976846641273`*^9}}],

Cell[CellGroupData[{

Cell["Data", "Subsubsection",
 CellChangeTimes->{{3.576861969265642*^9, 3.576861969743217*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp4", " ", "=", " ", 
   RowBox[{"PowerAdjust", "[", 
    RowBox[{
     RowBox[{"MakePoints", "[", 
      RowBox[{"30", ",", "4"}], "]"}], ",", "0.3"}], "]"}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576338361643358*^9, 3.576338361918153*^9}, {
   3.5763384714927893`*^9, 3.576338471698634*^9}, 3.5763386668081303`*^9, 
   3.576403627412294*^9, {3.576862027662146*^9, 3.576862033337348*^9}, {
   3.5769827590688753`*^9, 3.576982767519785*^9}, 3.5770548599433823`*^9, {
   3.5770549743932858`*^9, 3.577054975071498*^9}, 3.5770565785352783`*^9, {
   3.577116852313116*^9, 3.577116852633822*^9}, {3.57711912825035*^9, 
   3.577119128728258*^9}, {3.577145116840229*^9, 3.577145172699586*^9}, {
   3.577145206622233*^9, 3.577145207283553*^9}, {3.579954913396944*^9, 
   3.57995491369061*^9}, {3.5813977038549833`*^9, 3.581397707363308*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Subsubsection",
 CellChangeTimes->{{3.5768619754955063`*^9, 3.5768619774961567`*^9}}],

Cell[BoxData[
 RowBox[{"ResTimingCount", "[", 
  RowBox[{"CHNAlgorithms", ",", "tp4"}], "]"}]], "Input",
 CellChangeTimes->{{3.581578383947483*^9, 3.5815783864891663`*^9}}],

Cell[BoxData[
 RowBox[{"SmallResCounts", "[", 
  RowBox[{"CHNAlgorithms", ",", "10", ",", "tp4"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577233318672*^9, 3.581577281470837*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2D Spherical", "Subsection",
 CellChangeTimes->{{3.576157183610499*^9, 3.576157184767345*^9}, {
  3.581582714431115*^9, 3.5815827152547283`*^9}}],

Cell[CellGroupData[{

Cell["Shared", "Subsubsection",
 CellChangeTimes->{{3.576157192477216*^9, 3.57615719322399*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"A", " ", "global", " ", 
   RowBox[{"variable", " ", "--"}], " ", "should", " ", "not", " ", "expect", 
   " ", "to", " ", "change", " ", "this", " ", "very", " ", "much"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.576159272580755*^9, 3.576159284296278*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SphereDisplaySubdiv", " ", "=", " ", "16"}], ";"}], 
  " "}]], "Input",
 CellChangeTimes->{{3.576159201589819*^9, 3.5761592206482267`*^9}, 
   3.576159270560989*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Draw", " ", "arcs", " ", "and", " ", "strings", " ", "of", " ", "arcs", 
   " ", 
   RowBox[{"(", 
    RowBox[{"polylines", " ", "and", " ", "polygons"}], ")"}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.576159407620082*^9, 3.576159422704516*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SphereArc", "[", 
   RowBox[{"v1_", ",", "v2_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "ns", ",", "smlt", ",", "k", ",", "f"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"a", " ", "=", " ", 
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"v1", ".", "v2"}], ")"}], ",", 
        RowBox[{"Norm", "[", 
         RowBox[{"Cross", "[", 
          RowBox[{"v1", ",", "v2"}], "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"smlt", " ", "=", " ", 
      RowBox[{"1", "/", 
       RowBox[{"Sin", "[", "a", "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ns", "=", " ", 
      RowBox[{"Ceiling", "[", 
       RowBox[{"SphereDisplaySubdiv", "*", 
        RowBox[{"a", "/", 
         RowBox[{"(", 
          RowBox[{"Pi", "/", "2"}], ")"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"f", " ", "=", " ", 
          RowBox[{"k", "/", "ns"}]}], ";", 
         RowBox[{"smlt", "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"v1", "*", 
             RowBox[{"Sin", "[", 
              RowBox[{"a", "*", 
               RowBox[{"(", 
                RowBox[{"1", "-", "f"}], ")"}]}], "]"}]}], "+", 
            RowBox[{"v2", "*", 
             RowBox[{"Sin", "[", 
              RowBox[{"a", "*", "f"}], "]"}]}]}], ")"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "0", ",", "ns"}], "}"}]}], "]"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.480460707910082*^9, 3.480460738718381*^9}, {
  3.4804608140220327`*^9, 3.480460862681045*^9}, {3.480460932410253*^9, 
  3.4804609382788982`*^9}, {3.480461008042885*^9, 3.480461037986713*^9}, {
  3.4804611496583233`*^9, 3.480461154380534*^9}, {3.572144395607492*^9, 
  3.572144401240058*^9}, {3.572353026885408*^9, 3.5723530274367*^9}, {
  3.572353060445355*^9, 3.572353085788224*^9}, {3.57235318204062*^9, 
  3.572353281782219*^9}, {3.572353457314473*^9, 3.572353482049366*^9}, {
  3.572354275990099*^9, 3.572354280644128*^9}, {3.5723555883361177`*^9, 
  3.5723556059654713`*^9}, {3.572356375780257*^9, 3.57235638964835*^9}, {
  3.576159436624126*^9, 3.5761594681221657`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpherePolyline", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "vts", "]"}], "\[GreaterEqual]", " ", "2"}], ",", 
    " ", 
    RowBox[{
     RowBox[{
      RowBox[{"SphereArc", " ", "@@", " ", "#"}], "&"}], " ", "/@", " ", 
     RowBox[{"Partition", "[", 
      RowBox[{"vts", ",", "2", ",", "1"}], "]"}]}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.576159577486739*^9, 3.576159604130621*^9}, {
  3.5761596372618723`*^9, 3.576159652660356*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpherePolygon", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"SpherePolyline", "[", 
   RowBox[{"MakeLoop", "[", "vts", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5761596552023497`*^9, 3.576159666411063*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"showsph", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"x", ",", 
    RowBox[{"SphericalRegion", "\[Rule]", "True"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"ConstantArray", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "3"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5761592491623697`*^9, 3.576159260473995*^9}, {
  3.576160216483481*^9, 3.5761602241061077`*^9}, {3.576862528424678*^9, 
  3.576862559981793*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Data", "Subsubsection",
 CellChangeTimes->{{3.576159935125493*^9, 3.57615993560427*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp2sph0", " ", "=", " ", 
   RowBox[{"Normalize", " ", "/@", " ", 
    RowBox[{"MakePoints", "[", 
     RowBox[{"30", ",", "3"}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576159944566291*^9, 3.576160004503516*^9}, 
   3.576160236260688*^9, 3.576160313324945*^9, 3.57616034863727*^9, {
   3.5761612655723867`*^9, 3.5761612803891973`*^9}, {3.579954865245064*^9, 
   3.579954865346662*^9}, {3.5815827464036407`*^9, 3.581582747833502*^9}, {
   3.5815828383964577`*^9, 3.581582848314096*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp2sph", " ", "=", " ", 
   RowBox[{"Normalize", " ", "/@", " ", 
    RowBox[{"DoOffset", "[", 
     RowBox[{
      RowBox[{"MakePoints", "[", 
       RowBox[{"30", ",", "3"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "4"}], "}"}]}], "]"}]}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576159944566291*^9, 3.576160004503516*^9}, 
   3.576160236260688*^9, 3.576160313324945*^9, 3.57616034863727*^9, {
   3.5761612655723867`*^9, 3.5761612803891973`*^9}, {3.579954865245064*^9, 
   3.579954865346662*^9}, {3.5815827464036407`*^9, 3.581582747833502*^9}, {
   3.581582862268944*^9, 3.5815828638907127`*^9}, {3.581663932088255*^9, 
   3.581663934387974*^9}, {3.582115022460333*^9, 3.582115023117091*^9}, {
   3.582115055293283*^9, 3.582115055546886*^9}}],

Cell[BoxData[
 RowBox[{"show3d", "[", 
  RowBox[{"LabeledPoints", "[", "tp2sph", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.575787858142146*^9, 3.575787861436318*^9}, 
   3.575788024182509*^9, {3.5761489363393307`*^9, 3.57614894081273*^9}, {
   3.576156601611534*^9, 3.576156609699395*^9}, 3.5761588740922813`*^9, {
   3.576160015674193*^9, 3.576160020022545*^9}, 3.581582752436376*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Subsubsection",
 CellChangeTimes->{{3.576157200209798*^9, 3.576157202039459*^9}}],

Cell[BoxData[
 RowBox[{"ResTimingCount", "[", 
  RowBox[{"CH2SphAlgorithms", ",", "tp2sph"}], "]"}]], "Input",
 CellChangeTimes->{{3.581578459996334*^9, 3.581578463098987*^9}, {
  3.581582761499969*^9, 3.581582783945053*^9}}],

Cell[BoxData[
 RowBox[{"SmallResCounts", "[", 
  RowBox[{"CH2SphAlgorithms", ",", "10", ",", "tp2sph"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577233318672*^9, 3.581577281470837*^9}, {
   3.581577388082443*^9, 3.581577407993726*^9}, 3.581577468194708*^9, {
   3.581582767825778*^9, 3.581582787019264*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH2SphDisplay", "[", 
   RowBox[{"vts_", ",", "CH_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "hull", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"showsph", "[", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"SpherePolygon", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", "hull", "]"}], "]"}], "]"}], ",", "Black", ",", 
        RowBox[{"Point", "[", "vts", "]"}]}], "}"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5757172983676577`*^9, 3.575717336624107*^9}, {
   3.575717415688117*^9, 3.575717475300501*^9}, 3.576158874107559*^9, {
   3.5761594938780403`*^9, 3.576159519475691*^9}, {3.576159551212329*^9, 
   3.576159551635791*^9}, {3.576159833393524*^9, 3.576159844694521*^9}, {
   3.576160272871484*^9, 3.576160283766573*^9}, {3.576862575062087*^9, 
   3.576862576027919*^9}, 3.581582770036899*^9}],

Cell[BoxData[
 RowBox[{"CH2SphDisplay", "[", 
  RowBox[{"tp2sph", ",", "CH2SphIncremental"}], "]"}]], "Input",
 CellChangeTimes->{{3.575717534290081*^9, 3.575717544512763*^9}, 
   3.575723363003695*^9, 3.575982076294798*^9, 3.575988962172961*^9, 
   3.576066133044301*^9, 3.5761588741084414`*^9, {3.5761598127823353`*^9, 
   3.576159826339902*^9}, {3.5761600545361843`*^9, 3.576160055646586*^9}, {
   3.5815827723849697`*^9, 3.581582798492082*^9}, 3.5816639772859297`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Delaunay Triangulation", "Subsubsection",
 CellChangeTimes->{{3.579637950371211*^9, 3.5796379582893467`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DT2SphAlgorithms", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "DT2SphIncrFlip", ",", "DT2SphIncrFlipLin", ",", " ", "DT2SphIncrFlipRad",
      ",", "DTSphConvexHull"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.582083599291912*^9, 3.5820836791956367`*^9}, 
   3.582083824803515*^9, {3.582083862799141*^9, 3.582083885815756*^9}, {
   3.582083959712982*^9, 3.582083964032537*^9}, {3.582084292257731*^9, 
   3.58208429305011*^9}, {3.5820843274367647`*^9, 3.582084330140123*^9}, {
   3.582114965196063*^9, 3.582114989578547*^9}}],

Cell[BoxData[
 RowBox[{"DTResTimingCount", "[", 
  RowBox[{"DT2SphAlgorithms", ",", "tp2sph0"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577864166395*^9, 3.581577869507538*^9}, {
   3.5820837156531887`*^9, 3.582083721037146*^9}, {3.582084159011468*^9, 
   3.582084174112439*^9}, {3.582114991474173*^9, 3.582114991817675*^9}, 
   3.5821153919183607`*^9}],

Cell[BoxData[
 RowBox[{"DTSmallResCounts", "[", 
  RowBox[{"DT2SphAlgorithms", ",", "20", ",", "tp2sph"}], "]"}]], "Input",
 CellChangeTimes->{{3.5815771740779552`*^9, 3.581577185044374*^9}, 
   3.582083717219101*^9, {3.582084171108334*^9, 3.582084198370859*^9}, {
   3.582115009994384*^9, 3.58211501031374*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphDisplay", "[", 
   RowBox[{"vts_", ",", "DT_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "dtres", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dtres", " ", "=", " ", 
      RowBox[{"DT", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "dtres", "]"}], " ", "\[GreaterEqual]", " ", 
        "4"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"dtres", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"showsph", "[", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"SpherePolygon", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
         RowBox[{"dtres", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", "Blue", ",", 
        RowBox[{"SpherePolygon", "[", 
         RowBox[{"vts", "[", 
          RowBox[{"[", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1"}], "]"}], "]"}], "]"}], "]"}], "]"}], ",", 
        "Black", ",", 
        RowBox[{"Point", "[", "vts", "]"}]}], "}"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579638686792259*^9, 3.5796387360221043`*^9}, {
   3.579638806791815*^9, 3.579638809856439*^9}, 3.579638843240652*^9, {
   3.5796388763599586`*^9, 3.579638927248644*^9}, {3.5796391953012857`*^9, 
   3.579639195613209*^9}, {3.579834268264625*^9, 3.579834269544162*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTSphRawConvexHull", "[", "vts_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "hull", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH3QuickHull", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"showsph", "[", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"HollowPolygon", "[", 
           RowBox[{"vts", "[", 
            RowBox[{"[", "#", "]"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", "Black", ",", 
        RowBox[{"Point", "[", "vts", "]"}]}], "}"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5799079295719423`*^9, 3.579908009258964*^9}, {
  3.579908043681075*^9, 3.5799080443126497`*^9}, {3.579908116618969*^9, 
  3.5799081637337093`*^9}, {3.579908196585259*^9, 3.579908199375896*^9}}],

Cell[BoxData[
 RowBox[{"DTSphDisplay", "[", 
  RowBox[{"tp2sph", ",", "DT2SphIncrFlip"}], "]"}]], "Input",
 CellChangeTimes->{{3.579907661137442*^9, 3.579907664973761*^9}, {
   3.579954503081132*^9, 3.5799545062542343`*^9}, 3.5815841188182096`*^9, {
   3.5815845689921017`*^9, 3.581584582524358*^9}, 3.582085968435417*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3D Spherical", "Subsection",
 CellChangeTimes->{{3.5795030917605877`*^9, 3.579503093757493*^9}, {
  3.58158271746277*^9, 3.581582720542695*^9}}],

Cell[CellGroupData[{

Cell["Data", "Subsubsection",
 CellChangeTimes->{{3.579503295743339*^9, 3.5795032962931833`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp3sph0", " ", "=", " ", 
   RowBox[{"Normalize", " ", "/@", " ", 
    RowBox[{"MakePoints", "[", 
     RowBox[{"100", ",", "4"}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576159944566291*^9, 3.576160004503516*^9}, 
   3.576160236260688*^9, 3.576160313324945*^9, 3.57616034863727*^9, {
   3.5761612655723867`*^9, 3.5761612803891973`*^9}, {3.579503415670579*^9, 
   3.579503516705687*^9}, {3.5815829037418613`*^9, 3.58158290581853*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "13666", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.575708981150353*^9, 3.5757089851640453`*^9}, {
  3.5761563159553423`*^9, 3.5761563265541153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tp3sph", " ", "=", " ", 
   RowBox[{"Normalize", " ", "/@", " ", 
    RowBox[{"DoOffset", "[", 
     RowBox[{
      RowBox[{"MakePoints", "[", 
       RowBox[{"100", ",", "4"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"4", ",", "0", ",", "0", ",", "0"}], "}"}]}], "]"}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.576156624611845*^9, 3.576156624747431*^9}, 
   3.57615675442301*^9, {3.576159944566291*^9, 3.576160004503516*^9}, 
   3.576160236260688*^9, 3.576160313324945*^9, 3.57616034863727*^9, {
   3.5761612655723867`*^9, 3.5761612803891973`*^9}, {3.579503415670579*^9, 
   3.5795034897620487`*^9}, {3.58158281553258*^9, 3.581582815802868*^9}, {
   3.5815829105498466`*^9, 3.5815829138048973`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Convex Hull", "Subsubsection",
 CellChangeTimes->{{3.579503304039064*^9, 3.579503316066745*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3NSphAlgorithms", " ", "=", " ", 
   RowBox[{"Join", "[", 
    RowBox[{"CH3SphAlgorithms", ",", "CHNSphAlgorihms"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5815785059992857`*^9, 3.581578552726502*^9}, {
  3.58158295088347*^9, 3.581582958771008*^9}}],

Cell[BoxData[
 RowBox[{"ResTimingCount", "[", 
  RowBox[{"CH3NSphAlgorithms", ",", "tp3sph"}], "]"}]], "Input",
 CellChangeTimes->{{3.581578557549777*^9, 3.581578580507704*^9}, {
   3.581582928662126*^9, 3.581582930210183*^9}, 3.581582967301713*^9}],

Cell[BoxData[
 RowBox[{"SmallResCounts", "[", 
  RowBox[{"CH3NSphAlgorithms", ",", "10", ",", "tp3sph"}], "]"}]], "Input",
 CellChangeTimes->{{3.581577233318672*^9, 3.581577281470837*^9}, {
   3.581577388082443*^9, 3.581577407993726*^9}, 3.581577468194708*^9, {
   3.5815776127395887`*^9, 3.581577615472761*^9}, {3.581577645737298*^9, 
   3.5815776476407967`*^9}, {3.581578488309493*^9, 3.581578491706565*^9}, {
   3.5815785624470463`*^9, 3.581578585408684*^9}, {3.581578917170581*^9, 
   3.581578918032454*^9}, 3.581582933301594*^9, 3.581582976669182*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CH3SphDisplay", "[", 
   RowBox[{"vts_", ",", "CH_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "hull", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"hull", " ", "=", " ", 
      RowBox[{"CH", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"show3d", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{
        RowBox[{"Rest", " ", "/@", " ", "vts"}], ",", " ", 
        RowBox[{"Polygon", "[", 
         RowBox[{"hull", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.5799558675896482`*^9, 3.579955956336698*^9}, {
  3.581582941972661*^9, 3.581582943157104*^9}}],

Cell[BoxData[
 RowBox[{"CH3SphDisplay", "[", 
  RowBox[{"tp3sph", ",", "CH3SphIncremental"}], "]"}]], "Input",
 CellChangeTimes->{{3.579955959641947*^9, 3.579955968975896*^9}, 
   3.581578921083164*^9, {3.581582935053179*^9, 3.581582938426811*^9}, {
   3.58158302295093*^9, 3.581583024059987*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Delaunay Tessellation", "Subsubsection",
 CellChangeTimes->{{3.5799560207032824`*^9, 3.579956024364729*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTHpsDisplay", "[", 
   RowBox[{"vts_", ",", "DT_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "dtres", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dtres", " ", "=", " ", 
      RowBox[{"DT", "[", "vts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "dtres", "]"}], " ", "\[GreaterEqual]", " ", 
        "4"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"dtres", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"show3d", "[", 
      RowBox[{"GraphicsComplex", "[", 
       RowBox[{
        RowBox[{"Rest", " ", "/@", " ", "vts"}], ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"HollowPolygon", "  ", "/@", " ", 
              RowBox[{"SimplexSubs", "[", "#", "]"}]}], ")"}], "&"}], "/@", 
           " ", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ",", "Blue", ",", 
          RowBox[{"HollowPolygon", " ", "/@", " ", 
           RowBox[{"dtres", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "2"}], "]"}], "]"}]}], ",", "Black", ",", 
          RowBox[{"Point", "[", 
           RowBox[{"Rest", " ", "/@", " ", "vts"}], "]"}]}], "}"}]}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.579605908394616*^9, 3.5796059264517508`*^9}, {
  3.5796062871384497`*^9, 3.5796063234325037`*^9}, {3.5796072235167713`*^9, 
  3.579607281614471*^9}, {3.579607373680635*^9, 3.5796074052948217`*^9}, {
  3.5796391450288877`*^9, 3.579639187292923*^9}, {3.579834251650813*^9, 
  3.5798342531599283`*^9}, {3.57995544797571*^9, 3.579955558551714*^9}, {
  3.579955660972047*^9, 3.579955662352312*^9}, {3.579955712636674*^9, 
  3.579955724721059*^9}, {3.579956050158009*^9, 3.57995606190775*^9}, {
  3.579956146750188*^9, 3.579956148228747*^9}}],

Cell[BoxData[
 RowBox[{"DTHpsDisplay", "[", 
  RowBox[{
   RowBox[{"Take", "[", 
    RowBox[{"tp3sph", ",", "10"}], "]"}], ",", "DTSphConvexHull"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.579955550510859*^9, 3.579955582806818*^9}, {
  3.579955647820961*^9, 3.5799556784913263`*^9}, {3.579956082729539*^9, 
  3.579956103889612*^9}, {3.579956236371254*^9, 3.579956240344462*^9}, {
  3.581916620821135*^9, 3.581916622952862*^9}}]
}, Closed]]
}, Closed]]
}, Closed]]
},
WindowSize->{640, 696},
WindowMargins->{{Automatic, -1}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (January 30, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 97, 1, 67, "Section"],
Cell[667, 25, 9069, 158, 2668, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9773, 188, 91, 1, 67, "Section"],
Cell[CellGroupData[{
Cell[9889, 193, 95, 1, 34, "Subsection"],
Cell[9987, 196, 290, 6, 27, "Input"],
Cell[10280, 204, 310, 7, 27, "Input"],
Cell[10593, 213, 383, 9, 27, "Input"],
Cell[10979, 224, 371, 8, 43, "Input"],
Cell[11353, 234, 441, 10, 43, "Input"],
Cell[11797, 246, 587, 15, 43, "Input"],
Cell[12387, 263, 588, 15, 43, "Input"],
Cell[12978, 280, 538, 11, 43, "Input"],
Cell[13519, 293, 494, 10, 43, "Input"],
Cell[14016, 305, 446, 10, 43, "Input"],
Cell[14465, 317, 396, 9, 43, "Input"],
Cell[14864, 328, 252, 6, 27, "Input"],
Cell[15119, 336, 334, 9, 27, "Input"],
Cell[15456, 347, 207, 4, 27, "Input"],
Cell[15666, 353, 598, 17, 43, "Input"],
Cell[16267, 372, 281, 6, 27, "Input"],
Cell[16551, 380, 1224, 31, 118, "Input"],
Cell[17778, 413, 171, 3, 27, "Input"],
Cell[17952, 418, 459, 12, 27, "Input"],
Cell[18414, 432, 190, 4, 27, "Input"],
Cell[18607, 438, 746, 20, 73, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[19390, 463, 92, 1, 26, "Subsection"],
Cell[19485, 466, 309, 7, 27, "Input"],
Cell[19797, 475, 494, 15, 27, "Input"],
Cell[20294, 492, 331, 7, 27, "Input"],
Cell[20628, 501, 748, 23, 43, "Input"],
Cell[21379, 526, 917, 23, 88, "Input"],
Cell[22299, 551, 369, 7, 27, "Input"],
Cell[22671, 560, 561, 16, 27, "Input"],
Cell[23235, 578, 257, 5, 27, "Input"],
Cell[23495, 585, 241, 5, 27, "Input"],
Cell[23739, 592, 343, 7, 27, "Input"],
Cell[24085, 601, 885, 24, 43, "Input"],
Cell[24973, 627, 448, 12, 27, "Input"],
Cell[25424, 641, 356, 8, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[25817, 654, 153, 2, 26, "Subsection"],
Cell[25973, 658, 465, 9, 43, "Input"],
Cell[26441, 669, 497, 10, 27, "Input"],
Cell[26941, 681, 256, 5, 27, "Input"],
Cell[27200, 688, 298, 7, 27, "Input"],
Cell[27501, 697, 323, 7, 27, "Input"],
Cell[27827, 706, 471, 11, 27, "Input"],
Cell[28301, 719, 301, 6, 27, "Input"],
Cell[28605, 727, 450, 9, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[29092, 741, 90, 1, 26, "Subsection"],
Cell[29185, 744, 325, 6, 27, "Input"],
Cell[29513, 752, 841, 23, 43, "Input"],
Cell[30357, 777, 256, 5, 27, "Input"],
Cell[30616, 784, 243, 6, 27, "Input"],
Cell[30862, 792, 422, 8, 27, "Input"],
Cell[31287, 802, 902, 22, 43, "Input"],
Cell[32192, 826, 527, 13, 27, "Input"],
Cell[32722, 841, 454, 9, 27, "Input"],
Cell[33179, 852, 268, 6, 27, "Input"],
Cell[33450, 860, 615, 17, 27, "Input"],
Cell[34068, 879, 183, 4, 27, "Input"],
Cell[34254, 885, 342, 7, 27, "Input"],
Cell[34599, 894, 858, 20, 88, "Input"],
Cell[35460, 916, 871, 22, 88, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[36368, 943, 153, 2, 26, "Subsection"],
Cell[36524, 947, 530, 11, 43, "Input"],
Cell[37057, 960, 699, 13, 27, "Input"],
Cell[37759, 975, 315, 6, 27, "Input"],
Cell[38077, 983, 327, 7, 27, "Input"],
Cell[38407, 992, 381, 8, 43, "Input"],
Cell[38791, 1002, 737, 16, 27, "Input"],
Cell[39531, 1020, 253, 6, 27, "Input"],
Cell[39787, 1028, 356, 8, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[40180, 1041, 91, 1, 26, "Subsection"],
Cell[40274, 1044, 349, 8, 43, "Input"],
Cell[40626, 1054, 576, 12, 58, "Input"],
Cell[41205, 1068, 649, 20, 43, "Input"],
Cell[41857, 1090, 363, 8, 43, "Input"],
Cell[42223, 1100, 555, 16, 43, "Input"],
Cell[42781, 1118, 416, 9, 27, "Input"],
Cell[43200, 1129, 664, 19, 43, "Input"],
Cell[43867, 1150, 1156, 26, 88, "Input"],
Cell[45026, 1178, 433, 11, 27, "Input"],
Cell[45462, 1191, 499, 14, 27, "Input"],
Cell[45964, 1207, 349, 7, 27, "Input"],
Cell[46316, 1216, 251, 6, 27, "Input"],
Cell[46570, 1224, 197, 4, 27, "Input"],
Cell[46770, 1230, 523, 12, 27, "Input"],
Cell[47296, 1244, 484, 13, 27, "Input"],
Cell[47783, 1259, 306, 7, 27, "Input"],
Cell[48092, 1268, 138, 2, 27, "Input"],
Cell[48233, 1272, 434, 10, 58, "Input"],
Cell[48670, 1284, 437, 12, 43, "Input"],
Cell[49110, 1298, 394, 10, 43, "Input"],
Cell[49507, 1310, 236, 6, 27, "Input"],
Cell[49746, 1318, 1046, 25, 73, "Input"],
Cell[50795, 1345, 560, 13, 27, "Input"],
Cell[51358, 1360, 456, 10, 27, "Input"],
Cell[51817, 1372, 519, 14, 43, "Input"],
Cell[52339, 1388, 590, 14, 43, "Input"],
Cell[52932, 1404, 265, 6, 27, "Input"],
Cell[53200, 1412, 858, 20, 88, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[54095, 1437, 100, 1, 26, "Subsection"],
Cell[54198, 1440, 375, 7, 27, "Input"],
Cell[54576, 1449, 257, 6, 27, "Input"],
Cell[54836, 1457, 197, 4, 27, "Input"],
Cell[55036, 1463, 457, 9, 27, "Input"],
Cell[55496, 1474, 280, 6, 27, "Input"],
Cell[55779, 1482, 310, 7, 27, "Input"],
Cell[56092, 1491, 138, 2, 27, "Input"],
Cell[56233, 1495, 497, 11, 58, "Input"],
Cell[56733, 1508, 521, 14, 43, "Input"],
Cell[57257, 1524, 511, 11, 43, "Input"],
Cell[57771, 1537, 168, 5, 27, "Input"],
Cell[57942, 1544, 585, 12, 27, "Input"],
Cell[58530, 1558, 447, 8, 27, "Input"],
Cell[58980, 1568, 442, 10, 27, "Input"],
Cell[59425, 1580, 703, 18, 43, "Input"],
Cell[60131, 1600, 643, 14, 43, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[60811, 1619, 117, 1, 26, "Subsection"],
Cell[60931, 1622, 686, 18, 58, "Input"],
Cell[61620, 1642, 375, 10, 27, "Input"],
Cell[61998, 1654, 457, 12, 27, "Input"],
Cell[62458, 1668, 449, 12, 43, "Input"],
Cell[62910, 1682, 606, 17, 58, "Input"],
Cell[63519, 1701, 545, 11, 58, "Input"],
Cell[64067, 1714, 411, 11, 27, "Input"],
Cell[64481, 1727, 599, 16, 43, "Input"],
Cell[65083, 1745, 577, 15, 27, "Input"],
Cell[65663, 1762, 601, 14, 43, "Input"],
Cell[66267, 1778, 659, 18, 58, "Input"],
Cell[66929, 1798, 1012, 19, 103, "Input"],
Cell[67944, 1819, 1625, 41, 133, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[69618, 1866, 124, 2, 37, "Section"],
Cell[CellGroupData[{
Cell[69767, 1872, 358, 5, 34, "Subsection"],
Cell[CellGroupData[{
Cell[70150, 1881, 104, 1, 24, "Subsubsection"],
Cell[70257, 1884, 5313, 131, 448, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[75607, 2020, 130, 2, 18, "Subsubsection"],
Cell[75740, 2024, 314, 8, 43, "Input"],
Cell[76057, 2034, 7309, 151, 448, "Input"],
Cell[83369, 2187, 2193, 39, 73, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[85599, 2231, 111, 1, 18, "Subsubsection"],
Cell[85713, 2234, 346, 7, 43, "Input"],
Cell[86062, 2243, 4037, 87, 448, "Input"],
Cell[90102, 2332, 135, 2, 27, "Input"],
Cell[90240, 2336, 12718, 286, 823, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[102995, 2627, 108, 1, 18, "Subsubsection"],
Cell[103106, 2630, 221, 5, 27, "Input"],
Cell[103330, 2637, 5756, 125, 403, "Input"],
Cell[109089, 2764, 359, 8, 27, "Input"],
Cell[109451, 2774, 579, 12, 58, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[110067, 2791, 102, 1, 18, "Subsubsection"],
Cell[110172, 2794, 337, 7, 43, "Input"],
Cell[110512, 2803, 3162, 86, 178, "Input"],
Cell[113677, 2891, 13833, 302, 988, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[127547, 3198, 102, 1, 18, "Subsubsection"],
Cell[127652, 3201, 12046, 277, 868, "Input"],
Cell[139701, 3480, 331, 7, 43, "Input"],
Cell[140035, 3489, 413, 8, 43, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[140485, 3502, 109, 1, 18, "Subsubsection"],
Cell[140597, 3505, 526, 11, 43, "Input"],
Cell[141126, 3518, 2086, 52, 193, "Input"],
Cell[143215, 3572, 2803, 61, 238, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[146055, 3638, 173, 2, 18, "Subsubsection"],
Cell[146231, 3642, 2004, 48, 223, "Input"],
Cell[148238, 3692, 679, 12, 27, "Input"],
Cell[148920, 3706, 681, 12, 27, "Input"],
Cell[149604, 3720, 615, 12, 43, "Input"],
Cell[150222, 3734, 300, 6, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[150571, 3746, 339, 5, 26, "Subsection"],
Cell[CellGroupData[{
Cell[150935, 3755, 106, 1, 24, "Subsubsection"],
Cell[151044, 3758, 1274, 34, 103, "Input"],
Cell[152321, 3794, 428, 12, 27, "Input"],
Cell[152752, 3808, 508, 11, 27, "Input"],
Cell[153263, 3821, 15523, 341, 1078, "Input"],
Cell[168789, 4164, 891, 21, 73, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[169717, 4190, 111, 1, 18, "Subsubsection"],
Cell[169831, 4193, 3542, 85, 313, "Input"],
Cell[173376, 4280, 8767, 188, 523, "Input"],
Cell[182146, 4470, 135, 2, 27, "Input"],
Cell[182284, 4474, 26836, 575, 1573, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[209157, 5054, 108, 1, 18, "Subsubsection"],
Cell[209268, 5057, 1025, 28, 73, "Input"],
Cell[210296, 5087, 542, 11, 58, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[210875, 5103, 102, 1, 18, "Subsubsection"],
Cell[210980, 5106, 586, 12, 58, "Input"],
Cell[211569, 5120, 7920, 180, 688, "Input"],
Cell[219492, 5302, 337, 7, 43, "Input"],
Cell[219832, 5311, 2529, 68, 163, "Input"],
Cell[222364, 5381, 16288, 352, 1123, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[238689, 5738, 102, 1, 18, "Subsubsection"],
Cell[238794, 5741, 5876, 137, 523, "Input"],
Cell[244673, 5880, 405, 9, 43, "Input"],
Cell[245081, 5891, 605, 11, 58, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[245723, 5907, 118, 1, 18, "Subsubsection"],
Cell[245844, 5910, 1689, 41, 193, "Input"],
Cell[247536, 5953, 737, 12, 43, "Input"],
Cell[248276, 5967, 793, 13, 43, "Input"],
Cell[249072, 5982, 318, 7, 43, "Input"],
Cell[249393, 5991, 251, 5, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[249693, 6002, 163, 2, 26, "Subsection"],
Cell[CellGroupData[{
Cell[249881, 6008, 108, 1, 24, "Subsubsection"],
Cell[249992, 6011, 1437, 33, 133, "Input"],
Cell[251432, 6046, 398, 10, 27, "Input"],
Cell[251833, 6058, 582, 12, 27, "Input"],
Cell[252418, 6072, 18827, 427, 1393, "Input"],
Cell[271248, 6501, 965, 23, 73, "Input"],
Cell[272216, 6526, 566, 12, 43, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[272819, 6543, 105, 1, 18, "Subsubsection"],
Cell[272927, 6546, 1124, 29, 73, "Input"],
Cell[274054, 6577, 640, 13, 58, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[274731, 6595, 102, 1, 18, "Subsubsection"],
Cell[274836, 6598, 597, 11, 58, "Input"],
Cell[275436, 6611, 8125, 181, 673, "Input"],
Cell[283564, 6794, 337, 7, 43, "Input"],
Cell[283904, 6803, 2487, 66, 163, "Input"],
Cell[286394, 6871, 13236, 288, 1048, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[299667, 7164, 101, 1, 18, "Subsubsection"],
Cell[299771, 7167, 7054, 160, 568, "Input"],
Cell[306828, 7329, 566, 11, 58, "Input"],
Cell[307397, 7342, 699, 12, 58, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[308133, 7359, 94, 1, 18, "Subsubsection"],
Cell[308230, 7362, 289, 7, 43, "Input"],
Cell[308522, 7371, 197, 4, 27, "Input"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[308780, 7382, 194, 2, 37, "Section"],
Cell[CellGroupData[{
Cell[308999, 7388, 113, 1, 34, "Subsection"],
Cell[309115, 7391, 137, 2, 27, "Input"],
Cell[309255, 7395, 38419, 824, 2758, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[347711, 8224, 177, 2, 26, "Subsection"],
Cell[347891, 8228, 194, 4, 27, "Input"],
Cell[348088, 8234, 2046, 51, 208, "Input"],
Cell[350137, 8287, 214, 5, 27, "Input"],
Cell[350354, 8294, 12064, 270, 823, "Input"],
Cell[362421, 8566, 16713, 360, 1123, "Input"],
Cell[379137, 8928, 2171, 45, 133, "Input"],
Cell[381311, 8975, 546, 14, 43, "Input"],
Cell[381860, 8991, 4868, 112, 448, "Input"],
Cell[386731, 9105, 609, 13, 58, "Input"],
Cell[387343, 9120, 540, 9, 27, "Input"],
Cell[387886, 9131, 538, 9, 27, "Input"],
Cell[388427, 9142, 731, 14, 58, "Input"],
Cell[389161, 9158, 594, 10, 27, "Input"],
Cell[389758, 9170, 595, 10, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[390390, 9185, 111, 1, 26, "Subsection"],
Cell[390504, 9188, 373, 6, 43, "Input"],
Cell[390880, 9196, 9427, 202, 523, "Input"],
Cell[400310, 9400, 615, 12, 58, "Input"],
Cell[400928, 9414, 1469, 38, 133, "Input"],
Cell[402400, 9454, 187, 4, 27, "Input"],
Cell[402590, 9460, 214, 4, 27, "Input"],
Cell[402807, 9466, 166, 3, 27, "Input"],
Cell[402976, 9471, 166, 3, 27, "Input"],
Cell[403145, 9476, 203, 5, 27, "Input"],
Cell[403351, 9483, 226, 5, 27, "Input"],
Cell[403580, 9490, 188, 4, 27, "Input"],
Cell[403771, 9496, 188, 4, 27, "Input"],
Cell[403962, 9502, 254, 5, 27, "Input"],
Cell[404219, 9509, 2148, 50, 178, "Input"],
Cell[406370, 9561, 2124, 48, 178, "Input"],
Cell[408497, 9611, 1028, 24, 88, "Input"],
Cell[409528, 9637, 1488, 36, 103, "Input"],
Cell[411019, 9675, 2406, 60, 193, "Input"],
Cell[413428, 9737, 602, 11, 43, "Input"],
Cell[414033, 9750, 584, 11, 43, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[414666, 9767, 141, 2, 67, "Section"],
Cell[CellGroupData[{
Cell[414832, 9773, 143, 2, 34, "Subsection"],
Cell[414978, 9777, 469, 10, 58, "Input"],
Cell[415450, 9789, 347, 9, 27, "Input"],
Cell[415800, 9800, 337, 9, 27, "Input"],
Cell[416140, 9811, 305, 9, 27, "Input"],
Cell[416448, 9822, 653, 18, 43, "Input"],
Cell[417104, 9842, 791, 20, 88, "Input"],
Cell[417898, 9864, 206, 4, 27, "Input"],
Cell[418107, 9870, 475, 13, 43, "Input"],
Cell[418585, 9885, 234, 5, 27, "Input"],
Cell[418822, 9892, 279, 6, 27, "Input"],
Cell[419104, 9900, 371, 8, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[419512, 9913, 89, 1, 26, "Subsection"],
Cell[CellGroupData[{
Cell[419626, 9918, 148, 2, 24, "Subsubsection"],
Cell[419777, 9922, 206, 4, 27, "Input"],
Cell[419986, 9928, 869, 15, 27, "Input"],
Cell[420858, 9945, 286, 6, 27, "Input"],
Cell[421147, 9953, 402, 7, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[421586, 9965, 204, 3, 18, "Subsubsection"],
Cell[421793, 9970, 170, 3, 27, "Input"],
Cell[421966, 9975, 183, 3, 27, "Input"],
Cell[422152, 9980, 917, 21, 88, "Input"],
Cell[423072, 10003, 438, 7, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[423547, 10015, 113, 1, 18, "Subsubsection"],
Cell[423663, 10018, 585, 11, 43, "Input"],
Cell[424251, 10031, 273, 5, 27, "Input"],
Cell[424527, 10038, 286, 5, 27, "Input"],
Cell[424816, 10045, 1573, 38, 118, "Input"],
Cell[426392, 10085, 854, 13, 27, "Input"],
Cell[427249, 10100, 370, 11, 43, "Input"],
Cell[427622, 10113, 3100, 77, 208, "Input"],
Cell[430725, 10192, 250, 4, 27, "Input"],
Cell[430978, 10198, 264, 5, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[431291, 10209, 92, 1, 26, "Subsection"],
Cell[CellGroupData[{
Cell[431408, 10214, 95, 1, 24, "Subsubsection"],
Cell[431506, 10217, 206, 4, 27, "Input"],
Cell[431715, 10223, 896, 17, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[432648, 10245, 106, 1, 18, "Subsubsection"],
Cell[432757, 10248, 239, 6, 27, "Input"],
Cell[432999, 10256, 172, 3, 27, "Input"],
Cell[433174, 10261, 210, 4, 27, "Input"],
Cell[433387, 10267, 1902, 37, 88, "Input"],
Cell[435292, 10306, 737, 15, 27, "Input"],
Cell[436032, 10323, 938, 14, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[437007, 10342, 112, 1, 18, "Subsubsection"],
Cell[437122, 10345, 1918, 44, 118, "Input"],
Cell[439043, 10391, 216, 4, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[439308, 10401, 91, 1, 26, "Subsection"],
Cell[CellGroupData[{
Cell[439424, 10406, 95, 1, 24, "Subsubsection"],
Cell[439522, 10409, 206, 4, 27, "Input"],
Cell[439731, 10415, 944, 17, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[440712, 10437, 106, 1, 18, "Subsubsection"],
Cell[440821, 10440, 172, 3, 27, "Input"],
Cell[440996, 10445, 181, 3, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[441226, 10454, 151, 2, 26, "Subsection"],
Cell[CellGroupData[{
Cell[441402, 10460, 96, 1, 24, "Subsubsection"],
Cell[441501, 10463, 311, 6, 43, "Input"],
Cell[441815, 10471, 212, 6, 27, "Input"],
Cell[442030, 10479, 304, 8, 27, "Input"],
Cell[442337, 10489, 2360, 58, 103, "Input"],
Cell[444700, 10549, 577, 15, 43, "Input"],
Cell[445280, 10566, 250, 5, 27, "Input"],
Cell[445533, 10573, 577, 15, 43, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[446147, 10593, 94, 1, 18, "Subsubsection"],
Cell[446244, 10596, 206, 4, 27, "Input"],
Cell[446453, 10602, 602, 11, 27, "Input"],
Cell[447058, 10615, 206, 4, 27, "Input"],
Cell[447267, 10621, 881, 18, 27, "Input"],
Cell[448151, 10641, 389, 6, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[448577, 10652, 102, 1, 18, "Subsubsection"],
Cell[448682, 10655, 225, 4, 27, "Input"],
Cell[448910, 10661, 309, 5, 27, "Input"],
Cell[449222, 10668, 1071, 23, 73, "Input"],
Cell[450296, 10693, 472, 7, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[450805, 10705, 115, 1, 18, "Subsubsection"],
Cell[450923, 10708, 573, 11, 43, "Input"],
Cell[451499, 10721, 355, 6, 27, "Input"],
Cell[451857, 10729, 312, 5, 27, "Input"],
Cell[452172, 10736, 1651, 41, 103, "Input"],
Cell[453826, 10779, 1003, 23, 88, "Input"],
Cell[454832, 10804, 322, 5, 27, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[455203, 10815, 150, 2, 26, "Subsection"],
Cell[CellGroupData[{
Cell[455378, 10821, 97, 1, 24, "Subsubsection"],
Cell[455478, 10824, 206, 4, 27, "Input"],
Cell[455687, 10830, 550, 10, 27, "Input"],
Cell[456240, 10842, 206, 4, 27, "Input"],
Cell[456449, 10848, 748, 16, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[457234, 10869, 102, 1, 18, "Subsubsection"],
Cell[457339, 10872, 298, 7, 27, "Input"],
Cell[457640, 10881, 249, 4, 27, "Input"],
Cell[457892, 10887, 556, 8, 27, "Input"],
Cell[458451, 10897, 772, 19, 73, "Input"],
Cell[459226, 10918, 297, 5, 27, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[459560, 10928, 114, 1, 18, "Subsubsection"],
Cell[459677, 10931, 2060, 47, 118, "Input"],
Cell[461740, 10980, 429, 9, 27, "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
}
]
*)

(* End of internal cache information *)
